------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
--                                                                                                                    --
--                                                                                                                    --
--                                                                                                                    --  
--                                                                                                               --
------------------------------------------------------------------------------------------------------------------------

module Opdracht2 where
import List
import ParseLib

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
data Prop
  = En [Prop]     --                
  | Of [Prop]     --                
  | Niet Prop
  | Prop :-> Prop --                  
  | Var String    --          
  | Bool Bool     --           

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
type Bedeling = [String]

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                
evalueer :: Prop -> Bedeling -> Bool
evalueer (En props)        lijst     = all (\prop -> evalueer prop lijst) props   --                                                                                                                                                            
evalueer (Of props)        lijst     = any (\prop -> evalueer prop lijst) props   --                                                                                                      
evalueer (Niet prop)       lijst     = not (evalueer prop lijst)
evalueer (prop1 :-> prop2) lijst
        | evalueer prop1 lijst
          &&
          not (evalueer prop2 lijst) = False                                      --   -               
        | otherwise                  = True
evalueer (Var s)           lijst     = elemBy eqString s lijst
evalueer (Bool b)          _         = b

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                                                                   
vervulbaar :: Prop -> [Bedeling]
vervulbaar p = filter (evalueer p) (subs (geefVariabelen p))

--                                                                                       
subs :: [a] -> [[a]]
subs []     = [[]]
subs (x:xs) = map (x:) subsxs ++ subsxs
        where subsxs = subs xs

--                                                          
geefVariabelen :: Prop -> [String]
geefVariabelen (En props)        = concatMap geefVariabelen props
geefVariabelen (Of props)        = concatMap geefVariabelen props
geefVariabelen (Niet prop)       = geefVariabelen (prop)
geefVariabelen (prop1 :-> prop2) = geefVariabelen (prop1)++geefVariabelen (prop2)
geefVariabelen (Var s)           = [s]
geefVariabelen (Bool _)          = []

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                                                 
tautologie :: Prop -> Bool
tautologie p = all (evalueer p) (subs (geefVariabelen p))

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                                                 
contradictie :: Prop -> Bool
contradictie p = all (not.evalueer p) (subs (geefVariabelen p))

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                 
toon :: Prop -> String
toon (En [])          = "    "
toon (En props)       = " " ++ foldr1 (\x y -> x++"     "++y) (map toon props) ++ " "
toon (Of [])          = "     "
toon (Of props)       = " " ++ foldr1 (\x y -> x++"     "++y) (map toon props) ++ " "
toon (Niet prop)      = "  "++ toon prop ++ " "
toon (prop1 :-> prop2)= " " ++toon prop1 ++ " "++ "    "++ " " ++toon prop2 ++ " "
toon (Var s)          = s
toon (Bool b)         = if b then "    "
                             else "     "

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                                                
equivalent :: Prop -> Prop -> Bool
equivalent p q = tautologie (En [p :-> q, q :-> p ])        --                              -                               -       -                      

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                   
deMorgan :: Prop -> Prop
deMorgan (Niet (En props))    = Of (map (\x -> deMorgan (Niet x)) props)      --                                     
deMorgan (Niet (Of props))    = En (map (\x -> deMorgan (Niet x)) props)      --                                     
deMorgan (En props)           = En (map deMorgan props)
deMorgan (Of props)           = Of (map deMorgan props)
deMorgan (Niet prop)          = Niet (deMorgan prop)
deMorgan (prop1 :-> prop2)    = (deMorgan prop1) :-> (deMorgan prop2)
deMorgan (Var s)              = Var s
deMorgan (Bool b)             = Bool b

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                 
--           -                       -    -   
howard:: Prop -> Prop
howard ((En [prop1, prop2]) :-> prop3) = (howard prop1) :-> (howard prop2) :-> (howard prop3) 
howard (En props)                      = En (map howard props)
howard (Of props)                      = Of (map howard props)
howard (Niet prop)                     = Niet (howard prop)
howard (prop1 :-> prop2)               = (howard  prop1) :-> (howard  prop2)
howard (Var s)                         = Var s
howard (Bool b)                        = Bool b

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                        
--                            
dubNeg :: Prop -> Prop
dubNeg (Niet (Niet prop))   = (dubNeg prop)
dubNeg (En props)           = En (map dubNeg props)
dubNeg (Of props)           = Of (map dubNeg props)
dubNeg (Niet prop)          = Niet (dubNeg prop)
dubNeg (prop1 :-> prop2)    = (dubNeg prop1) :-> (dubNeg prop2)
dubNeg (Var s)              = Var s
dubNeg (Bool b)             = Bool b

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                                                     
--                                                        
--                                                         
distr :: Prop -> Prop
distr (En [p, (Of [q, r])])               = Of [(En [(distr p), (distr q)]), (En [(distr p), (distr r)])]
distr (Of [p, (En [q, r])])               = En [(Of [(distr p), (distr q)]), (Of [(distr p), (distr r)])]
distr (En props)                          = En (map distr props)
distr (Of props)                          = Of (map distr props)
distr (Niet prop)                         = Niet (distr prop)
distr (prop1 :-> prop2)                   = (distr prop1) :-> (distr prop2)
distr (Var s)                             = Var s
distr (Bool b)                            = Bool b

------------------------------------------------------------------------------------------------------------------------
--                                                                                                                    --
------------------------------------------------------------------------------------------------------------------------
--                                                   
ontleed :: String -> Prop
ontleed t = fst.head (filter (null.snd) results)
    where results     = term (filter (\c-> not (c `eqChar` ' ' || c `eqChar` '\t')) t)

--                                                              
term :: Parser Char Prop
term =   factor <|>
         (\f _ g -> f :-> g)  <$> factor <*> token eqChar "  "   <*> term                           <|>     --                      
         (\s ts -> En (s:ts)) <$> factor <*> greedy ((\_ t -> t) <$> token eqChar "   " <*>  term ) <|>     --                       
         (\s ts -> Of (s:ts)) <$> factor <*> greedy ((\_ t -> t) <$> token eqChar "   " <*>  term )         --                      

--                                           
factor :: Parser Char Prop
factor = (\_ -> Bool True )  <$> token eqChar "    "                                                                   <|>       --                 
         (\_ -> Bool False)  <$> token eqChar "     "                                                                  <|>       --                  
         (\c s -> Var (c:s)) <$> satisfy (\c -> ord c >= ord 'a' && ord c <= ord 'z') <*> greedy (satisfy isAlphaNum)  <|>       --                                                                
         (\_ g -> Niet g)    <$> token eqChar " " <*> term                                                             <|>       --                   
         (\_ t _  -> t)      <$> token eqChar " " <*> term <*> token eqChar " "                                                  --                  


