data Prop
   = En [Prop]     --                
   | Of [Prop]     --                
   | Niet Prop
   | Prop :-> Prop --                  
   | Var String    --          
   | Bool Bool     --           

type Bedeling = [String]

--        
evalueer :: Prop -> Bedeling -> Bool
evalueer (En props)    bedeling = all (\prop -> evalueer prop bedeling) props
evalueer (Of props)    bedeling = any (\prop -> evalueer prop bedeling) props
evalueer (Niet prop)   bedeling = not (evalueer prop bedeling)
evalueer (Var string)  bedeling = elemBy eqString string bedeling
evalueer (Bool bool)   _        = bool
evalueer (propA :-> propB) bedeling = not (evalueer propA bedeling) || (evalueer propB bedeling)

--          
vervulbaar :: Prop -> [Bedeling]
vervulbaar prop = filter (evalueer prop) (subs (zoekVar prop))

--    
toon :: Prop -> String
toon (En props)        = (drop 4 (concatMap ("     " ++) (map (\prop -> toon' prop) props)))
toon (Of props)        = (drop 4 (concatMap ("     " ++) (map (\prop -> toon prop) props)))
toon (Niet prop)       = " " ++ (toon' prop)
toon (propA :-> propB) = (toon propA) ++ "    " ++ (toon propB)
toon (Var string)      = string
toon (Bool bool)       = if bool then "    " else "     "

toon' :: Prop -> String
toon' (Of props)        = " " ++ (drop 4 (concatMap ("     " ++) (map (\prop -> toon prop) props))) ++ " "
toon' (En props)        = " " ++ (drop 4 (concatMap ("     " ++) (map (\prop -> toon' prop) props))) ++ " "
toon' (propA :-> propB) = " " ++ (toon propA) ++ "    " ++ (toon propB) ++ " "
toon' props             = toon props

--          
tautologie :: Prop -> Bool
tautologie props = all (evalueer props) (subs (zoekVar props))

--            
contradictie :: Prop -> Bool
contradictie props = not (any (evalueer props) (subs (zoekVar props)))

--            
equivalent :: Prop -> Prop -> Bool
equivalent propsA propsB = eqList eqBool (map (evalueer propsA) bedeling) (map (evalueer propsB) bedeling)
                         where bedeling = subs((zoekVar propsA) ++ (zoekVar propsB))

--        
deMorgan :: Prop -> Prop
deMorgan (propsA :-> propsB)       = (deMorgan propsA) :-> (deMorgan propsB)
deMorgan (Niet(En props))          = Of (map (\prop -> deMorgan' prop) props)
deMorgan (Niet(Of props))          = En (map (\prop -> deMorgan' prop) props)
deMorgan props                     = props

--                       
deMorgan' :: Prop-> Prop
deMorgan' (Niet(En props))         = En props
deMorgan' (Niet(Of props))         = Of props
deMorgan' (En props)               = deMorgan (Niet (En props))
deMorgan' (Of props)               = deMorgan (Niet (Of props))
deMorgan' (Var string)             = Niet(Var string)
deMorgan' (Niet(Var string))       = Var string
deMorgan' props                    = props

--                                    
delDubbelNiet :: Prop -> Prop
delDubbelNiet (Niet (Niet (props)))          = props
delDubbelNiet (En props)                     = En (map (\prop -> delDubbelNiet prop) props)
delDubbelNiet (Of props)                     = Of (map (\prop -> delDubbelNiet prop) props)
delDubbelNiet props                          = props

--                                
subs :: [a] -> [[a]]
subs [] = [[]]
subs (x:xs) = map (x:) subsxs ++ subsxs
            where subsxs = subs xs

--                                    
zoekVar :: Prop -> [String]
zoekVar (En props)        = concatMap (\prop -> zoekVar prop) props
zoekVar (Of props)        = concatMap (\prop -> zoekVar prop) props
zoekVar (Niet prop)       = zoekVar prop
zoekVar (Var string)      = [string]
zoekVar (Bool _)          = []
zoekVar (propA :-> propB) = zoekVar propA ++ zoekVar propB



--      
--                  -      
--                       

ontleed :: String -> Prop
ontleed string = split (words string)

split :: [String] -> Prop
split (x:"   ":xss) = (En [split [x], split xss] )
split (x:"   ":xss) = (Of [(split [x]), split xss])
split (x:"   ":xss) = ((split [x]) :-> (split xss))
--        
split [x]           = Var x

--                     
        {-                  "   "                                     
                            "   "                                     
                                                    
                            " "                                      
                                                    -}

--            -       
--                                     



--                                                      
--            -             
--                        
--                        
--                          
--                         



