module AnalysisToolGUI where
  
import Thesis.Logging.Settings
import Auxilary (maybeToString)

import Graphics.UI.WX
import Graphics.UI.WXCore hiding (Event)
import WXGrid

-- should be via Main or different
import Thesis.Logging.LogfileParser
import Thesis.Logging.Logging
import Thesis.Logging.LoggingPP
import Statistics.Analysis

{-

Unfinished, first implement cmd line tool approach

TODO:
- Add next: split window for gLogsStats  :D
- catch or forcome Gdk-CRITICAL error when opening filedialog
  (http://caml.inria.fr/pub/ml-archives/ocaml-beginners/2003/06/d3f2a3dd3399b1ac11351439adc544e4.en.htm)
- use MVC model??

-}


-- Settings, to Settings.hs
applicationName :: String
applicationName = "Helium Big Brother Logging Analyser"

windowHeight = 400
windowWidth = 500

-- main window size
windowSize = sz windowHeight windowWidth

-- Specification of file types in open dialog.
openFiles
   = [("Any file", ["*"])
     ,("logfiles (*.log)", ["*.log"])
     ,("Text files (*.txt)", ["*.txt","*.text"])
     ]

-- to Statistics.Analysis.hs ?
logcolnames = ["Identity", "Phase", "Compiler Version", "Date/Time", "Location"]

logstatcolnames = ["1", "2","3","4","5"]
{-
type analysisOnLogs a :: [Logging] -> (String, a)

totaalAantal logs = ("Aantal Loggings", frequency logs)
totaalAantalIds logs = ("Aantal identities", frequency (nubBy logs )) -}



----------------------------------------------------------------------
-- Graphical Representation
----------------------------------------------------------------------

analyserUI :: IO ()
analyserUI
  = do f <- frame [text := applicationName, picture := "./bitmaps/eye.ico", visible := False, fullRepaintOnResize := True ]
       p <- panel f []  -- keyboard navigation
       s <- splitterWindow p []
       
       -- add a scrollable window widget in the frame
       -- sw     <- scrolledWindow f [scrollRate := sz 10 10, bgcolor := white, fullRepaintOnResize := False]

       -- use a mutable variable to hold settings / state?
--       vsetting <- variable [value := Nothing]
       -- a mutable variable for holding the filename of logfile, currently analyzed
       vlogfile <- varCreate Nothing -- special attribute: variable [value := Nothing]

       -- a mutable variable for holding the loggings from a logfile
       vloggings <- varCreate []

       -- grid for loggings from logfile
       gLoggings <- setUpGrid s [] logcolnames 
       pLogsStats <- panel s []

       windowOnKeyDown gLoggings (onGridKeyDown gLoggings)
       set gLoggings [on gridEvent := onGrid]

       -- create file menu
       fileM  <- menuPane      [text := "&File"]
       open   <- menuItem fileM [text := "&Open\tCtrl+O",  help := "Open a logfile"]
       save   <- menuItem fileM [text := "Save selection &as...",  help := "Save a selection to a new logfile"]
       closeMI <- menuItem fileM [text := "&Close\tCtrl+C", help := "Close logfile", enabled := False]
       menuLine fileM
       quit   <- menuQuit fileM [help := "Quit the demo"]

       -- create analysis menu
       editM  <- menuPane      [text := "&Edit"]
       prefs   <- menuItem editM [text := "&Preferences",  help := "Change preferences"]
       
       -- create analysis menu
       analysisM  <- menuPane      [text := "&Analysis"]
       run   <- menuItem analysisM [text := "&Run\tCtrl+R",  help := "Run an analysis"]
       halt <- menuItem analysisM [text := "&Halt\tCtrl+H", help := "Halt an analysis", enabled := False]

       -- create Help menu
       hlpM    <- menuHelp      []
       aboutHel  <- menuItem hlpM [text := "&About Helium", help := "About Helium"]
       aboutBBLA  <- menuItem hlpM [text := ("&About " ++ applicationName), help := ("About " ++ applicationName)]

       -- create Toolbar related to frequent used menus
       tbar   <- toolBar f []
       toolMenu tbar open  "Open" "images/fileopen16.png" []
       toolMenu tbar save "Save" "images/disk.ico" []
       toolMenu tbar closeMI "Close" "images/empty.png" []
       toolMenu tbar run "Run" "images/wxwin16.png" []
       toolMenu tbar halt "Halt" "images/empty2.png" []
       toolMenu tbar quit "Quit" "images/empty.png" []
       toolMenu tbar aboutBBLA "About" "images/wxwin16.png" []

       -- create statusbar field
       status <- statusField   [text := ("Welcome to " ++ applicationName)]
{-
       -- set layout of splitwindow
       set s [layout := hsplit s1 5 80 (widget gLoggings)  
                                       (widget gLogsStat)
             ]
  -}     
       -- set layout of pane
       set p [layout := margin 10 ( fill $ column 1 [hfill $ hrule 1  -- add divider between toolbar and scrolledWindow
                                                     , hsplit s 5 400 {- (round (0.8 * windowHeight)) -} (fill $ {- minsize (sz 300 100) $ -} widget gLoggings)  --,fill $ minsize (sz 300 100) $ (widget gLoggings)  -- works for single widget view
                                                                      (fill $ widget pLogsStats)
                                                       
                                                       --,floatBottomRight $ row 5 [widget runAnaBt, widget resetBt, widget quitBt]
                                                       ]
                                            )
             ]
       -- set the statusbar, menubar, layout, and add menu item event handlers
       -- note: set the layout before the menubar!
       set f [layout := fill $ widget p
                        , outerSize        := windowSize    -- niceness, standard start up size
                        -- Menu handling
                        ,on (menu open)   := onOpen f gLoggings vlogfile vloggings closeMI status 
                        ,on (menu closeMI) := onClose  gLoggings vlogfile vloggings closeMI status 
                        ,on (menu quit)   := close f
            
                        ,on (menu aboutBBLA) := infoDialog f ("About " ++ applicationName)  ("This is " ++ applicationName)
                        ,on (menu aboutHel)  := heliumInfoDialog
              , statusbar        := [status]
              , menubar          := [fileM,hlpM]
             ]
       set f [visible := True]  -- reduce flicker at startup
       focusOn gLoggings
       return ()
   	`catch` \err -> do justAWindow "Error" "asfd" -- (show err) --repaint sw   -- should be handled 
   where
    onOpen :: Frame a -> Grid b -> Var (Maybe String) -> Var ([Logging]) -> MenuItem c -> StatusField -> IO ()
    onOpen f sw vlogfile vloggings mclose status
      = do mlogfilename <- fileOpenDialog f False {- change current directory -} True {- allow readonly files -} "Open logfile" openFiles datasetdirFP0304 ""  -- throws: Gdk-CRITICAL **: file gdkwindow.c: line 1390 (gdk_window_get_size): assertion `window != NULL' failed.
           case mlogfilename of
              Nothing    -> return ()
              Just fname -> openLogfile sw vlogfile vloggings mclose status fname
           `catch` \err -> print "foutje!" --justAWindow "Error" (show err) --repaint sw   -- should be handled 

    onClose sw vlogfile vloggings mclose status
      = do set mclose [enabled := False]
           set sw     [virtualSize := sz 0 0]
           clearGridContent sw
           mfilename <- varGet vlogfile -- (get vlogfile value)
           set status [text := (maybeToString mfilename ++ " is closed")]
           set vlogfile [ value := Nothing ]
           set vloggings []
           repaint sw

    openLogfile sw vlogfile vloggings mclose status filename
      = do varSet vlogfile (Just filename)
           set mclose [enabled := True]
           set status [text := (filename ++ " is opened")]
           parseResult <- parseLogfile (filename) -- IO Action           -- can fail with error is Left a
           case (parseResult) of 
              Left err       -> do justAWindow "Error while parsing..." (show err)
                              
              Right loggings -> do set vloggings [value := loggings] 
                                   appendGridContent sw (map ppLoggingSplit loggings)
                                   repaint sw
                                   -- add extra logs stat
                                   

-- grid funs
    onGridKeyDown sw (EventKey key mods pt)
      = case key of
          KeyReturn ->          
            do logMessage "keyEnter"
               gridMoveNext sw
          _ -> propagateEvent

    onGrid ev
      = case ev of
          GridCellChange row col veto
            -> logMessage ("cell changed: " ++ show (row,col))
          _ -> propagateEvent

    heliumInfoDialog :: IO ()
    heliumInfoDialog = 
       do 
          f <- frame [text := "About Helium"]
          t1 <- staticText f [text:="Helium"]
          t2 <- textCtrl f [ text:= "Helium is a user-friendly compiler especially designed for teaching and learning the functional programming language Haskell. The compiler can be equipped with a feature called Big Brother for logging all compiled programs.",
                                        wrap := WrapWord, enabled := False, alignment := AlignCentre] 
          set f [layout := margin 10 $ fill $ column 1 [halignCenter $ widget t1, fill $ widget t2], clientSize := sz 300 200]
          return ()

justAWindow :: String -> String -> IO ()
justAWindow title content = do
   f <- frame [text := title]
   t1 <- staticText f [text:= content]
   set f [layout := margin 10 $ widget t1, clientSize := sz 300 200]
   return ()
   
{-
class Show a => gShowable a where 
   gshow :: a -> Control a
  
   

--paneWithAnalyses :: gShowable a => Window -> Panel () 
--panelWithAnalyses gShowable a => parent analyses [(name, ([logging] -> a))] = 
panelWithAnalyses panel analyses w1 w2 =
   do  -- panelP <- panel parent []
      set panel [layout := row 5 [widget w1, widget w2]]
      return panel
   -}


{-

-- Examples: 

    onPaint vlogfile dc viewArea
      = do mlogfile <- get vlogfile value
           case mlogfile of
             Nothing -> return () 
             Just bm -> --drawBitmap dc bm pointZero False []

    varToString var
      = do varValue <- get var value
           case varValue of
             Nothing -> ""
             Just bm -> --drawBitmap dc bm pointZero False []

{-
    set f [layout := margin 10 (column 5 [floatCentre (label "Select logfile to analyse")
                                     ,floatCentre (widget quit)
                                     ] ) ]

--         layout grid
          set f [layout := column 5 [fill (dynamic (widget gLoggings))
                                   ,hfill $ minsize (sz 20 80) $ widget textlog]
               ] 

       -- set the statusbar, menubar, layout, and add menu item event handlers
       -- note: set the layout before the menubar!
--       set p [layout           := minsize (sz 400 300) $ 
  --                                margin 10 ( column 1 [hfill $ hrule 1  -- add divider between toolbar and scrolledWindow
  --                                                     ,fill (widget gLoggings)
  --                                                     ,floatBottomRight $ row 5 [widget runAnaBt, widget resetBt, widget quitBt]
  --                                                     ]
  --                                          )
  --           ,statusbar        := [status]
  --           ,menubar          := [file,hlp]
--             ,clientSize        := sz 400 300    -- niceness

             -- Menu handling
             ,on (menu open)   := onOpen f gLoggings vlogfile mclose status 
             ,on (menu mclose) := onClose  gLoggings vlogfile mclose status 
             ,on (menu quit)   := close f

             ,on (menu aboutBBLA)  := infoDialog f ("About " ++ applicationName)  ("This is " ++ applicationName)
             ,on (menu aboutHel)  := heliumInfoDialog
             ] -}

-}

