module Auxilary ( module Auxilary
                , module Distribution.Compat.FilePath
                ) where

import System.Time hiding (Day)
import System.Locale
-- import Data.Time -- new time lib
import Distribution.Compat.FilePath  -- For platform independent filepath handling, from Cabal package

import Text.Regex
--import RRegex  -- new lib
import System.IO.Unsafe (unsafePerformIO)
import Data.Char
import Data.Maybe
import Data.PackedString
import Data.List --(elemIndex, sort, group, groupBy, sortBy, isPrefixOf, partition)
import Data.Typeable

-- turns on explicit error messages, in different parts of the Statistics package
-- to be moved to better place
debug = False
debug_dateParsing = False
debug_weekNum = False
debug_key = False   -- See Statistics.DescriptiveAnalysis.KeyHistoryListImpl

type CalendarDate = String

-- week information (year, week of year)
data Year_Week = Year_Week { year :: Int, week :: Int}
   deriving ({-Show, -} Read, Eq, Ord, Typeable)

instance Show Year_Week where
   show (Year_Week year week) = unwords $ show year:show week:[]

-------------------------------------------------------
-- String utils, from heliumNT/scr/utils/Utils.hs (3 funs)
-------------------------------------------------------
ltrimOnlySpace :: String -> String
ltrimOnlySpace (' ':xs) = ltrim xs
ltrimOnlySpace xs       = xs

rtrimOnlySpace :: String -> String
rtrimOnlySpace = reverse . ltrimOnlySpace . reverse

trimOnlySpace :: String -> String
trimOnlySpace = ltrimOnlySpace . rtrimOnlySpace

ltrim :: String -> String
ltrim = dropWhile isSpace

rtrim :: String -> String
rtrim = reverse . ltrim . reverse

trim :: String -> String
trim = ltrim . rtrim

trimInside :: String -> String
trimInside = filter (not . isSpace)

commaList :: [String] -> String
commaList = sepList ", "

-- for vertical pprinting in result file
verticalList :: Show a => [a] -> String
verticalList [] = ""
verticalList [x] = show x
verticalList (x:xs) = show x ++ "\n " ++ verticalList xs

sepList :: String -> [String] -> String
sepList sep [] = ""
sepList sep [x] = x
sepList sep (x:xs) = x ++ sep ++ (sepList sep xs)

sepNonEmptyList :: String -> [String] -> String
sepNonEmptyList sep = sepList sep . (filter (not.null))

-- checking whether seq1 as a whole occurs in seq2
isSubList :: (Ord a) => [a] -> [a] -> Bool
isSubList [] (y:ys) = error "no substring provided or to small"
isSubList _ []      = False
isSubList (x:xs) (y:ys)
  | x == y = (frontSubStr (x:xs) (y:ys) || isSubStr (x:xs) ys)
  | otherwise = isSubStr (x:xs) ys
  where frontSubStr str1 str2 = if (length str2 >= length str1)
                                   then str1 == take (length str1) str2
                                   else False
{-
-- returns index of first occurence of substring
indexSubList :: (Ord a) => [a] -> [a]-> Maybe Int
indexSubList sub string =
   if (isSubList sub string)
      then
         indexSubList' sub string
      else
         Nothing
         
indexSubList' [] (y:ys) = error "no substring provided or to small"
indexSubList' _ []      = Nothing
indexSubList' (x:xs) (y:ys)
  | x == y = (frontSubStr (x:xs) (y:ys) || isSubStr (x:xs) ys)
  | otherwise = isSubStr (x:xs) ys
  where frontSubStr str1 str2 = if (length str2 >= length str1)
                                   then str1 == take (length str1) str2
                                   else False
-}

isSubStr :: (Ord a) => [a] -> [a]-> Bool
isSubStr = isSubList

replaceStrInStr :: String -> String -> String -> String
replaceStrInStr oristr str repl = subRegex (mkRegex oristr) str repl

dropPrefix :: String -> String -> String  -- [a] -> [a] -> [a]
dropPrefix prefix str =
   let ln = length prefix
   in
      case (isPrefixOf prefix str) of
         True  -> drop ln str
         False -> str

-- Split string on char
splitWith :: (Char -> Bool) -> String -> [String]
splitWith fun = (map unpackPS) . (splitWithPS fun) . packString

--takeTill :: ([a] -> Bool) -> [a] -> [a]
--takeTill pred list = takeWhile (not . pred) (inits list)

-- Split string on smaller sub strings
partitionAtString :: String -> String -> (String, String)
partitionAtString splitStr str =
   --(takeWhile (not . isPrefixOf splitStr) (subStrings str), rest))
   let
      rest = dropWhile (not . isPrefixOf splitStr) (subStrings str)
   in
      if (length rest > 0)
         then
            (take ((length str) - (length $ head rest)) str, head rest)
         else
            (str,"")
   
-- Generate all substrings
subStrings :: String -> [String]
subStrings input = tails input -- map (flip drop $ input) [0..length input]

-- Split string on smaller sub strings
splitAtString :: String -> String -> [String]
splitAtString splitStr str = filter (isPrefixOf splitStr) (subStrings str)
   where
      subStrings :: String -> [String]
      subStrings input = map (flip drop $ input) [0..length input]

--([^W]|.[^a]|..[^r]|...[^n]|....[^i]|.....[^n]|......[^g])
generateNonMatchingWord :: String -> String
generateNonMatchingWord = embrace . (sepList "|") . (map makeNon) . (zip [0..])
   where
      makeNon :: (Int, Char) -> String
      makeNon (i,c) = (take i $ repeat '.') ++ "[^"++ (c:"") ++"]"
      embrace wrd = "(" ++ wrd ++ ")"

-- Generic elem
elemBy :: (a -> Bool) -> [a] -> Bool
elemBy = any

enlist :: a -> [a]
enlist v = [v]

{-- Just for renaming elemIndex to a more usual name -}
indexOf :: Eq a => a -> [a] -> Maybe Int
indexOf = elemIndex

{--- Returns the index of the last occurrence of the given element in the given list -}
lastIndexOf :: Eq a => a -> [a] -> Maybe Int
lastIndexOf x xs =
    case indexOf x (reverse xs) of
        Nothing     ->  Nothing
        Just idx    ->  Just (length xs - idx - 1)
        
---- Regex Analysis Helper functions, use (val) to create subexpressions to be returned

-- Mapping regexMatchAll over resulting "after" result, NOTE: last regex application will always return Nothing, so Nothing will be always at the end of the result list
-- not any more, added catMaybe, in multipleMatchRegexAll
-- multipleMatchRegex :: Regex -> String -> [Maybe [String]]
multipleMatchRegex :: Regex -> String -> [[String]]
multipleMatchRegex regex input = map ({-maybe Nothing (Just .-} fourth4{-)-}) (multipleMatchRegexAll regex input)

multipleMatchRegexAll :: Regex -> String -> [{-Maybe-} (String, String, String, [String])]
multipleMatchRegexAll regex = catMaybes . multipleMatchRegexAll' regex
   where
     multipleMatchRegexAll' regex input =
      case (matchRegexAll regex input) of 
         Nothing -> [Nothing]
         Just (before, matched, after, subexprs) ->
            Just (before, matched, after, subexprs) : (multipleMatchRegexAll' regex after)

-- Returns matched part of regex-match, not the subexpressions of a match!! Usefullness??
multipleMatchedRegexMatchings :: Regex -> String -> [String]
multipleMatchedRegexMatchings regex = map snd4 . multipleMatchRegexAll regex

-- Tuple functions

fst3 :: (a, b, c) -> a
fst3 (x, _, _) = x

snd3 :: (a, b, c) -> b
snd3 (_, x, _) = x

thd3 :: (a, b, c) -> c
thd3 (_, _, x) = x

fst4 :: (a, b, c, d) -> a
fst4 (x, _, _, _) = x

snd4 :: (a, b, c, d) -> b
snd4 (_, x, _, _) = x

thd4 :: (a, b, c, d) -> c
thd4 (_, _, x, _) = x

fourth4 :: (a, b, c, d) -> d
fourth4 (_, _, _, x) = x

xor :: Bool -> Bool -> Bool
xor b1 b2 = not (b1 == b2)

----
-- Date / Time functions
----

-- for converting "yyyy-mm-dd@hh_mm_ss_ppp" into a CalendarTime
stringToCalTime :: String -> CalendarTime
stringToCalTime input = 
   let yearAndRest = splitAt 4 input
       monAndRest  = splitAt 2 (dropWhile (=='-') $ snd yearAndRest)
       dayAndRest  = splitAt 2 (dropWhile (=='-') $ snd monAndRest)
       hourAndRest = splitAt 2 (dropWhile (=='@') $ snd dayAndRest)
       minAndRest  = splitAt 2 (dropWhile (=='_') $ snd hourAndRest)
       secAndRest  = splitAt 2 (dropWhile (=='_') $ snd minAndRest)
       msecAndRest = splitAt 3 (dropWhile (=='_') $ snd secAndRest)
   in {- traceVal (show "input: " ++ show monAndRest) $ -}  correctDftValues 
         CalendarTime {           ctYear    = read (fst yearAndRest)::Int
                                 ,ctMonth   = toEnum $ (read (fst monAndRest)::Int) - 1
                                 ,ctDay     = read (fst dayAndRest)::Int 
                                 ,ctHour    = read (fst hourAndRest)::Int
                                 ,ctMin     = read (fst minAndRest)::Int 
                                 ,ctSec     = read (fst secAndRest)::Int 
                                 ,ctPicosec = (1 * 10 ^ 9) * read (fst msecAndRest)::Integer
                                       -- 1 pico = 1 * 10 ^ -12 second or 1 msec = 1.000.000.000 picoseconds
                                 ,ctWDay    = Sunday  -- default value
                                 ,ctYDay    = 0       -- default value
                                 ,ctTZName  = ""      -- default value
                                 ,ctTZ      = 0       -- default value
                                 ,ctIsDST   = True    -- default value
                      }
   where correctDftValues = toUTCTime . toClockTime   -- conversion which ignore the default values, corrects the WDay, etc

-- Pp calendartime like in logfile
calTimeToString :: CalendarTime -> String
calTimeToString caltime = formatCalendarTime defaultTimeLocale ("20%y-%m-%d@%H_%M_%S_" ++ showMSec) caltime
   where
      msec = ctPicosec caltime `div` (1 * 10 ^ 9)
      showMSec = (concat $ take (3 - (length $ show msec)) $ repeat "0") ++ (show msec)  -- Text.Printf.printf "%03d\n" (5::Int)

diffCalendarTimes :: CalendarTime -> CalendarTime -> TimeDiff
diffCalendarTimes cal1 cal2 = normalizeTimeDiff $ diffClockTimes (toClockTime cal1) (toClockTime cal2)

setTimeTD :: Int -> Int -> Int -> Integer -> TimeDiff
setTimeTD hour min sec pico = normalizeTimeDiff $
   TimeDiff {
      tdYear = 0,
      tdMonth = 0,
      tdDay = 0,
      tdHour = hour,
      tdMin = min,
      tdSec = sec,
      tdPicosec = pico
   }

setDateTD :: Int -> Int -> Int -> TimeDiff
setDateTD year mon day = normalizeTimeDiff $
   TimeDiff {
      tdYear = year,
      tdMonth = mon,
      tdDay = day,
      tdHour = 0,
      tdMin = 0,
      tdSec = 0,
      tdPicosec = 0
   }

minutesOfTD  ::  Int -> TimeDiff
minutesOfTD min =    setTimeTD 0 min 0 0

secondsOfTD :: Int -> TimeDiff
secondsOfTD sec = setTimeTD sec 0 0 0

absTD :: TimeDiff -> TimeDiff
absTD timeDiff = normalizeTimeDiff $
   TimeDiff {
      tdYear    = abs $ tdYear    timeDiff, 
      tdMonth   = abs $ tdMonth   timeDiff,
      tdDay     = abs $ tdDay     timeDiff,
      tdHour    = abs $ tdHour    timeDiff,
      tdMin     = abs $ tdMin     timeDiff,
      tdSec     = abs $ tdSec     timeDiff,
      tdPicosec = abs $ tdPicosec timeDiff
   }
{-
instance Num TimeDiff where 
   abs = absTD
   t1 - t2 = undefined
-}

-- getLogClockTime = toClockTime . stringToCalTime . logDate

timediff :: ClockTime -> ClockTime -> Bool
timediff a b = a > b

timeDiffToMinRounded :: TimeDiff -> Integer
timeDiffToMinRounded timediff = round  $ realToFrac (timeDiffToSec timediff) / 60

timeDiffToMin :: TimeDiff -> Float
timeDiffToMin timediff = realToFrac (timeDiffToSec timediff) / 60

timeDiffToSec :: TimeDiff -> Integer
timeDiffToSec timediff_in =
   if (tdYear timediff > 0 || tdMonth timediff > 0 ) && debug
   then
      unsafePerformIO $ do
         putStrLn "Warning: timediff over more then a month is being calculated."
         return (toInteger calctime)
   else
      toInteger calctime
   where
      timediff = normalizeTimeDiff timediff_in
      calctime = tdSec timediff + tdMin timediff * 60 + tdHour timediff *60*60 + tdDay timediff *24*60*60 + tdMonth timediff *30*24*60*60 + tdYear timediff*12*30*24*60*60

weekOfYear :: CalendarTime -> Year_Week
weekOfYear caltime = Year_Week (ctYear caltime) ((read $ weekstr caltime) :: Int)
   where
      weekstr = formatCalendarTime defaultTimeLocale "%V" -- %U and %W doesnt work for 0203 and 0304

-- Show content of maybe or "" if Nothing
maybeToString :: Show a => Maybe a -> String
maybeToString = unwords . (map show) . maybeToList

showMaybe :: Show a => String -> Maybe a -> String
showMaybe _    (Just a) = show a
showMaybe noth (Nothing ) = noth

showMaybeData :: Show a => Maybe a -> String
showMaybeData = showMaybe "-"

----------------------------------------------------------------------
-- Trace / Debug function
----------------------------------------------------------------------
trace :: String -> a -> a
trace s x = unsafePerformIO $ do
   putStr s
   return x

-- trace with better show
traceVal :: String -> a -> a
traceVal str = trace ("   trace val: " ++ show str ++"\n")

-- Example use: trace (show input) (eval (function input))
f xs = trace ("<<" ++ show ys ++ ">>") (sum ys)
  where
    ys = map (2*) xs

----------------------------------------------------------------------
-- Instance decl. for (6-tuple), used in Statistics.hs
----------------------------------------------------------------------
-- instance (Show a,Show b,Show c,Show d,Show e,Show f) => Show (a, b ,c ,d, e, f) where
--   show (a, b, c, d, e, f) = "(" ++ show a ++ ", " ++ show b ++ ", " ++ show c ++ ", " ++ show d ++ ", " ++ show e ++ ", " ++ show f ++ ")"

----------------------------------------------------------------------
-- Strict evaluation of IO action, only allowed in cases of garanteed 
----------------------------------------------------------------------
--strictUnsafePerformIO :: (a -> IO a) -> a
strictUnsafePerformIO action =
   let v = unsafePerformIO action
   in v `seq` v

strictEval x = x `seq` x

mapTuple :: ((a->b), (c->d)) -> [(a,c)] -> [(b,d)]
mapTuple (fun1, fun2) = map (\(t1,t2) -> (fun1 t1, fun2 t2))

mapAndTuple :: (a -> b) -> [a] -> [(a,b)]
mapAndTuple fun = map (\a -> (a, fun a))

-- for easy mapping: map fmapTupleSnd sortByFstElm
fmapTuple :: (a -> c, b -> d) -> (a, b) -> (c, d)
fmapTuple (f1, f2) (v1, v2) = (f1 v1, f2 v2)

fmapTupleSnd :: (b -> d) -> (a, b) -> (a, d)
fmapTupleSnd = curry fmapTuple id

fmapTupleFst :: (a -> a1) -> (a, b) -> (a1, b)
fmapTupleFst f = curry fmapTuple f id

-- map function onto list pairwise
mapPairWise :: (a -> a -> b) -> [a] -> [Maybe b]
mapPairWise f [] = []
mapPairWise f [x] = [Nothing]
mapPairWise f (x:y:zs) = (Just $ f x y) : mapPairWise f (y:zs)

mapPairWise' :: (a -> Maybe a -> b) -> [a] -> [b]
mapPairWise' f [] = []
mapPairWise' f [x] = [f x Nothing]
mapPairWise' f (x:y:zs) = (f x $ Just y) : mapPairWise' f (y:zs)

lookup2           :: (Eq b) => b -> [(a,b)] -> Maybe a
lookup2 key []    =  Nothing
lookup2 key ((x,y):xys)
    | key == y   =  Just x
    | otherwise  =  lookup2 key xys

-- path handling
formatPath :: [String] -> String
formatPath = foldl joinFileName ""

-- concat paths
joinPaths' :: [FilePath] -> FilePath
joinPaths' = foldl joinPaths ""

-- normalizes paths
correctPath :: FilePath -> FilePath
correctPath = removePostFixedDots . joinPaths' . breakFilePath
      -- removePostFixedDots to avoid asdsdf

stringToFilePath :: String -> FilePath
stringToFilePath =
    filter  (\c ->
         (not $  (isSpace c ||
                  c=='.'  ||
                  c==','  ||
                  c==';'  ||
                  c==':'  ||
                  c=='"'  ||
                  c=='('  ||
                  c==')'
                 )
         ))

removePostFixedSlashes = reverse . dropWhile (\c -> (c=='/') || (c=='\\') ) . reverse

removePostFixedDots = reverse . dropWhile (\c -> (c=='.')) . reverse

-- The joinDirFileExt function is the opposite of splitFilePath. It joins directory, file name and file extention to form a complete file path.
joinDirFileExt :: String -> String -> String -> FilePath
joinDirFileExt dir file ext = joinFileName dir $ joinFileExt file ext

joinDirAndFileName = joinFileName

isOnlyDirectory :: FilePath -> Bool
isOnlyDirectory fp = ((snd $ splitFileName fp) == "." || (snd $ splitFileName fp) == "..")

-- combinatorial functions

-- all subsequences, but sequence of elms is of importance smth, ex. if groupings are dependant on each others results
subs [] = [ [] ]
subs (x:xs) = subsxs ++ map (x:) subsxs
   where subsxs = subs xs

-- tails [] = [ [] ]
-- tails (x:xs) = (x:xs) : tails xs

empty = null
