{-# OPTIONS_GHC -fallow-undecidable-instances #-}
module Statistics.BasicTypes where

-- for research gen
import Statistics.DescriptiveAnalysis.PrimAC (Analysis, runAnalysis, kickStartAnalysis, multiAnalyses, (<.>), (<$>), mapAnalysis)
import Statistics.DescriptiveAnalysis.KeyableAC (peekKey)
import Statistics.Presentation.MarkUpDoc


import Auxilary -- (Year_Week)

import Data.Maybe
import Data.Typeable

--
-- last change, adding datainfo, abstract data (for encapsulating measurement level into the type) but now also AD has to be in ord, real, and num, which is bit clumsy, could cause runtime exhausted pattern  (type) error.

{-

Data Classification System for Statistic Presentation

http://en.wikipedia.org/wiki/Nominal_data :

Level of measurement
From Wikipedia, the free encyclopedia.
(Redirected from Nominal data)
Jump to: navigation, search

The level of measurement of a variable in mathematics and statistics is a classification that was proposed in order to describe the nature of information contained within numbers assigned to objects and, therefore, within the variable. The levels were proposed by Stanley Smith Stevens in his 1946 article On the theory of scales of measurement. Different mathematical operations on variables are possible, depending on the level at which a variable is measured. According to the classification scheme, in statistics the kinds of descriptive statistics and significance tests that are appropriate depend on the level of measurement of the variables concerned.

Four levels of measurement were proposed by Stevens:
....
-}

-- Basic type for doing statistics

-- abstract data level kinds
data Kind = -- Level of Measurements
     Abstract   -- None really, data is not only showable, not directly plottable
   | Nominal
   | Ordinal
   | Interval
   | RatioKind
   | NomOrdinal
   | IntervalRatio
   deriving (Show, Read, Eq)

-- using type classes and option: {-# OPTIONS_GHC -fallow-undecidable-instances #-}
-- to automatically derive the required classes for each level of measurement of the data (statistically speaking)

-- member function could be used to define related operations, like mode, median related to summarization of each level of measurement
class (Eq a) => Nominal a where
--   modeN :: [a] -> Maybe a
--   modeN = undefined
instance (Eq a) => Nominal a where

class (Nominal a, Ord a) => Ordinal a where
--   medianO :: [a] -> a
--   medianO = undefined
instance (Nominal a, Ord a) => Ordinal a where

class (Ordinal a, Num a, Real a) => Interval a where
--   meanI :: (Fractional b) => [a] -> Maybe b
--   meanI = mean
instance (Ordinal a, Num a, Real a) => Interval a where

class (Interval a) => RatioValue a where
instance (Interval a) => RatioValue a where

data Range a = Range a a
             | RangeToMax a
             | RangeFromMinTo a

type CustomShow a = a -> String

data Axis = XAxis | YAxis

instance Show Axis where
   show XAxis = "x"
   show YAxis = "y"

-- DataInfo related data types

data ComplementInfo a =
     SummableBounded [a]
   | SummableSerie ([a] -> [a])
   | NeuralElm a  -- to be removed
   | NoComplement
   | NoCompletion

-- Class for adding extra information to a datatype to be able to group and present it
-- new: mainly to do proper grouping, including preventing missing values, and special presentation in columns
-- class (Show a, Ord a, Enum a, Eq a, Typeable a) => DataInfo a where
class (Show a, Eq a, Typeable a) => DataInfo a where
   showType :: a -> String
   showType = show . typeOf
   showValue :: a -> String
   showValue = show
   ordIndex :: a -> Int  -- maybe a -> [a] -> Int
   allvalues :: ComplementInfo a -- Maybe [a] -- Either [a] ([a] -> [a])  -- left for 'opsombare' en bounded ordinal values, right for unbounded allvalues based in input set
   allvalues = NoCompletion

-- old junk
   dataKind :: a -> Kind  -- really that used?
   shortname :: a -> String  -- decr, used full for axis description
   shortname = show . typeOf
   description :: a -> Maybe String -- unused
   description v = Nothing
   -- for displaying combined (tupled) results
   showContents :: a -> [String]
   showContents v = [show v]   -- MIND: showContents v = [showContents v] -- will run forever

-- DataInfo instance with a special rendering function (especially for table rendering)
instance DataInfo () where
   dataKind _ = error $ "Presentation Error: requesting information of non-used tuple \"()\" value"
   shortname _    = "NoValue (this column should not be visible)"
   showContents _ = []

instance DataInfo a => DataInfo [a] where
   dataKind = dataKind
   allvalues = NoComplement -- error "Error: no allvalues defined for 'instance DataInfo a => DataInfo [a]'"
   --   showContents = concat $ map showContents

instance DataInfo a => DataInfo (Maybe a) where
   dataKind v = if isJust v then dataKind v
                            else Abstract
{-
   shortname v = if isJust v then shortname v
                             else "No value available"
-}
   showContents v = if isJust v then showContents $ fromJust v
                                else  ["-"]
   allvalues = case (allvalues:: ComplementInfo a) of
--                  SummableBounded values -> SummableBounded $ map Just $ fromJust values
                  NoComplement -> NoComplement

instance (DataInfo a, DataInfo b) => DataInfo (a,b) where
   dataKind _ = Abstract
   showContents (x,y) = showContents x ++ showContents y
   allvalues = 
      case (allvalues, allvalues) of
         (SummableBounded allval, SummableBounded allvalB) -> undefined -- SummableBounded 
         (NeuralElm neutralval,     SummableBounded allvalB) -> undefined
         (SummableBounded allval, NeuralElm neutralval) -> SummableBounded  $ map (\val -> (val, neutralval)) allval
         (NoComplement,     NoComplement    ) -> undefined
         (_, _) -> undefined
   ordIndex (v1,v2) = ordIndex v1

instance (DataInfo a, DataInfo b, DataInfo c) => DataInfo (a,b, c) where
   dataKind _ = Abstract
   showContents (x,y,z) = showContents x ++ showContents y ++ showContents z

-- datatype for result of one or two components, like a median result for an ord list
data SingleOrBetween a = 
     Single a
   | Between a a
   deriving (Eq, Show, Read, Typeable, Ord {- tmp test -})

fromEitherToSingleOrBetween (Left x) = Single x
fromEitherToSingleOrBetween (Right (x,y)) = Between x y

instance DataInfo a => DataInfo (SingleOrBetween a) where
   dataKind = dataKind
   shortname = shortname
   showContents (Single x) = showContents x
   showContents (Between x y) = ["Between " ++ show x ++ " and " ++ show y] 
      -- or other fancy use of showContents 
      -- but!! no [showContents x] without flattening

instance DataInfo Bool where
   dataKind _ = Nominal
   allvalues = NoComplement

-- for key in displayAsHistogram
instance DataInfo (Char) where  -- and string as well, because of DataInfo a => DataInfo [a] implementation
   dataKind _ = Abstract
   allvalues = NoComplement

instance DataInfo Int where
   dataKind _ = Interval
   shortname _ = ""
   allvalues = NeuralElm 0
   ordIndex v = v

instance DataInfo Integer where
   dataKind _ = Interval
   shortname _ = "Amount"

instance DataInfo Float where
   dataKind _ = RatioKind
   shortname _ = "Amount"

-- untested
instance DataInfo Double where
   dataKind _ = RatioKind
   shortname _ = "Amount"

-- special datatypes, move to other file?
instance DataInfo (Year_Week {-(Integer, Int)-}) where
   dataKind _ = Interval
   showType _ = "week"
   shortname = showType
   ordIndex val = year val * 52 + week val

-- instances for Phase and Logging in Logging.hs

-- Research data type
type Research key a = ([a] -> key -> FilePath -> IO ())

----
-- Optional data, wrapper for maybe, but now with own show, for missing value
----

data OptData a  =  NaN
                |  Value a
             deriving (Eq, Ord, Typeable)

-- Note: Nan should come before Value, to derive correct Ord instance

fromValue (Value a) = a
fromValue NaN = error "no value in Optdata"

isValue (Value a) = True
isValue NaN = False

optdata :: b -> (a->b) -> OptData a -> b
optdata dflt fun opt = if isValue opt then fun $ fromValue opt else dflt


instance (Show a) => Show (OptData a) where
   show (Value a) = show a
   show NaN       = "-"

instance DataInfo a => DataInfo (OptData a) where
   dataKind v = if isValue v then dataKind v
                             else Abstract
{-
   shortname v = if isJust v then shortname v
                             else "No value available"
-}
   showContents v = if isValue v then showContents $ fromValue v
                                 else  ["-"]
   showType v = showType v
   allvalues = case (allvalues:: ComplementInfo a) of
--                  SummableBounded values -> SummableBounded $ map Just $ fromJust values
                  NoComplement -> NoComplement

-- like with Maybe, for relOrdValueCount in Common
instance (Real a, Ord (OptData a)) => Real (OptData a) where
   toRational NaN = traceVal ("Warning: not possible to provide Real (OptData a)") $ toRational (-9)
   toRational (Value val) = toRational val

toRealOptData :: (Real a) => (OptData a) -> a
toRealOptData NaN = 0
toRealOptData (Value val) = val

-- fun :: (Real a, Fractional b) => Maybe a -> b
-- fun = realToFrac . toRat
 --fromRational . toRational

-- bit dangerous, to be removed
instance (Eq a, Show a, Real a, Num a) => Num (OptData a) where
   m1 + m2 = Value $ (toRealOptData  m1) + (toRealOptData  m2)
   fromInteger = Value . fromIntegral
   m1 * m2 = Value $ (toRealOptData  m1) * (toRealOptData  m2)