module Statistics.Completion where

import Statistics.Ordering
import Data.List

----
-- Abstract functions for completion
----

-- completion of data on the 'a' values, 1 dim data
complementGen :: Eq a => ([a]) -> b -> [(a,b)] -> [(a,b)]
complementGen xelms neutralyelm input =
   let xinpelms = map fst input
       missingxs = xelms \\ xinpelms
   in input ++ (zip missingxs $ repeat neutralyelm)

-- completion of data on the 'a' and 'b' data
complementGen2d :: (Eq a, Eq b) => [a] -> [b] -> c -> [(a,[(b,c)])] -> [(a,[(b,c)])]
complementGen2d xelms yelms neutralzelm input = 
          map (\(x,ys) -> (x, complementGen yelms neutralzelm ys)) $ 
             complementGen xelms [] input

-- completion of data on inner dimenstion, on 'b' 
complementGen2dYs :: (Eq a, Eq b) => [b] -> c -> [(a,[(b,c)])] -> [(a,[(b,c)])]
complementGen2dYs = complementGen2d [] 

-- complementGen2d and ordering of 'a' and 'b' values
complementGen2dAndOrderXY :: (Eq a, Eq b) => (a->a->Ordering) -> (b->b->Ordering) -> [a] -> [b] -> c -> [(a,[(b,c)])] -> [(a,[(b,c)])]
complementGen2dAndOrderXY xord yord xelms yelms neutralzelm input = 
   {- filter (not . null . snd) $ -} sortUnder xord $ map (\(x, ys) -> (x, sortUnder yord ys)) $ complementGen2d xelms yelms neutralzelm input

-- complementGen2d (completion on 'b') and ordering of 'b' values
complementGen2dAndOrderY :: (Eq a, Eq b) => (b->b->Ordering) -> [a] -> [b] -> c -> [(a,[(b,c)])] -> [(a,[(b,c)])]
complementGen2dAndOrderY yord xelms yelms neutralzelm input = 
   map (\(x, ys) -> (x, sortUnder yord ys)) $ complementGen2d xelms yelms neutralzelm input

-- from Stat/DAC/KeyHistoryListImpl.hs
complementGen' :: Eq a => ([a]) -> [(a,b)] -> [(a, Maybe b)]
complementGen' vals = (complementGen vals Nothing) . map (\(x,y) -> (x, Just y))
