module Statistics.DescriptiveAnalysis.PrimGroupingAC where

import Grouping
import Data.Maybe

-- Grouping of analysis combinator results
-- Combinators or just complementory combinator functions


-- resultGrouping compareable to the general grouping algoritms

{-
dataGrouping :: (Eq a) => (KeyHistory -> KeyHistory -> Bool) -> [(KeyHistory,a)] -> [[(KeyHistory,a)]]
dataGrouping keycompare = groupAllByEq' newcmp
   where newcmp (keyhist1, v1) (keyhist2, v2) = keycompare keyhist1 keyhist2
-}

groupACResult' :: (Eq a, Eq key) => (key -> key -> Bool) -> [(key, a)] -> [[(key, a)]]
groupACResult' keycompare = groupAllByEq newcmp
   where newcmp (key1, v1) (key2, v2) = keycompare key1 key2

splitAnalysis' :: (Eq a, Eq key) => (key -> key -> Bool) -> [(key, a)] -> [[(key, a)]]
splitAnalysis' keycompare = groupAllByEq newcmp
   where newcmp (key1, v1) (key2, v2) = keycompare key1 key2
-- groupAllByEq :: forall a. (a -> a -> Bool) -> [a] -> [[a]]

{- Example:

sameStudentKH :: KeyHistory -> KeyHistory -> Bool
sameStudentKH hist1 hist2 =
   foldKeyHistory returnValueGroupStudentName hist1 == foldKeyHistory returnValueGroupStudentName hist2

-}

-- grouping of analysis result, grouping on non-occuring datasets results in an empty grouping list, slower implementation
groupACResult :: (Eq a, Eq key) => (key -> Maybe String) -> [(key, a)] -> [[(key, a)]]
groupACResult hist_char =
   groupACResult' (\hist1 hist2 -> hist_char hist1 == hist_char hist2) . filter (isJust . hist_char . fst)

{- Example (see also splitAnalysis in PrimAC.hs):

selectStudentGroups = selectGroups (foldKeyHistory returnValueGroupStudentName)

-}
