{-# OPTIONS_GHC -fallow-undecidable-instances #-}
module Statistics.Presentation.HtmlRepresentation (
           module Statistics.Presentation.HtmlRepresentation
         , module Text.Html
         ) where


-- general markup module

import Text.Html
import Data.Maybe

{- 
-- Examples from
- http://search.cpan.org/src/AUTRIJUS/Language-Haskell-0.01/hugs98-Nov2003/fptools/hslibs/text/html/doc/Html.htm
- http://www.cse.ogi.edu/~andy/html/intro.htm
see also in source doc from Text.Html, mind MARKUP is now named HTML

-- pb: ref generation
-}

{-
-- htmlPage :: [Html]
htmlPage
      = header
        << thetitle 
           << "My Haskell Home Page"
   +++ body ! [bgcolor "#aaff88"] << theBody

-- theBody :: [Html]
theBody =
      table ! [border 0] << tableContents
         +++ br
         +++ p << message
   where
      message
         = "Haskell is a general purpose, purely functional programming language."

tableContents :: HtmlTable
tableContents = (haskell `above` purely) `beside` lambda
    where
      haskell = td ! [align "center"]
                  << font ![size "7",face "Arial Black"] 
                      << "Haskell"
      purely  = td << font ! [size "6"] 
                      << "A Purely Functional Language"
      lambda  = td << image ! [src "lambda.gif"]

linkie = hotlink url htmls
   where
      url = "test.html"
      htmls = [stringToHtml "linkie"]

-- main = writeFile "example.htm" (renderHtml htmlPage)
-}

type MarkUpDoc = Html

class (HTML a) => MARKUPDOC a where
   toMarkUpDoc     :: a -> MarkUpDoc
   toMarkUpDoc     = toHtml
   toMarkUpDocFromList :: [a] -> MarkUpDoc
   toMarkUpDocFromList xs = toHtmlFromList xs -- MarkupDoc (concat [ x | (Html x) <- map toMarkUpDoc xs])

instance (HTML a) => MARKUPDOC a where

----
-- General Marup Combinators
----

document :: (MARKUPDOC a, MARKUPDOC b) => a -> b -> MarkUpDoc
document title bodyContent=
      header
        << thetitle 
          << title
   +++
      body ! [bgcolor "#aaff88"]
         << bodyContent

-- without brackets, the last parameter(s) are not shown!
section :: (MARKUPDOC a) => String -> Maybe String -> a -> Html
section title reference rest =
      (if isJust reference  -- coldlink
         then
            mark (fromJust reference)
         else
            mark (fromJust reference)
      )
      +++
         h3
            << title
      +++   
         rest

subsection :: (MARKUPDOC a) => String -> Maybe String -> a -> Html
subsection title reference rest =
      (if isJust reference  -- coldlink
         then
            mark (fromJust reference)
         else
            mark (fromJust reference)
      )
      +++
         h4
            << title
      +++   
         rest

-- coldlink or in document link/reference, skribe mark
doclink aname =
   anchor ! [name aname] << noHtml

doclink' aname contents=
   anchor ! [name aname] << contents

mark = doclink

reference :: String -> Html -> Html
reference ident = anchor ! [href ('#':ident)]

newline = br

emptylines n = concatHtml $ take n $ repeat newline 

emptycell = td << spaceHtml

nocell = td << noHtml

--table2DTable :: [[b]] -> Html
table2DTable  :: String -> [[String]] -> Html
table2DTable title content =
   simpleTable [] [] ((map . map) stringToHtml content) +++ p << ("Table: " ++ title)

-- examples:

testdoc :: MARKUPDOC b => b -> MarkUpDoc
testdoc =
   document
      "My Haskell Home Page"

testsec1 :: Html
testsec1 = section "Peter is ziek" (Just "ziek") << "Moet i ook maar niet gaan vollyeballen, wat hij moet doen is " +++ reference "beter" << "bij sectie 2 kijken"
testsec2 = section "Peter is beter" (Just "beter") << "Kan die weer lekker gaan vollyeballen" +++ reference "ziek" << "bij sectie 1 kijken"

testsec3 = section "Peter weet het even niet meer" (Just "nietmeer") << "Kan die weer lekker gaan vollyeballen" +++ reference "ziek" << "bij sectie 1 kijken"

completedoc = testdoc << (testsec1 +++ dirTable [("peter.html", 13000),("example.html",40000)] +++ testsec2 +++ emptylines 20 +++ testsec3)

main = writeFile "example.html" $ renderHtml completedoc

writeToExFile :: (MARKUPDOC a) => a -> IO ()
writeToExFile = writeFile "example.html" . renderHtml

dirTable :: [(String, Int)] -> Html
dirTable inputs =
   let
      -- getFromLists i = (map . map) (flip (!!) i)
      eta sz speed = flip (++) "s" $ show (floor (fromIntegral sz / speed))
      filerow :: String -> Int -> Html
      filerow fn fs = tr << (td << fn +++ td << (show fs ++ "k")
         +++ td << eta fs 1024
         +++ td << eta fs  512
         +++ td << eta fs 10240
         )
   in
      table ! [border 1, cellspacing 1]
         <<
            (
               tr << ((emptycell ! [colspan 2]) +++ (th ! [colspan 3] << "Estimated download times"))
               +++
               tr << (emptycell +++ (th << "Size") +++ (th << "56kb") +++ (th << "512kb") +++ (th << "10mb"))
               +++
               (concatHtml $ map (\(file, size) -> filerow file size) inputs) -- tr << ((td noHtml)) -- `besides` (th [] "56kb")) --  `besides` (th [] "512kb") `besides` (th [] "10mb")
            )

{-
(define (directory-download dir base)
   (define (eta size speed)
      (format "~as" (/ size (* speed 10))))
   (define (file-download file)
      (let ((sz (file-size file)))
         (tr (td (tt (basename file)))
            (td (/ sz 1024) "k")
            (td (eta sz 56)) (td (eta sz 512)) (td (eta sz 10240)))))
   (table :cellspacing 1 :border 1 :frame 'border :rules 'all
      (tr (td :colspan 2) (th :colspan 3 (it "Estimated download times")))
      (tr (td) (th "size") (th "56kb") (th "512kb") (th "10mb"))
      (map file-download
           (sort (filter (lambda (x) 
                            (and (prefix? x base) (suffix? x "tar.gz")))
                         (directory->list dir))
                 string>?))))

-}

myBody =
      table ! [border 0, {-cellpadding 1, -}cellspacing 1] << tableContents
         +++ br
         +++ p << message
   where
      message
         = "Haskell is a general purpose, purely functional programming language."

tableContents :: HtmlTable
tableContents = (haskell `above` purely) `beside` lambda
    where
      haskell = td ! [align "center"]
                  << font ![size "7",face "Arial Black"] 
                      << "Haskell"
      purely  = td << font ! [size "6"] 
                      << "A Purely Functional Language"
      lambda  = td << image ! [src "lambda.gif"]


data PlotData a b =
     PlotData2DPlain [[a]]
   | PlotData2D [(a,[b])]
   | PlotData1D [a] 
   deriving (Show, Read, Eq)

data HtmlTableLayoutOption x y =
     PD (PlotData x y)
   | CustomShowX (x -> String)
   | CustomShowY (y -> String)
   | AddRowTotal (Maybe Int)
   | AddColumnHeader [String]
   | InsertNumberingColumn (Either Int Char)
   | SortableColumns (Maybe [Int])
--    deriving (Show, Read, Eq)

toHtmlTable :: [HtmlTableLayoutOption a b] -> Html
toHtmlTable options = undefined

data HTMLImgLayoutOption =
     ARef FilePath

toHtmlImg :: [HTMLImgLayoutOption] -> Html
toHtmlImg = undefined

data OutputFormat =
     LateXFormat
   | HtmlFormat

renderMarkUpDoc :: OutputFormat -> MarkUpDoc -> String
renderMarkUpDoc format doc =
   case format of
      LateXFormat -> renderLateX doc
      HtmlFormat-> renderHtml'' doc

renderLateX = undefined
renderHtml'' doc = undefined