module Statistics.Presentation.Ploticus where

{-

PLOTICUS_PREFABS should be set to use prefabs, on linux
export PLOTICUS_PREFABS=/usr/share/ploticus/

A partial GNUPlot script renderer is available from http://darcs.haskell.org/htam/src/GNUPlot.hs

-}


import Statistics.Presentation.TextRepresentation(genSepValuesFile)
import Data.List
import Auxilary
import Statistics.Completion
import Statistics.Ordering

import System.Cmd
import Data.Maybe

{- Todo
- add vdist
- no plot 
/////////////  no data.. exit
<pond> if < apestaart>NRECORDS = 0
  # proc annotate
  location: 3 3
  text:	No data found that 
	corresponded to your request.

  #exit
<pond>endif

-}

ploticusprg = "pl"
ploticus = "pl" ++ ploticusextendlines
ploticusextendlines = " -maxproclines 20000 "  -- default is 5,000, larger box plot require more
outfileopt = " -o " 
pngformat = " -png "
gifformat = " -gif "
plotextension = "pls"
scriptfileopt = " -f "

data PlotOutput = PNGFormat | GIFFormat | PSFormat | X11 -- to screen

instance Show PlotOutput where
 show PNGFormat = pngformat
 show GIFFormat = gifformat
 show PSFormat  = " -ps "
 show X11       = " -x "

showExtention PNGFormat = "png" 
showExtention PSFormat  = "ps"
showExtention X11       = ""


 -- if not set then derived from input file file.pls -> file.png

data Plot =
     PlotScript     { getPlotStatements :: [Statement] }
   | PlotPrefabPlot { getPlotPrefab :: PlotPrefab,
                      getPlotData :: String   }
   deriving (Eq, Show, Read)

data Statement = Proc Name [Attribute]
               | Comment String | Include Bool String | Set Var Value
               | Other Name [Attribute]
   deriving (Eq, Show, Read)

data Attribute = Attribute String (Either String [String])  -- single line or multiple lines
   deriving (Eq, Show, Read)

type Var = String
type Value = String
type Name = String

foldPlot :: Plot -> String
foldPlot = undefined

class PlotPloticus a where
   generate :: a -> String

instance PlotPloticus Plot where
   generate (PlotPrefabPlot fabs _) = {-concatMap -} generate fabs
   generate (PlotScript stats) = unlines $ map generate stats

directive = "#"
newline = "\n"
tab = ""  -- tab breaks the rendering of labels on the xaxis

instance PlotPloticus Statement where
   generate (Proc name attrs) = directive ++ "proc " ++ name ++ newline ++ (unlines $ map generate attrs) ++ directive ++ "endproc" ++ newline
   generate (Comment cmt) = "//// " ++ cmt
   generate (Include plotdir name) = directive ++ "include " ++ (if plotdir then "$" else "") ++ name
   generate (Set var val) = directive ++ "set " ++ var ++ " = " ++ val

instance PlotPloticus Attribute where
   generate (Attribute name e) = tab ++ name ++ ": " ++ (either id ((newline++) . (tab++) . (++newline) . unlines . map (tab++)) e)

instance PlotPloticus PlotPrefab where
   generate (PlotPrefab name divs) = ploticus ++ " -prefab " ++ name ++ " " ++ (concat $ intersperse " " $ map generate divs)

instance PlotPloticus PrefabDirective where
   generate (PrefabDirective name value) = name ++ "=" ++value

-- generatePlotCSV ::

-- use renderPlot, to also properly write the data as a seperate file
-- generateBarPlot :: (Show a, Show b, Num b) => String -> [(a,b)] -> String
-- generateBarPlot title plotdata = generatePloticusScript $ barPlot title plotdata

-- current maximum of stacked groups of ploticus
maxBarPlot2D = 10

-- (stringToFilePath title) is used to name the data file
-- barPlot :: (Show a, Show b, Num b) => String -> [String] -> [(a,b)] -> Plot
-- barPlot title cols plotdata =
--    let headeropt = if length cols == 2 then "yes" else "no"
--    in
--       PlotPrefabPlot (
--          foldl addPrefabDiv barsprefab [
-- 	         dataplot `addValueToDiv` ((stringToFilePath title) ++ ".csv")
--                , header `addValueToDiv` "no" -- headeropt
--                , delim `addValueToDiv` "comma"
--                , vals `addValueToDiv` "yes"
--                , xaxis `addValueToDiv` "1"
--                , yaxis `addValueToDiv` "2" 
--        --        , plottitle `addValueToDiv` (show title)
--                ]
--          )
--          (generatePloticusData2D [] {-cols-}  (map (\(x,y) -> [show x, show y]) plotdata))
--          --(generatePloticusData cols plotdata)

barPlot :: (Show b, Num b) => String -> (String,String) -> [(String,b)] -> Plot
barPlot title (xaxis_title, yaxis_title) plotdata =
--    let headeropt = if length cols == 2 then "yes" else "no"
--    in
      PlotPrefabPlot (
         foldl addPrefabDiv barsprefab [
	         dataplot `addValueToDiv` ((stringToFilePath title) ++ ".csv")
               , header `addValueToDiv` "no" -- headeropt
               , delim `addValueToDiv` "comma"
               , vals `addValueToDiv` if length plotdata >10 then "no" else "yes"
               , xaxis `addValueToDiv` "1"
               , yaxis `addValueToDiv` "2"
--                , xlbl `addValueToDiv` (show xaxis_title)
--                , plottitle `addValueToDiv` (show title)
               , PrefabDirective "stubvert" "yes"
               ]
         )
         (generatePloticusData2D [] {-cols-}  (map (\(x,y) -> [x, show y]) plotdata))
         --(generatePloticusData cols plotdata)

showCustomMaybe :: Num a => Maybe a -> String
showCustomMaybe (Just a) = show a
showCustomMaybe (Nothing) = "="

-- (stringToFilePath title) is used to name the data file
barPlotWithComplAndOrder :: (Show a, Eq a, Show b, Num b) => Maybe (AxisInfo a) -> Maybe (AxisInfo b) -> String -> [String] -> [(a,b)] -> Plot
barPlotWithComplAndOrder xaxisInfo yaxisInfo title cols plotdata =
   let headeropt = if length cols == 2 then "yes" else "no"
       
       datacompletion = 
          if isJust xaxisInfo 
                    then either
                           (\allcats -> sortUnder (listOrdering allcats) . (complementGen allcats 0))
                           (\xorder -> sortUnder xorder) 
                           (orderAndCompletion $ fromJust xaxisInfo)
                    else id
   in
      PlotPrefabPlot (
         foldl addPrefabDiv barsprefab [
                 dataplot `addValueToDiv` ((stringToFilePath title) ++ ".csv")
               , header `addValueToDiv` "no" -- headeropt
               , delim `addValueToDiv` "comma"
               , vals `addValueToDiv` "yes"
               , xaxis `addValueToDiv` "1"
               , yaxis `addValueToDiv` "2"
            --   , xlbl `addValueToDiv` (show xaxis_title)  -- pvk
            --   , plottitle `addValueToDiv` (show title)
               , PrefabDirective "stubvert" "yes"
               ] -- ++ stubdirection
         )
         (case (xaxisInfo, yaxisInfo) of
             (Just xaxisI, Just yaxisI) ->
                generatePloticusData2D {-cols-} [] (map (\(x,y) -> [(customShow $ fromJust xaxisInfo) x, (customShow $ fromJust yaxisInfo) y]) $ datacompletion plotdata)
             (Just xaxisI, Nothing) ->
                generatePloticusData2D {-cols-} [] (map (\(x,y) -> [(customShow $ fromJust xaxisInfo) x, show y]) $ datacompletion plotdata)
             (Nothing, Just yaxisI) ->
                generatePloticusData2D {-cols-} [] (map (\(x,y) -> [show x, (customShow $ fromJust yaxisInfo) y]) $ datacompletion plotdata)
             (Nothing, Nothing) ->
                generatePloticusData2D {-cols-} [] (map (\(x,y) -> [show x, show y]) $ datacompletion plotdata)
         )
         {-
     where
        stubdirection | isJust xaxisInfo = if min $ map ((customShow $ fromJust xaxisInfo) . fst) plotdata
                      | otherwise = 
-}

data PlotOption = ScriptOpt {scriptopts :: [Statement] }
                | PrefabOpt {prefabopts :: [PrefabDirective] }

-- Predefined Options
yrange mini maxi = ScriptOpt $ [Set "yrange" $ show $ unwords [show mini, show maxi]]
singlelnlegend =   ScriptOpt $ [Set "legendfmt" "across"]
----
-- StackedBarPlot
-- Shows numeric values per group -- double grouped data, complementing the missing values from the data.
----

stackedBarPlot :: (Num b) => String -> (String, String) -> [String] -> [(String, [b])] -> [PlotOption] -> Plot
stackedBarPlot = barPlot2D



-- simple stacked barplot, no completion as much as could
-- barPlot2D :: (Show a, Show b, Num b) => String -> (String, String) -> [String] -> [(a,[b])] -> Plot
barPlot2D :: (Num b) => String -> (String, String) -> [String] -> [(String, [b])] -> [PlotOption] -> Plot
barPlot2D title (xaxis, yaxis) cats plotdata plotoptions =
   let numGrps = map (snd) plotdata
       -- some checks on the group names
       diffGrpsSizes = group $ map length numGrps
       noOfGrps = maximum $ map length numGrps
       chkGrpsOk = if length diffGrpsSizes > 1
                   then error $ "Error: number of category/group names differ in the input data, unlikely to render proper barplot from this"
                   else if noOfGrps > maxBarPlot2D
                   then error $ "Error: number of groups exceeds the maximum of a 2d barplot (" ++ show maxBarPlot2D ++ ")"
                   else if ((length cats /= noOfGrps) && debug)
                   then error "Error: number of provided category descriptions differ from the number of categories in set"
                   else True
       grpnames = cats -- if chkGrpsOk then head $ filter ((==) noOfGrps . length) grpnamesList else []
       transfplotdata = map (\(e1,l2) -> e1 : (map show l2)) plotdata
       catgrps = if noOfGrps > 0
                   then ((Set "y" $ show 2):(Set "name" (show $ head grpnames)):[]) ++
                    (
                        concat $ map (\(name,no) -> (Set ("y" ++ show no) (show $ no+1)):(Set ("name"++show no) (show name)):[]) $ zip (tail grpnames) [2..(min maxBarPlot2D noOfGrps)]
                    )
                   else ((Set "no_y_in_grps" $ show "empty categories"):[])
   in
      PlotScript $
         (Proc "getdata" [ Attribute "data" (Right $ [(generatePloticusData2D [] {- cols -} transfplotdata )]) -- column/groupnames in names directives
                         , Attribute "delim" (Left "comma")
                         ]):
         -- (Set "title" (show title)): -- curr disabled, if to long, then goes throug legenda
         -- (Set "vals" "yes"): -- doesn't work for stack
         -- (Set "xaxis" xaxis) -- what happens when empty?
         -- (Set "reverseleg" "yes"):  -- reverse legenda
         (Set "stubvert" "yes"):    -- vertical labels on the xaxis
         (Set "x" $ show 1):
         (Set "yrange" "0 "):
--         (Set "xlbl" $ show xaxis):
         [] ++ catgrps ++
         (concatMap scriptopts plotoptions) ++
         (Include True "stack_ext.pl"):
         []

-- stacked bar plot with automatic (in data) completion
barPlot2DWithAutoCompletion :: (Show a, Eq a, Show a2, Eq a2, Show b, Num b) => String -> (String, String) -> [(a,[(a2,b)])] -> Plot
barPlot2DWithAutoCompletion title (xaxis, yaxis) plotdata =
   let ycats = nub $ concat $ map (map fst . snd) plotdata 
       complementeddata input = 
          map (\(x,ys) -> (x, sortBy (\(y1,_) (y2,_) -> (listOrdering ycats) y1 y2) ys)) $ complementGen2dYs ycats 0 input
 
--       complementeddata input = map (\(x,ys) -> (x, sortBy (\(y1,_) (y2,_) -> (listOrdering ycats) y1 y2) $ complementGen ycats 0 ys)) input
       grpnamesList = map ((map fst) . snd) (complementeddata plotdata)
       grpnames = if null grpnamesList then [] else map show $ head $ grpnamesList
   in
      barPlot2D title (xaxis, yaxis) grpnames (map (\(e1, l2) -> (show e1, map (snd) l2)) (complementeddata plotdata)) []


{-
data AxisInfo a = AxisInfo { completion :: [a],
                             ordering :: Maybe (a -> a-> Ordering)
                           }

-- stacked bar plot with optional predefined completion, otherwise automatic indata completion
barPlot2DWithOptComplAndOrder :: (Show a, Show a2, Eq a2, Show b, Num b) => Maybe (AxisInfoE a2) -> String -> (String, String) -> [(a,[(a2,b)])] -> Plot
barPlot2DWithOptComplAndOrder xaxisinfo title (xaxis, yaxis) plotdata =
   let allycats = completion $ fromJust xaxisinfo
       yorder = AxisInfo maybe (listOrdering allycats) id (ordering $ fromJust xaxisinfo)
       complementeddata = map (\(x,ys) -> (x, sortBy (\(y1,_) (y2,_) -> yorder y1 y2) $ complementGen allycats 0 ys)) plotdata
       grpnamesList = map ((map fst) . snd) complementeddata
       grpnames = if null grpnamesList then [] else map show $ head $ grpnamesList
   in
      if isJust xaxisinfo
      then
         barPlot2D title (xaxis, yaxis) grpnames (map (\(e1, l2) -> (e1, map snd l2)) complementeddata)
      else 
         barPlot2DWithAutoCompletion title (xaxis, yaxis) plotdata
-}

data AxisInfo a = AxisInfo { orderAndCompletion :: Either (Completion a) (a -> a-> Ordering),
                             customShow         :: a -> String
                            }

type Completion a = [a]

-- todo: move to KeyHistoryImpl: (disadv. use of outputspecifics? just add 'features' as ploticus 'datefmt')

-- stacked bar plot with optional predefined completion, otherwise automatic indata completion
barPlot2DWithOptComplAndOrder :: (Show a, Eq a, Show a2, Eq a2, Show b, Num b) => Maybe (AxisInfo a) -> Maybe (AxisInfo a2) -> String -> (String, String) -> [(a,[(a2,b)])] -> Plot
barPlot2DWithOptComplAndOrder xaxisinfo yaxisinfo title (xaxis, yaxis) plotdata =
   let -- allycats = either (id) (error "error rendering") $ fromJust xaxisinfo -- completion $ fromJust xaxisinfo
       -- yorder = either (listOrdering) id (fromJust xaxisinfo)
       -- complementeddata = map (\(x,ys) -> (x, sortBy (\(y1,_) (y2,_) -> yorder y1 y2) $ complementGen allycats 0 ys)) plotdata
   in
      case (xaxisinfo, yaxisinfo) of
         (Just xaxisI, Just yaxisI) -> 
--          case (orderAndCompletion xaxisI, orderAndCompletion yaxisI) of
--           (Left  
          either
            (\xcats ->
              either  
               (\ycats ->  -- Left xaxisI, Left yaxisI
                  let
                     complementeddata = complementGen2dAndOrderXY (listOrdering xcats) (listOrdering ycats) xcats ycats 0 plotdata
--                        map (\(x,ys) -> (x, sortBy (\(y1,_) (y2,_) -> listOrdering ycats y1 y2) $ complementGen ycats 0 ys)) plotdata
                     grpnamesList = map ((map fst) . snd) complementeddata
                     grpnames = if null grpnamesList then [] else map (customShow $ fromJust yaxisinfo) $ head $ grpnamesList
                  in
                     barPlot2D title (xaxis, yaxis) grpnames
                        (map (\(e1, l2) -> ((customShow $ fromJust xaxisinfo) e1, map snd l2)) complementeddata)
                        [] 
               )
               (\yorder ->  -- Left xaxisI, Right yaxisI 
                   undefined {-
                  let
                     complementeddata = complementGen2dAndOrderXY (listOrdering xcats) yorder xcats [] 0 plotdata
                     grpnamesList = map ((map fst) . snd) complementeddata
                     grpnames = if null grpnamesList then [] else map (customShow $ fromJust yaxisinfo) $ head $ grpnamesList
                  in
                     barPlot2D title (xaxis, yaxis) grpnames
                        (map (\(e1, l2) -> ((customShow $ fromJust xaxisinfo) e1, map snd l2)) complementeddata)
--                   barPlot2DWithAutoCompletion title (xaxis, yaxis)
   --                  (map (\(x,ys) -> (x, sortUnder yorder ys)) plotdata)
                             -}
               )
               (orderAndCompletion $ fromJust yaxisinfo)
            ) 
            (\xorder ->
              either  
               (\ycats ->  -- Right xaxisI, Left yaxisI
                  let
                     complementeddata = complementGen2dAndOrderXY (xorder) (listOrdering ycats) [] ycats 0 plotdata
                     grpnamesList = map ((map fst) . snd) complementeddata
                     grpnames = if null grpnamesList then [] else map (customShow $ fromJust yaxisinfo) $ head $ grpnamesList
                  in
                     barPlot2D title (xaxis, yaxis) grpnames
                        (map (\(e1, l2) -> ((customShow $ fromJust xaxisinfo) e1, map snd l2)) complementeddata)
                        []
               )
               (\yorder ->  -- Right xaxisI, Right yaxisI 
                  let
                     ycats = nub $ concat $ map (map fst . snd) plotdata
                     complementeddata = complementGen2dAndOrderXY (xorder) yorder [] ycats 0 plotdata
                     grpnamesList = map ((map fst) . snd) complementeddata
                     grpnames = if null grpnamesList then [] else map (customShow $ fromJust yaxisinfo) $ head $ grpnamesList
                  in
                     barPlot2D title (xaxis, yaxis) grpnames
                        (map (\(e1, l2) -> ((customShow $ fromJust xaxisinfo) e1, map snd l2)) complementeddata)
                        []
               )
               (orderAndCompletion $ fromJust yaxisinfo)
 
            )
            (orderAndCompletion $ fromJust xaxisinfo)
         (Just xaxisI, Nothing) -> error "undefined case 2"
         (Nothing, Just yaxisI) -> error "undefined case 3"
         (Nothing, Nothing) -> barPlot2DWithAutoCompletion title (xaxis, yaxis) plotdata

generatePloticusData2D :: [String] -> [[String]] -> String
generatePloticusData2D cols plotdata = if null cols then csvdata plotdata else csvdata $ cols : plotdata
   where
      -- allbutcomma = filter (/=',')
      csvdata input = unlines $ map (concat . intersperse "," . map (toPlotiString)) input
      toPlotiString = concat . map f
         where
            f c | c == '"' = "\"\""
                | c == ',' = "_"
                | otherwise = [c]

-- Limitation can play a role in range bar plots / box plots
-- max of 5.000 lines in a proc, http://ploticus.sourceforge.net/doc/limits.html

boxPlot :: (Show b, Ord b, Num b) => String -> (String, String) -> [(String, [b])] -> Plot
boxPlot = rangeBarPlot

rangeBarPlot :: (Show b, Ord b, Num b) => String -> (String, String) -> [(String, [b])] -> Plot
rangeBarPlot name (xaxistitle , yaxistitle) plotdata =
   let
      maxValue [] = 10 -- default y axis if [b] is empty
      maxValue pdata = if (not $ null innermax) then maximum $ innermax else 10
         where innermax = map maximum $ filter (/=[]) $ map snd $ pdata
      grps = map fst plotdata
      -- generatePloticusData2D :: [String] -> [[String]] -> String
      transfplotdata pdata | null pdata = [["-"]]
                           | null $ nub (map snd pdata) = [["-"]]
                           | otherwise = transpose
                                          $ map (\list -> take (maxlist {-+1-}) (list ++ repeat ""))
                                          $ map (\(x,y) -> {- show x : -}map show y) pdata
                              where maxlist = maximum $ map (length .snd) (plotdata ++ [("",[])]) -- latter to avoid error
      rangebars [] = []
      rangebars pdata = mainbar : map cloneB [2..length plotdata]
      mainbar = (Proc "rangebar" [ Attribute "datafield" (Left "1")
                          , Attribute "barloc" (Left "1") -- or 0.5
                          , Attribute "nlocation" (Left "0.1 (s)") -- or 0.5
                          -- , Attribute "showstats" (Left "yes")
                          , Attribute "color" (Left "pink")
                          -- , Attribute "mediansym" (Left "radius=0")
                          , Attribute "meansym" (Left "shape=circle style=fill fillcolor=blue radius=0.03")
                          , Attribute "mediansym" (Left "shape=circle style=fill fillcolor=green radius=0.03")
                          , Attribute "showoutliers" (Left "yes") -- circle and astersks for outliers as default
                          , Attribute "tailmode" (Left "5/95")  -- just to present option
                          , Attribute "showoutliers" (Left "yes")
                          , Attribute "outlierlinelen" (Left "0.1")
                          , Attribute "outlierlinedetails" (Left "color=black width=1.0")
                          , Attribute "#saveas B" (Right [""])
                          ])
      -- copies rangebar saved under B
      cloneB num = (Proc "rangebar" [ Attribute "#clone B" (Left "")
                         , Attribute "datafield" (Left $ show num)
                         , Attribute "barloc" (Left $ show num)
                         ])
      
   in
      PlotScript $  
         (Proc "getdata" [ Attribute "data" (Right $ [(generatePloticusData2D [] {- cols -} $
                                                   transfplotdata plotdata)]) -- column/groupnames in names directives
                         , Attribute "delim" (Left "comma")
                         ]):
         (Proc "areadef" [ Attribute "rectangle" (Left "1 1 5 5")
                         , Attribute "xrange" (Left $ "0 " ++ (show $ length plotdata + 1))    -- set num of rangebars, or autorange
                         , Attribute "yrange" (Left $ "0 " ++ (show $ maxValue plotdata)) -- set max, or autorange
                           -- //  xscaletype: categories // would make a cat . x axis, so locbar would not work anymore
                         ]):
         -- first render y axis and colored grid, then xaxis, to get a black xaxis, instead of green
         (Proc "yaxis" [ Attribute "grid" (Left "color=green")
                         , Attribute "stubs" (Left "inc")
                         -- , Attribute "tics" (Left "none")
                         -- , Attribute "axisline" (Left "none")
                         ]):
         (Proc "xaxis" [ -- Attribute "axisline" (Left "none")
                           Attribute "stubs" (Right grps) -- throws warning!! should off be of form stubs: text \ grp1 \ grp2
                         , Attribute "stubvert" (Left "yes")
                         ]):
         (rangebars plotdata)

-- to show stat on stdout
-- showbriefstats: yes  -- on one line, tab delim
-- showstats: yes

{-
Examples:
-- http://ploticus.sourceforge.net/gallery/rangebar1.htm
--http://ploticus.sourceforge.net/gallery/mouse.htm

ploticus, calc the needed data, from dev/plots/exampls

// set up the area and scaling..
#
proc areadef
  rectangle: 1 1 5 5
  xrange: 0 9   -- set num of rangebars, or autorange
  yrange: 0 250 i-- set max, or autorange
//  xscaletype: categories // would make a cat . x axis, so locbar would not work anymore

 #proc xaxis
  axisline: none
  stubs: text
    Week 6  -- set titles
    Week 7
    Week 8
    Week 9
    Week 10
    Week 11
    Week 12
    Week 13
    Week 14

// do a Y axis (grid only).. 
 #proc yaxis
  grid: color=gray(0.8)"
  stubs: inc

// do the 1st boxplot.. 
 #proc rangebar
  datafield: 1
  barloc: 1
  nlocation: 0.5 (s)
  //showstats: yes
  color: pink
  mediansym: radius=0
  meansym: shape=circle style=fill fillcolor=blue radius=0.03
  mediansym: shape=circle style=fill fillcolor=green radius=0.03
  #saveas B

// do the 2nd boxplot, cloning the 1st.. 
 #proc rangebar
  #clone B
  datafield: 2   -- extend number
  barloc: 2

// and so on.. 
 #proc rangebar
  #clone B
  datafield: 3
  barloc: 3

-}

-- Vdist Prefab: pl -prefab vdist y=1 data=distr_data.txt dist=yes mediansym=dot -png

scatterPlot :: (Show a, Ord a, Num a, Show b, Ord b, Num b) => String -> (String, String) -> [(a, [b])] -> Plot
scatterPlot name (xaxistitle , yaxistitle) []  = undefined
scatterPlot name (xaxistitle , yaxistitle) plotdata = undefined


----
-- Prefab formulation, for direct cmd call of ploticus
-- Requires to have csv file first
----
data PlotAction = PlotAction {
      pa_name :: String,
      pa_goals :: [ActionGoal]
   }
   deriving (Eq, Show, Read)

data ActionGoal = ActionGoal {
      ag_name :: String,
      ag_value :: [String]
   }
   deriving (Eq, Show, Read)

data PlotPrefab = PlotPrefab {
      pf_name :: String,
      pf_divs :: [PrefabDirective]
   }
   deriving (Eq, Show, Read)

data PrefabDirective = PrefabDirective {
      pd_name :: String,
      pd_value :: String -- [String]
   }
   deriving (Eq, Show, Read)

prefab :: String -> PlotPrefab
prefab name = PlotPrefab name []

addPrefabDiv :: PlotPrefab -> PrefabDirective -> PlotPrefab
addPrefabDiv (PlotPrefab name divs) dv = (PlotPrefab name (dv:divs))

-- provided prefabs
bars = "vbars"  -- simple barplots
stack = "stack"  -- line and bars
chron = "chron" -- timeline

barsprefab = prefab bars
stackedbarsprefab = prefab stack
stackprefab = stackedbarsprefab

-- prefab directives
filedata = PrefabDirective "data" ""
inlinedata = PrefabDirective "inlinedata" ""
dataplot = PrefabDirective "data" ""
xaxis = PrefabDirective "x" ""
xlbl = PrefabDirective "xlbl" ""
yaxis = PrefabDirective "y" ""
yname = PrefabDirective "name" ""
vals =  PrefabDirective "vals" ""
delim = PrefabDirective "delim" ""
header = PrefabDirective "header" ""
plottitle = PrefabDirective "title" ""

addValueToDiv :: PrefabDirective -> String -> PrefabDirective
addValueToDiv (PrefabDirective name _) value = (PrefabDirective name value)

-- q's: data in file /inline?
-- prefab vs script?

-- different ways to handle plots

-- generate prefab call, used by renderPlot and writePlotFile
formPlotCall :: FilePath -> FilePath -> PlotOutput -> Plot -> String
formPlotCall fp filename output plot=
  ("cd " ++ fp ++ "; " ++ generate plot ++ show output ++ outfileopt ++ (filename ++ "." ++ showExtention output)) 

-- writes the datafile first to the fp ++ filename
-- rendering by changing dir into fp
-- barplot applies stringToFilePath to name
renderPlot :: FilePath -> FilePath -> PlotOutput -> Plot -> IO FilePath
-- prefab call
renderPlot fp filename output complPlot@(PlotPrefabPlot plot csvdata) = do
   writeFile datafile csvdata
   system $ formPlotCall fp (stringToFilePath filename) output complPlot
   if debug then putStrLn $ formPlotCall fp (stringToFilePath filename) output complPlot
            else return ()
--   system $ ("rm " ++ fp)
   return (fp ++ pathSeparator : filename ++ "." ++ showExtention output)  -- should be joinDirFileExt   
   where 
     datafile = joinDirFileExt fp (stringToFilePath filename) "csv"

-- script call
renderPlot fp filename output complPlot@(PlotScript stats) = do
   if debug then putStrLn $ scriptcall
            else return ()
   writeFile (joinFileName fp scriptfile) script
   system $ scriptcall
   return $ joinFileName fp imgfile -- (fp ++ pathSeparator : filename ++ "." ++ showExtention output)
   where
      scriptcall = ("cd " ++ fp ++ "; " ++ ploticus ++ scriptfileopt ++ show scriptfile ++ show output
            ++ outfileopt -- ++ fp ++ filename ++ "." ++ showExtention output
            ++ imgfile)
      scriptfile = joinFileExt filename plotextension -- joinDirFileExt fp filename plotextension
      imgfile = joinFileExt filename (showExtention output) -- joinDirFileExt fp filename (showExtention output)
      script = generate complPlot

--X11 only
renderAndShowPlot :: FilePath -> FilePath -> Plot -> IO ()
renderAndShowPlot fp filename plot = 
   do renderPlot fp filename X11 plot
      return () 

renderPNGPlot :: FilePath -> FilePath -> Plot -> IO FilePath
renderPNGPlot fp filename plot = 
	renderPlot fp filename PNGFormat plot

writePlotFiles :: FilePath -> FilePath -> PlotOutput -> Plot -> IO FilePath
writePlotFiles fp filename output complPlot@(PlotPrefabPlot plot csvdata) = do
   writeFile (datafile ++ ".csv") csvdata
   writeFile (datafile ++ ".pls") $ formPlotCall fp filename output complPlot
   return (datafile ++ ".pls")
   where 
     datafile = fp ++ filename

