module Statistics.Presentation.RepresentationActions (
            module Statistics.Presentation.RepresentationActions
          , module System.Directory
          , module System.Cmd
          , module System.Exit
          , module Distribution.Compat.FilePath
          ) where

import System.Cmd
import System.Directory
import System.Exit
import Distribution.Compat.FilePath

import Auxilary(traceVal)

-- IO related actions to handle result, example: write to file, call tool etc.

showFile :: FilePath -> IO ()
showFile file  = do content <- readFile file; putStrLn content

catFile = showFile

----
-- browser actions
---

browser = "firefox"
dataURI = "data:"
htmltext = "text/html"
plaintext = "text/plain"

-- see:
-- http://en.wikipedia.org/wiki/URL
-- http://en.wikipedia.org/wiki/Data:_URL
-- http://www.mozilla.org/quality/networking/docs/aboutdata.html (base

showHTMLInBrowser :: String -> IO ExitCode
showHTMLInBrowser content =
   system $ browser ++ " " ++ dataURI ++ htmltext ++ "," ++ content

showFileInBrowser :: FilePath -> IO ExitCode
showFileInBrowser fp =
   system $ browser ++ " " ++ fp

-- showInBrowserViaTmpFile 
--    do tmpFile = 

-- research actions
writeToFile filepath = writeFile (filepath)

writeToFiles :: [FilePath] -> [String] -> IO ()
writeToFiles [] (content:_) = error "Not enough filename provided when writing set of data results"
writeToFiles (filenm:outfiles) (content:contents) =
   do writeFile filenm content
      writeToFiles outfiles contents

writeToFiles' :: [(FilePath,String)] -> IO ()
-- writeToFiles [] (content:_) = error "Not enough filename provided when writing set of data results"
writeToFiles' = mapM_ (\(filenm, content) -> writeFile filenm content)

writeToNumberedFiles :: String -> String -> [String] -> IO ()
writeToNumberedFiles filename_prefix extension= writeToFiles' . (zip $ (map (\no -> filename_prefix ++ (show no) ++ '.':extension) [0..]))

runTool = undefined

-- defined in Statistics.Presentation.GNUPlot
-- runGNUPlot :: String -> IO ()
-- runGNUPlot = runTool

runToolOn :: FilePath -> IO ExitCode
runToolOn fp = system fp

runToolAndDeferTo :: FilePath -> FilePath -> IO ExitCode
runToolAndDeferTo fp to = system $ fp ++ " > " ++ to

-- to have a \n nicely put after every string in the input list
sequencePutStrLn :: [String] -> IO ()
sequencePutStrLn = sequence_ . map putStrLn


runCmdOnRemoteServer :: String -> String -> IO ExitCode
runCmdOnRemoteServer host runcmd =
   do let cmd = "ssh " ++ host ++ " '" ++ runcmd ++ "'"
      err <- system cmd
      case err of
         ExitSuccess -> putStrLn $ "Command " ++ cmd ++ " succesfully run on remote host " ++ host
         ExitFailure errcode -> putStrLn $ "Unable to run cmd " ++ cmd ++ " on remote host " ++ host ++ ". Returned error code: " ++ show errcode
      return err

makeRemoteDir :: String -> String -> IO ExitCode
makeRemoteDir host dir = runCmdOnRemoteServer host ("(mkdir -p " ++ dir ++ " && chmod a+rx " ++ dir ++ " )")

-- copying file to remote server
-- basedir is excluded from dir structure on remote server
publishOnWebSpace :: String -> FilePath -> [FilePath] -> FilePath -> IO ExitCode
publishOnWebSpace host basedir localfiles remotepath =
   do let cmd = "(cd " ++ basedir ++
               " && tar -cf - " ++ (sumfiles localfiles) ++
                     ") | ssh " ++ host ++ " '( cd " ++ remotepath ++ " && tar -xf - && chmod -R a+rx *" ++ ")'"
          -- "scp -p -r " ++ files ++ "<to>"
      err <- system cmd

      case err of
         ExitSuccess -> putStrLn $ "Files " ++ (sumfiles localfiles) ++ " from " ++ basedir ++ " copied to " ++ remotepath ++ " on " ++ host
         ExitFailure errcode -> putStrLn $ "Unable to publish files on remote server. Returned error code: " ++ show errcode
      return err

-- when error occures, output to screen, not always represented in ExitCode, (for instance if basedir is non existent)

sumfiles :: [FilePath] -> String
sumfiles files = foldl (\sumedfiles file -> sumedfiles ++ " " ++ file) "" files

removePublishedAnalysesOnWebSpace :: String -> FilePath -> [FilePath] -> IO ExitCode
removePublishedAnalysesOnWebSpace host remotepath files =
   do let cmd =
               " ssh " ++ host ++ " '(cd " ++ remotepath ++ " && rm -rf " ++ (sumfiles files) ++ ")'"
      
      err <- system cmd
      case err of
         ExitSuccess -> putStrLn $ "Files " ++ (sumfiles files) ++ " removed from " ++ remotepath ++ " on " ++ host
         ExitFailure errcode -> putStrLn $ "Unable to remove files from remote server. Returned error code: " ++ show errcode
      return err

removePublishedAllAnalysesOnWebSpace host remotepath = removePublishedAnalysesOnWebSpace host remotepath ["*"]

sed :: {-Text.Regex -}a -> FilePath -> IO String
sed regex fp =
   do content <- readFile fp
      let result = error $ "no sed implementation yet"  -- apply regex content
      return (result) 