module Statistics.Presentation.TextRepresentation where

-- Simple text representations, pp

import Data.List (transpose, intersperse)
import Text.PrettyPrint

dash :: Doc
dash = char '-'

showAsTableGen :: String -> [String] -> (a -> String) -> (b -> String) -> [(a,[b])] -> String
showAsTableGen title {- (xaxis, yaxis) -} columns showa showb input =
      render $
         space <> text title
            $+$ (hcat $ replicate ((length title) + 2) dash)
            $+$ (vcat $ map text $ lines $ buildTable columns $ map (\(v1, list) -> (showa v1, map showb list)) input)
            $+$ text "" -- empty line


showAsTable2DSimple :: String -> {-(String, String) -> -} [String] -> (a -> String) -> (b -> String) -> [(a,b)] -> String
showAsTable2DSimple title columns showa showb input = showAsTableGen title columns showa showb $ map (\(v1, v2) -> (v1, [v2])) input

showAsTable2DCat :: String -> {-(String, String) -> -} [String] -> (a -> String) -> (b -> String) -> [(a,[b])] -> String
showAsTable2DCat title columns showa showb input = showAsTableGen title columns showa showb input


-- class for abstracting over datatype for showing table, datatype(s) of a should all have one single preferred Show instance, which is not always the case (ex. keyhistory and/or date/time), then use the above functions
class TextTableable a where
   showAsTable :: String -> [String] -> a -> String

showAsTableGen' :: (Show a, Show b) => String -> [String] -> [(a,[b])] -> String
showAsTableGen' title {- (xaxis, yaxis) -} columns input = showAsTableGen title columns show show input

instance (Show a, Show b) => TextTableable [(a,[b])] where
   showAsTable title columns input = showAsTableGen' title columns input

instance (Show a, Show b) => TextTableable [(a,b)] where
   showAsTable title columns input = showAsTableGen' title columns $ map (\(v1, v2) -> (v1, [v2])) input

----
-- Table generating function from J. Meacham
----

--  $Id: GenUtil.hs,v 1.48 2006/01/29 05:22:26 john Exp $
-- arch-tag: 835e46b7-8ffd-40a0-aaf9-326b7e347760


-- Copyright (c) 2002 John Meacham (john@foo.net)
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the
-- "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish,
-- distribute, sublicense, and/or sell copies of the Software, and to
-- permit persons to whom the Software is furnished to do so, subject to
-- the following conditions:
--
-- The above copyright notice and this permission notice shall be included
-- in all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-- CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

-- | 'concat' composed with 'List.intersperse'. Can be used similarly to join in perl.
concatInter :: String -> [String] -> String
concatInter x = concat . (intersperse x)

-- | place spaces before each line in string.
indentLines :: Int -> String -> String
indentLines n s = unlines $ map (replicate n ' ' ++)$ lines s

-- pvk: fst argument are the col header of the snd values in the list of input tuples ( the b elms in in the [(a,[b])]), so the first column does not get any header.. :(
buildTable ::  [String] -> [(String,[String])] -> String
buildTable ts rs = bt [ x:xs | (x,xs) <- ("",ts):rs ] where
    bt ts = unlines (map f ts) where
        f xs = concatInter " " [  es n s | s <- xs | n <- cw ]
        cw = [ maximum (map length xs) | xs <- transpose ts]
    es n s = replicate (n - length s) ' ' ++ s


buildTableRL :: [(String,String)] -> [String]
buildTableRL ps = map f ps where
    f (x,"") = x
    f (x,y) = replicate (bs - length x) ' ' ++ x ++ replicate 4 ' ' ++ y
    bs = maximum (map (length . fst) [ p | p@(_,_:_) <- ps ])

buildTableLL :: [(String,String)] -> [String]
buildTableLL ps = map f ps where
    f (x,y) = x ++ replicate (bs - length x) ' ' ++ replicate 4 ' ' ++ y
    bs = maximum (map (length . fst) ps)

-- end used code from J. Meacham

buildTable' ::  [String] -> [(String,[String])] -> String
buildTable' titles rows = undefined


-- End J. Meacham

----
-- CSV Generation, GNU LGPL 
----

{- arch-tag: CSV and TSV utilities
Copyright (C) 2005 John Goerzen <jgoerzen@complete.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-}

{- |
   Module     : MissingH.Str.CSV
   Copyright  : Copyright (C) 2005 John Goerzen
   License    : GNU GPL, version 2 or above

   Maintainer : John Goerzen <jgoerzen@complete.org>
   Stability  : provisional
   Portability: portable

Haskell Parsec parsers for comma-separated value (CSV) files.

Written by John Goerzen, jgoerzen\@complete.org
-}

-- module MissingH.Str.CSV(csvFile, genCsvFile) where
{-
eol = (try $ string "\n\r") <|> (try $ string "\r\n") <|> string "\n" <|>
      string "\r" <?> "End of line"
cell = quotedcell <|> many (noneOf ",\n\r")
quotedchar = noneOf "\"" 
             <|> (try $ do string "\"\""
                           return '"'
                 )
quotedcell :: CharParser st String
quotedcell = do char '"'
                content <- many quotedchar
                char '"'
                return content
line = sepBy cell (char ',')

{- | Parse a Comma-Separated Value (CSV) file.  The return value is a list of
lines; each line is a list of cells; and each cell is a String.

Please note that CSV files may have a different number of cells on each line.
Also, it is impossible to distinguish a CSV line that has a call with no data
from a CSV line that has no cells.

Here are some examples:

>Input (literal strings)          Parses As (Haskell String syntax)
>-------------------------------- ---------------------------------
>1,2,3                            [["1", "2", "3"]]
>
>l1                               [["l1"], ["l2"]]
>l2
>
> (empty line)                    [[""]]
>
>NQ,"Quoted"                      [["NQ", "Quoted"]]
>
>NQ,"Embedded""Quote"             [["NQ", "Embedded\"Quote"]]

To parse a String, you might use:

>import Text.ParserCombinators.Parsec
>import MissingH.Str.CSV
>....
>parse csvFile "" mystring

To parse a file, you might instead use:

>do result <- parseFromFile csvFile "/path/to/file"

Please note that the result of parsing will be of type
(Either ParseError [[String]]).  A Left result indicates an error.
For more details, see the Parsec information.
-}

csvFile :: CharParser st [[String]]
csvFile = endBy line eol
-}

-- own code

data PlotData a b =
     PlotData2DPlain [[a]]
   | PlotData2D [(a,[b])]
   | PlotData1D [a]
   deriving (Show, Read, Eq)

data CSVOption a b =
     CSVShowX (a -> String)
   | CSVShowY (b -> String)
   | CSVSeperator String
   | CSVData (PlotData a b)


-- Generic showAsCSVFile, with flexible type
renderAsCSVFileWithOptions :: (Show a, Show b) => [CSVOption a b] -> String
renderAsCSVFileWithOptions options =
   let
      defaultOpts :: (Show a, Show b) => (Maybe (PlotData a b), String, (a-> String, b->String), Bool)
      defaultOpts = (Nothing, ",", (show, show), False)  -- data, sep, custom show functions of data, show Header)
      (optdata, sep, showFuns, setTitle) = getCSVSettings options defaultOpts

      getCSVSettings :: (Show a, Show b) => [CSVOption a b] -> (Maybe (PlotData a b), String, (a-> String, b->String), Bool) -> (Maybe (PlotData a b), String, (a-> String, b->String), Bool)
      getCSVSettings options preset@(pre_csvdata, pre_sep, pre_showFuns, pres_header) = case options of
         ([]) -> preset
         (x:xs) -> case x of
            CSVShowX showA -> getCSVSettings xs (pre_csvdata, pre_sep, (showA, snd pre_showFuns), pres_header)
            CSVShowY showB -> getCSVSettings xs (pre_csvdata, pre_sep, (fst pre_showFuns, showB), pres_header)
            CSVSeperator sep -> getCSVSettings xs (pre_csvdata, sep, pre_showFuns, pres_header)
            CSVData csvdata -> getCSVSettings xs (Just csvdata, pre_sep, pre_showFuns, pres_header)
            otherwise -> error "Unsupported CSV representation option"

--    csvdataToStrings :: (Show a, Show b) => Maybe (PlotData a b) -> [[String]]
      csvdataToStrings optdata =
         case optdata of
            Just csvdata ->
               case csvdata of
                  PlotData2D csvdatalist -> map (\(x, ys) -> ((fst showFuns) x) : (map (snd showFuns) ys) ) csvdatalist
                  -- tba
                  otherwise -> error "Plotdata not supported by CSV generator"
            Nothing -> [[]]
   in genSepValuesFile sep (csvdataToStrings optdata)

{- | Generate CSV data for a file.  The resulting string can be
written out to disk directly. -}
genSepValuesFile :: String -> [[String]] -> String
genSepValuesFile sepstr inp =
    unlines . map csvline $ inp
    where csvline :: [String] -> String
          csvline l = concat . intersperse sepstr . map csvcells $ l
          csvcells :: String -> String
          csvcells "" = ""
          csvcells c = '"' : convcell c ++ "\""
          convcell :: String -> String
          convcell c = concatMap convchar c
          convchar '"' = "\"\""
          convchar x = [x]

renderAsCSVFile2DWithOptions :: (Show a, Show b) => [CSVOption a b] -> [(a,[b])] -> String
renderAsCSVFile2DWithOptions options csvdata = renderAsCSVFileWithOptions (options ++ [CSVData (PlotData2D csvdata)])

renderAsCSVFile2D :: (Show a, Show b) => [(a,[b])] -> String
renderAsCSVFile2D = renderAsCSVFile2DWithOptions []

renderAsCSVFile1DWithOptions options csvdata = undefined -- showAsCSVFileWithOptions (options ++ [CSVData (PlotData1D csvdata)])
renderAsCSVFile1D = undefined

renderAsCSVFile2DPlainWithOptions options csvdata = undefined -- showAsCSVFileWithOptions (options ++ [CSVData (PlotData2DPlain csvdata)])
renderAsCSVFile2DPlain = undefined


-- Generates Comma Seperated Value (CSV) file
renderCsvFile = genSepValuesFile ","

-- Generates SemiColon seperated value file
renderSsvFile = genSepValuesFile ";"

renderCSVFile = renderCsvFile

renderSSVFile = renderSsvFile 