module Thesis.Grouping.HeliumErrors where

import Grouping
-- import Grouping.Similar
import Thesis.HeliumUniversal.HeliumErrors
import Thesis.HeliumUniversal.LexerError 
import Thesis.HeliumUniversal.ParseError 
import Thesis.HeliumUniversal.ImportError
import Thesis.HeliumUniversal.StaticError
import Thesis.HeliumUniversal.TypeError 

-- here?
groupSimilarErrors_ :: [HeliumErrorFlat] -> [[HeliumErrorFlat]]
groupSimilarErrors_ = groupSimilar

----
-- Specification of similarity of heliumerrors for grouping
----

-- seperate grouping instances
-- so differen grouping methods can be specified
-- and instrumented compiler files are being limited

instance Similar a => Similar (HeliumErrorComplete a) where
   similar (HeliumErrorComplete val1 val2)
           (HeliumErrorComplete val3 val4) = similar val2 val4
{-
instance Similar HeliumError where
   similar (LexerPhError v1) (LexerPhError v2)   = similar v1 v2
   similar (LexerPhError v1) _                   = False
   similar (ParsePhError v1) (ParsePhError v2)   = similar v1 v2
   similar (ParsePhError v1) _                   = False
   similar (ImportPhError v1) (ImportPhError v2) = True -- similar v1 v2
   similar (ImportPhError v1) _                  = False
   similar (StaticPhError v1) (StaticPhError v2) = True -- similar v1 v2
   similar (StaticPhError v1) _                  = False
   similar (TypePhError v1) (TypePhError v2)     = True -- similar v1 v2
   similar (TypePhError v1) _                    = False
-}
instance Similar HeliumErrorFlat where
   similar (LexerPhErrorSingle v1) (LexerPhErrorSingle v2)   = similar v1 v2
   similar (LexerPhErrorSingle v1) _                   = False
   similar (ParsePhErrorSingle v1) (ParsePhErrorSingle v2)   = similar v1 v2
   similar (ParsePhErrorSingle v1) _                   = False
   similar (ImportPhErrorSingle v1) (ImportPhErrorSingle v2) = True -- similar v1 v2
   similar (ImportPhErrorSingle v1) _                  = False
   similar (StaticPhErrorSingle v1) (StaticPhErrorSingle v2) = similar v1 v2
   similar (StaticPhErrorSingle v1) _                  = False
   similar (TypePhErrorSingle v1) (TypePhErrorSingle v2)     = similar v1 v2
   similar (TypePhErrorSingle v1) _                    = False


instance Similar LexerError where
   similar (LexerError sourcePos1 lexerErrorInfo1)
           (LexerError sourcePos2 lexerErrorInfo2)
           = similar lexerErrorInfo1 lexerErrorInfo2

instance Similar LexerErrorInfo where
   similar (UnexpectedChar c) (UnexpectedChar c2)           = True
   similar (NonTerminatedChar _) (NonTerminatedChar _)     = True
   similar (TooManyClose _) (TooManyClose _) = True
   similar (UnexpectedClose _ _ _) (UnexpectedClose _ _ _) = True
   similar (StillOpenAtEOF _) (StillOpenAtEOF _) = True
   similar e1 e2 = e1 == e2


instance Similar ParseError where
   similar (ParseError sourcePos1 msgs1)
           (ParseError sourcePos2 msgs2) = similar msgs1 msgs2

instance Similar Message where
   similar (SysUnExpect str1) (SysUnExpect str2) = True
   similar (UnExpect    str1) (UnExpect    str2) = True
   similar (Expect      str1) (Expect      str2) = True
   similar (Message     str1) (Message     str2) = True
   similar e1 e2                                           = False

instance Similar ResolveError where
   similar (Ambiguous assoc1 name1 assoc2 name2)
           (Ambiguous assoc3 name3 assoc4 name4) = True -- similar

instance Similar StaticError where
   similar (NoFunDef _ _ _) (NoFunDef _ _ _) = True
   similar (Undefined _ _ _ _) (Undefined _ _ _ _) = True
   similar (Duplicated _ _) (Duplicated _ _) = True
   similar (LastStatementNotExpr _) (LastStatementNotExpr _) = True
   similar (WrongFileName _ _ _) (WrongFileName _ _ _) = True
   similar (TypeVarApplication _) (TypeVarApplication _) = True
   similar (ArityMismatch _ _ _ _) (ArityMismatch _ _ _ _) = True
   similar (DefArityMismatch _ _ _) (DefArityMismatch _ _ _) = True
   similar (RecursiveTypeSynonyms _) (RecursiveTypeSynonyms _) = True
   similar (PatternDefinesNoVars _) (PatternDefinesNoVars _) = True
   similar (IntLiteralTooBig _ _) (IntLiteralTooBig _ _) = True
   similar (NegateNeeded _) (NegateNeeded _) = True
   similar (OverloadingDisabled _) (OverloadingDisabled _) = True
   similar (OverloadedRestrPat _) (OverloadedRestrPat _) = True
   similar (WrongOverloadingFlag _) (WrongOverloadingFlag _) = True
   similar (AmbiguousContext _) (AmbiguousContext _) = True
   similar (UnknownClass _) (UnknownClass _) = True
   similar (NonDerivableClass _) (NonDerivableClass _) = True
   similar (CannotDerive _ _) (CannotDerive _ _) = True
   similar (TupleTooBig _) (TupleTooBig _) = True
   similar e1 e2 = False

 
instance Similar TypeError where
   similar v1 v2 = True
   {-
           (Helium1v1LikeTypeError val1)
           (Helium1v1LikeTypeError val2) = similar val1 val2
   -}