module Thesis.Grouping.Logging where

import Thesis.Helium.Logging

import Grouping
import Auxilary -- for compilationTimeDiff, weekOfYear

import Data.List (delete, partition)
import System.Time -- for compilationTimeDiff

----
-- Group functions for the basic (predefined) logging characteristic (for convenience)
----

groupPerUsername_        :: [Logging] -> [[Logging]]
groupPerUsername_        = groupAllByOrd usernameOrd

groupPerPhase_           :: [Logging] -> [[Logging]]
groupPerPhase_           = groupAllByOrd phaseOrd

groupPerHeliumVersion_   :: [Logging] -> [[Logging]]
groupPerHeliumVersion_   = groupAllByOrd heliumVersionOrd

groupPerWeek_            :: [Logging] -> [[Logging]]
groupPerWeek_            = groupAllByOrd weekOrd

groupPerWeekFromSeq_     ::  [Logging] -> [[Logging]]
groupPerWeekFromSeq_     =   groupBy weekEq


groupPerUsername         :: [Logging] -> [(Username, [Logging])]
groupPerUsername         =  enTupleGroup username       . groupAllByOrd usernameOrd

groupPerPhase            :: [Logging] -> [(Phase, [Logging])]
groupPerPhase            =  enTupleGroup phase          . groupAllByOrd phaseOrd

groupPerHeliumVersion    :: [Logging] -> [(HeliumVersion, [Logging])]
groupPerHeliumVersion    =  enTupleGroup heliumVersion  . groupAllByOrd heliumVersionOrd

-- helper function
getWeekLogDate = weekOfYear . logDate

-- groupPerWeek             :: [Logging] -> [((Integer,Int), [Logging])]
groupPerWeek             =  enTupleGroup getWeekLogDate   . groupAllByOrd weekOrd
groupPerWeekFromSeq      =  enTupleGroup getWeekLogDate   . groupBy weekEq


----
-- Ordering predicate functions for predefined logging characteristic
----
heliumVersionOrd             :: Logging -> Logging -> Ordering
heliumVersionOrd  l1 l2      =  compare   (heliumVersion l1)  (heliumVersion l1)
usernameOrd       l1 l2      =  compare   (username l1)       (username l2)
phaseOrd          l1 l2      =  compare   (phase l1)          (phase l2)
weekOrd           l1 l2      =  compare   (getWeekLogDate l1) (getWeekLogDate l2)

weekEq            l1 l2      =  (==)      (getWeekLogDate l1)    (getWeekLogDate l2)

----
-- Coherence functions / predicate functions for coherence combinators
----
sameSourceFile  ::  Logging -> Logging -> Bool
sameSourceFile l1 l2 = (sourceFile $ logPath l1) == (sourceFile . logPath $ l2)

{- 
maxCompilationTimeDiff :: TimeDiff -> Logging -> Logging -> Bool
maxCompilationTimeDiff maxTimeDiff l1 l2 =
   diffClockTimes (toClockTime $ stringToCalTime $ logDate l1) (toClockTime $ stringToCalTime $ logDate l2)
      < maxTimeDiff

groupPerSession_ :: [Logging] -> [[Logging]]
groupPerSession_ =
   groupByCoherence
      (\l1 l2 -> sameSourceFile l1 l2
                     &&
                        maxCompilationTimeDiff (minutesOfTD 10) l1 l2)

newtype Session = Session Int
   deriving (Show, Read, Eq, Ord)

newtype Trace = Trace Int
   deriving (Show, Read, Eq, Ord)

groupPerSession :: [Logging] -> [(Session, [Logging])]
groupPerSession = mapTuple (Session, id) . zip ([0..]::[Int]) . groupPerSession_

groupPerTrace_ :: (Logging -> Logging -> Bool) -> [Logging] -> [[Logging]]
groupPerTrace_ = undefined
-}
-- Test
-- test = groupPerSession [log0, log1, log2, log3, log4, log5]
