module Thesis.Helium.Parser.ResearchLexer where

import Thesis.Helium.Parser.Lexer hiding (lexMultiLineComment, mainLexer, Lexer, lexer, lexOneLineComment)
import Thesis.Helium.Parser.LexerMonad as LexerMonad
import Thesis.Helium.Parser.LexerMessage as LexerMessage
import Thesis.Helium.Parser.LexerToken as LexerToken
-- import Helium1v2.Src.Parsec.Text.ParserCombinators.Parsec.Pos as Text.ParserCombinators.Parsec.Pos
import Text.ParserCombinators.Parsec.Pos
import Thesis.Helium.Parser.Utils as Utils(internalError)

import Control.Monad

----
-- stripped version of Helium1v2 lexer, to get information like de LinesOfDoc
-- should never fail
-- use: linelexer source
----

lexer :: String -> [Char] -> Either LexerError ([SourceSegment], [LexerWarning])
lexer fileName input = runLexerMonad fileName (mainLexer input)

linelexer :: String -> [SourceSegment]
linelexer = either (errorResearchLexer) (filter (not . isEmptyCodeLine) . fst) . (lexer "")

segmentlexer = linelexer

-- all errors should be disabled now
errorResearchLexer err = error $ "Error thrown while lexing, by custom research lexer, this should not have been happend. (LexerMonad) Error: " ++ show err

-- sourceline, or segments (since more then one can be on a line)
data SourceSegment = CodeLine String
                | SingleCommentLine String
                | MultiCommentLine String
                | EmptyLine String
   deriving (Show, Eq, Ord)

-- segementation sim. to sep, code, cmt, empty
sourceSegmentSimilar :: SourceSegment -> SourceSegment -> Bool
sourceSegmentSimilar s1 s2
   | isCodeLine s1 && isCodeLine s2 = True
   | isSingleCommentLine s1 && isSingleCommentLine s2 = True
   | isMultiCommentLine s1 && isMultiCommentLine s2 = True
   | isMultiCommentLine s1 && isSingleCommentLine s2 = True
   | isSingleCommentLine s1 && isMultiCommentLine s2 = True
   | (isEmptyLine s1 && isEmptyLine s2) = True
   | otherwise = False

isEmptyCodeLine :: SourceSegment -> Bool
isEmptyCodeLine (CodeLine line) = emptyline line
isEmptyCodeLine _ = False

isCodeLine (CodeLine str) = True
isCodeLine _ = False

isSingleCommentLine (SingleCommentLine str) = True
isSingleCommentLine _ = False

isMultiCommentLine (MultiCommentLine str) = True
isMultiCommentLine _ = False

isEmptyLine (EmptyLine _) = True
isEmptyLine _ = False

-- stricly only necessary for MultiCommentLine
segmentLines :: SourceSegment -> [String]
segmentLines (CodeLine line)           = lines line
segmentLines (SingleCommentLine line)  = lines line
segmentLines (MultiCommentLine line)   = lines line
segmentLines (EmptyLine line)          | line == "" = [""]   -- lines return [] on input ""
                                       | otherwise  = lines line

sourcelines :: [SourceSegment] -> [String]
sourcelines = concat . map segmentLines

{-

Todo:
- to counter effect of dropping all optional whitespace, that would be come EmptyLine, better is to check, if the whitespace runs untill the \n, then only -> emptyline. example: " asdf {- asdfl -}      sd \n" -> codeline, cmt line, ( empty line (if no white sp is dropped / code line "sd  "). this fixed but if

- to handle inner cmt, extra \n s
- to handle unterminated  cmts, take first string, reeval the rest,
  (what is the effect of the extra \ns from the inner cmts?) might bites each other, since you changed the source, extra lines, maybe rare diff behaviour if -- was in the multiline comment; no

example:
Thesis.Helium.Parser.ResearchLexer> lexer "" " -sdf\nkl\n --o    \n"
Right ([CodeLine " -sdf",CodeLine "kl",CodeLine " ",SingleCommentLine "o    "],[])

(MIND: the CodeLine " ", is filtered out, (or should be empty line, but is really just a single cmt line)

*Thesis.Helium.Parser.ResearchLexer> lexer "" "s fsdf \n   -- cmt \n {- sd \n -- hj-} l;{- lkjh -}"
Right ([CodeLine "s fsdf ",CodeLine "   ",SingleCommentLine " cmt ",CodeLine " ",MultiCommentLine " sd \n -- hj",CodeLine " l;",MultiCommentLine " lkjh "],[])
*Thesis.Helium.Parser.ResearchLexer> lexer "" "s fsdf \n   -- cmt \n {- sd \n -- hj-} l;{- lkjh }"
Right ([CodeLine "s fsdf ",CodeLine "   ",SingleCommentLine " cmt ",CodeLine " ",MultiCommentLine " sd \n -- hj",CodeLine " l;",MultiCommentLine " lkjh }|>UnterminatedComment<|"],[])
*Thesis.Helium.Parser.ResearchLexer> lexer "" "s fsdf \n   -- cmt \n {- sd \n -- hj-} l;{- "
Right ([CodeLine "s fsdf ",CodeLine "   ",SingleCommentLine " cmt ",CodeLine " ",MultiCommentLine " sd \n -- hj",CodeLine " l;",MultiCommentLine " |>UnterminatedComment<|"],[])

use to be:
lexer "" "s fsdf \n   -- cmt \n {- sd \n -- hj-} l; -} "
> Left (LexerError (line 4, column 13) (TooManyClose '}'))

-} {- {-

now:
lexer "" "s fsdf \n   -- cmt \n {- sd \n -- hj-} l; -} "
>Right ([CodeLine "s fsdf ",CodeLine "   ",SingleCommentLine " cmt ",CodeLine " ",MultiCommentLine " sd \n -- hj",CodeLine " l;-} "],[])


{- {-
The remaining spaces of a multiple comment line should not be a EmptyLine

linelexer "s fsdf \n   -- cmt \n {- sd \n -- hj-} l;-} "
[CodeLine "s fsdf ",SingleCommentLine " cmt ",MultiCommentLine " sd \n -- hj",CodeLine " l;-} "]
*Thesis.Helium.Parser.ResearchLexer> linelexer "s fsdf \n   -- cmt \n {- sd \n -- hj-} {-l;-} "
[CodeLine "s fsdf ",SingleCommentLine " cmt ",MultiCommentLine " sd \n -- hj",MultiCommentLine "l;",EmptyLine " "]
*Thesis.Helium.Parser.ResearchLexer> linelexer "s fsdf \n   -- cmt \n {- sd \n -- hj-} {-l;-} "
[CodeLine "s fsdf ",SingleCommentLine " cmt ",MultiCommentLine " sd \n -- hj",MultiCommentLine "l;"]
*Thesis.Helium.Parser.ResearchLexer> linelexer "s fsdf \n   -- cmt \n {- sd \n -- hj-} {-l; {- -} "
[CodeLine "s fsdf ",SingleCommentLine " cmt ",MultiCommentLine " sd \n -- hj",MultiCommentLine "l;  "]
*Thesis.Helium.Parser.ResearchLexer> linelexer "s fsdf \n   -- cmt \n {- sd \n -- hj-} {-l; {- -} "
-}
-}

Mind: inner cmt are also separate lines (only is closing tag is found).
Mind: non terminated cmt, then only one line taken as multiline comment, rest is reevaluated, by the later.

-}



type Lexer = [Char] -> LexerMonad [SourceSegment]  -- we could easily add sourcepos

emptyDoc :: [SourceSegment]
emptyDoc = []


mainLexer :: Lexer
mainLexer [] = do
    return emptyDoc

mainLexer ('-':'-':cs) 
    | not (nextCharSatisfy isSymbol rest) = do
        incPos (2 + length minuses)
        lexOneLineComment rest
    where
        (minuses, rest) = span (== '-') cs
        
mainLexer ('{':'-':cs) = do 
    pos <- getPos 
    incPos 2
    lexMultiLineComment (""++) [pos] 0 cs

mainLexer input@(c:cs)
   | null line || emptyline line = returnLine (EmptyLine line) mainLexer (drop 1 rest)
   | otherwise = lexCodeLine (c:) cs
   where (line, rest) = span (/='\n') input
--     | myIsLower c || c == '_' = -- variable or keyword
--         lexName isLetter LexVar LexKeyword keywords input
--     | myIsSpace c = do
--         when (c == '\t') $ do
--             pos <- getPos
--             lexerWarning TabCharacter pos
--         nextPos c 
--         mainLexer cs        
--     | myIsUpper c = -- constructor
--         lexName isLetter LexCon (internalError "Lexer" "mainLexer" "constructor") [] input
--     | c == ':' = -- constructor operator
--         lexName isSymbol LexConSym LexResConSym reservedConSyms input
--     | isSymbol c = -- variable operator
--         lexName isSymbol LexVarSym LexResVarSym reservedVarSyms input
--     | c `elem` "([{" = do
--         openBracket c
--         returnToken (LexSpecial c) 1 mainLexer cs
--     | c `elem` ")]}" = do
--         closeBracket c
--         returnToken (LexSpecial c) 1 mainLexer cs
--     | c `elem` specialsWithoutBrackets = 
--         returnToken (LexSpecial c) 1 mainLexer cs
--     | myIsDigit c = -- number
--         lexIntFloat input
--     | otherwise = do
--         pos <- getPos
--         lexerError (UnexpectedChar c) pos

lexCodeLine :: (String -> String) -> Lexer
lexCodeLine codeSoFar input =
    case input of 
        '{':'-':cs -> do
            returnLine (CodeLine $ codeSoFar "") mainLexer ('{':'-':cs)
        '-':'-':cs -> do
            returnLine (CodeLine $ codeSoFar "") mainLexer ('-':'-':cs)
        '\n':cs -> do  
            returnLine (CodeLine $ codeSoFar "") mainLexer cs
            -- case is not supposed to happen, un the other hand it can even happen that a "  " becomes a code line, for instance, by "s fsdf \n   -- cmt "  => Right ([CodeLine "s fsdf ",CodeLine "   ",SingleCommentLine " cmt "],[]). Filtering is applied to remove the empty CodeLines.
        c:cs -> do
            lexCodeLine (codeSoFar . (c:)) cs
        [] -> 
            returnLine (CodeLine $ codeSoFar "") mainLexer []

returnLine  :: SourceSegment -> Lexer -> Lexer
returnLine line continue input = do
   lines <- continue input
   return (line:lines)

isWhiteSpace :: Char -> Bool
isWhiteSpace = (flip elem "\t ")

emptyline = and . map isWhiteSpace

-----------------------------------------------------------
-- Comment
-----------------------------------------------------------

lexOneLineComment :: Lexer
lexOneLineComment input =
    case input of
        [] -> mainLexer []
        chars -> do
               returnLine (SingleCommentLine cmt) mainLexer (drop 1 rest)
            where
               (cmt, rest) = span (/= '\n') chars

removeLeadingNL ('\n':cs) = cs
removeLeadingNL cs = cs

lexMultiLineComment :: (String -> String) -> [SourcePos] -> Int -> Lexer
lexMultiLineComment cmtSoFar starts level input =
    case input of 
        '-':'}':cs 
            | level == 0 -> do
                incPos 2
                let rest = (dropWhile isWhiteSpace cs)
                -- todo: to drop optional whitespace, that would be come EmptyLine, better is to check, if the whitespace runs untill the \n, then only -> emptyline.
                -- where (possibleWhiteSpace, newlineAndRest) = span (\='\n') cs --
                returnLine (MultiCommentLine $ cmtSoFar "") mainLexer (removeLeadingNL rest)

            | otherwise ->  do
                incPos 2               -- we could add a extra linefeed, to compensate
                lexMultiLineComment cmtSoFar (tail starts) (level - 1) cs
        '{':'-':cs -> do
            pos <- getPos
            lexerWarning (NestedComment (head starts)) pos
            incPos 2
            lexMultiLineComment (cmtSoFar) (pos:starts) (level+1) cs

        c:cs -> do
            nextPos c
            lexMultiLineComment (cmtSoFar . (c:)) starts level cs
        [] -> do
            returnLine (MultiCommentLine $ cmtSoFar "" ++ "|>UnterminatedComment<|") mainLexer []
--             -- if unterminated, take only one line (conservative), re eval rest
--             let (firstline, rest) = span (/='\n') (cmtSoFar "")
--             returnLine (MultiCommentLine $ firstline ++ "|>UnterminatedComment<|") mainLexer rest

            -- lexerError UnterminatedComment (head starts)
            -- at end-of-file show the most recently opened comment