module Thesis.HeliumInstrumentation.CallHeliumBin where

-- inspired by heliumNT/src/interpreter/Main.hs

import Prelude hiding (catch) -- catch used from Control.Exception
import Control.Exception
import System.Cmd
import System.Exit
import Control.Monad (when)
import System.IO.Unsafe(unsafePerformIO)
import Data.List

import Thesis.Settings(programdir, tmpdir, heliumcompilerslocation)

import Thesis.Helium.Logging
import Thesis.Helium.HeliumInfo
import Thesis.Helium.LoggingPP
import Thesis.HeliumUniversal.Args
import Thesis.HeliumInstrumentation.Message -- was Error
import Auxilary

call = catch

type CompilerCall = IO ()

-- Script to call the different Helium compiler, needed to set environment variables (at least LVMPATH should be set)
mainScript = programdir ++ "/" ++ "helium.pl " -- ++ -heliumsdir ++ show heliumcompilerslocation
versioncall = " -sv "
outputfilename = tmpdir ++ "tmp-output.txt"
options = "-l "

heliumCompiler :: HeliumVersion -> String
heliumCompiler helver
                      | helver == helium1v0   = error "Helium 1.0 is does not support instrumentation"
                      | helver == helium1v1   = mainScript ++ " " ++ versioncall ++ " " ++ heliumVersionNumber helium1v1 ++ " " ++ options

                      -- Using 1.1
                      | helver == helium1v1A  = mainScript ++ " " ++ versioncall ++ " " ++ heliumVersionNumber helium1v1 ++ " " ++ options
                      -- Using 1.1
                      | helver == helium1v1v1 = mainScript ++ " " ++ versioncall ++ " " ++ heliumVersionNumber helium1v1 ++ " " ++ options
                      -- Using 1.1
                      | helver == helium1v1v2 = mainScript ++ " " ++ versioncall ++ " " ++ heliumVersionNumber helium1v1 ++ " " ++ options
                      | helver == helium1v2   = undefined
                      | helver == helium1v2v1 = undefined
                      | helver == helium1v2v2 = undefined
                      | helver == helium1v5   = undefined
                      | helver == helium1v5v1 = undefined
                      | otherwise = {- HACK!!-}
                           mainScript ++ " " ++ versioncall ++ " " ++ heliumVersionNumber helium1v1 ++ " " ++ options
                      -- error ("Called compiler " ++ (show helver) ++ " is not supported for instrumentation.")

-- uses tmp files, better to replace with runProcess in ghc 6.4
doCmd :: String -> String -> IO (Bool, String)
doCmd cmd output =
   do
      exitCode <- system (cmd ++ "> " ++ output)
      contents <- readFile output
                  `catch` (\exception -> error ("Unable to read from file \"" ++ output ++ "\" (during logging compilation phase)"))
      return (exitCode == ExitSuccess, contents)

instrumentationErrorUnknown context log =
   error $ ("Unknow instrumentation error when retrieving metric " ++ context ++ ". Did you instrument the helium compiler correctly? Error cast when analysing logging " ++ show log)

instrumentationErrorNoOutput context log =
   error $ ("Helium call succeeded when retrieving metric " ++ context ++ " but no information was returned. Did you select the right options (not all metrics are applicable). (logging being analysed: " ++ show log++")")

doHeliumBinCallAnalysis :: HeliumVersion -> Logging -> [AnalysisOption] -> IO (Bool,String)
doHeliumBinCallAnalysis helver log opts = (heliumCall helver log) 
   where
      heliumCall helver log =
         do (success, instrumentationresult) <- doCmd ((heliumCompiler helver) ++ (ppPath $ logPath log) ++ (mapToCmdLine opts)) outputfilename
            return (success, filterResult instrumentationresult)
      instrumentationLines = isPrefixOf instrumentationMessage   -- optimalisation, msg can only appear at start of line
      filterResult = unlines . map (dropPrefix instrumentationMessage) . filter instrumentationLines . lines

doHeliumAnalysisIO :: HeliumVersion -> Logging -> [AnalysisOption] -> String -> IO String
doHeliumAnalysisIO helver log opts context = 
   do
      (success, instrumentationresult) <- doHeliumBinCallAnalysis helver log opts
      if (not $ success)
         then
            {- do -}evaluate $ instrumentationErrorUnknown context log
--               return 
         else
            if (length instrumentationresult == 0)
               then
                  {-do-} evaluate $ instrumentationErrorNoOutput context log
                     --return ()
               else
                  return instrumentationresult

doHeliumAnalysis :: HeliumVersion -> Logging -> [AnalysisOption] -> String -> String
doHeliumAnalysis helver log opts context =
   unsafePerformIO $ doHeliumAnalysisIO helver log opts context
   
{-
doHeliumAnalysisMaybe :: HeliumVersion -> Logging -> [AnalysisOption] -> String -> Maybe String
doHeliumAnalysisMaybe  helver log opts context = unsafePerformIO $ 
   do
      (success, instrumentationresult) <- doHeliumBinCallAnalysis helver log opts
      if success
         then
            return $ Just instrumentationresult
         else
            return Nothing
-}

tryConversion :: Read b => String -> String -> b
tryConversion input errorMsg = strictUnsafePerformIO $
   let tryRead a = evaluate (read a)
   in
      do catch
            (tryRead input)
            (\exception -> error $ "\nError while parsing helium result for metric " ++ errorMsg ++ " (error: " ++ show exception ++ " Input text: " ++ input ++ ")")
      

           
{-
      case x of
         Nothing       -> ioError $ userError ("metric (m1_3_2) not applyable to this logging (log: " ++ show log ++ ")")
         Just retrievedData -> do --putStr retrievedData
                                  return (read retrievedData)
-}
