{-# OPTIONS -cpp -DHELIUM_VERSION=1100 #-}
module HeliumInstrumentation.Hierar.DataTypeConversion where

import Data.Either
import Data.Typeable
import Auxilary

import HeliumInstrumentation.Hierar.Universable as UniversalDatatypes
import qualified Thesis.HeliumUniversal.HeliumErrors as UniversalDatatypes
import qualified HeliumUniversal.BasicTypes as UniversalDatatypes
import qualified Thesis.HeliumUniversal.LexerError as UniversalLexerError
import qualified Thesis.HeliumUniversal.ParseError as UniversalParseError
import qualified Thesis.HeliumUniversal.ImportError as UniversalImportError
import qualified Thesis.HeliumUniversal.StaticError as UniversalStaticError
import qualified Thesis.HeliumUniversal.TypeError as UniversalTypeError

-- lexer token
import qualified Thesis.HeliumUniversal.LexerToken as UniversalDatatypes

#if (HELIUM_VERSION >= 1100 && HELIUM_VERSION < 1200 ) /* Helium 1.1 and Helium 1.1.X */
import qualified Thesis.HeliumUniversal.TypeError.Helium1v1TypeError as UniversalTypeError
#elif (HELIUM_VERSION >= 1200 && HELIUM_VERSION < 1500 ) /* Helium 1.2 and Helium 1.2.X*/
import qualified Thesis.HeliumUniversal.TypeError.Helium1v2TypeError as UniversalTypeError
#elif (HELIUM_VERSION >= 1500 && HELIUM_VERSION < 1600 ) /* Helium 1.5 and Helium 1.5.X */
import qualified Thesis.HeliumUniversal.TypeError.Helium1v5TypeError as UniversalTypeError
#endif
 
-- Compiler specific imports
#if (HELIUM_VERSION == 1100) /* Helium 1.1 */
import Helium1v1.Src.Parsec.ParsecPos as HeliumSpecific (SourcePos(..))
import Helium1v1.Src.HeliumNT.Src.Parser.LexerMessage as HeliumSpecific
import Helium1v1.Src.Parsec.ParsecError as HeliumSpecific (ParseError(..), Message (..))
import Helium1v1.Src.HeliumNT.Src.Parser.ResolveOperators as HeliumSpecific -- (ResolveError (..), )
import Helium1v1.Src.HeliumNT.Src.Parser.OperatorTable as HeliumSpecific (Assoc (..))
import Helium1v1.Src.HeliumNT.Src.Syntax.UHA_Syntax as HeliumSpecific (Name (..), Names, Position (..), Range(..))
import Helium1v1.Src.HeliumNT.Src.Staticanalysis.Messages.StaticErrors as HeliumSpecific(Error (..)) -- should export own module, and reexport Entity (..) (from Messages)

-- Type error related imports
import Helium1v1.Src.HeliumNT.Src.Staticanalysis.Messages.TypeErrors as HeliumSpecific
import Helium1v1.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as HeliumSpecific hiding (Message (..)) -- does import (MessageLine (..), MessageBlocks, MessageBlock(..))
import qualified Helium1v1.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as Helium1v1TypeError (Message (..)) -- to avoid name clash with Message from other datatype
import Helium1v1.Src.HeliumNT.Src.Staticanalysis.Types.TypeRepresentation as HeliumSpecific (Tp (..), TpScheme (..))
import Helium1v1.Src.HeliumNT.Src.Utils.OneLiner as HeliumSpecific (Tree(..))

import Helium1v1.Src.HeliumNT.Src.Parser.LexerToken as HeliumSpecific


#elif (HELIUM_VERSION == 1101) /* Helium 1.1A */
-- ParsecPos  import
import Helium1v1A.Src.Parsec.ParsecPos as HeliumSpecific (SourcePos(..))
-- Lexical Error import(s)
import Helium1v1A.Src.HeliumNT.Src.Parser.LexerMessage as HeliumSpecific
-- Parse Error import(s)
import Helium1v1A.Src.Parsec.ParsecError as HeliumSpecific (ParseError(..), Message (..))

-- Import/resolve Operator Error import(s)
import Helium1v1A.Src.HeliumNT.Src.Parser.ResolveOperators as HeliumSpecific -- (ResolveError (..), )
import Helium1v1A.Src.HeliumNT.Src.Parser.OperatorTable as HeliumSpecific (Assoc (..))
import Helium1v1A.Src.HeliumNT.Src.Syntax.UHA_Syntax as HeliumSpecific (Name (..), Names, Position (..), Range(..))

-- Static Error import(s)
import Helium1v1A.Src.HeliumNT.Src.Staticanalysis.Messages.StaticErrors as HeliumSpecific(Error (..)) -- should export own module, and reexport Entity (..) (from Messages)

-- Type error related imports
import Helium1v1A.Src.HeliumNT.Src.Staticanalysis.Messages.TypeErrors as HeliumSpecific
import Helium1v1A.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as HeliumSpecific hiding (Message (..)) -- does import (MessageLine (..), MessageBlocks, MessageBlock(..))
import qualified Helium1v1A.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as Helium1v1TypeError (Message (..)) -- to avoid name clash with Message from other datatype
import Helium1v1A.Src.HeliumNT.Src.Staticanalysis.Types.TypeRepresentation as HeliumSpecific (Tp (..), TpScheme (..))
import Helium1v1A.Src.HeliumNT.Src.Utils.OneLiner as HeliumSpecific (Tree(..))

import Helium1v1A.Src.HeliumNT.Src.Parser.LexerToken as HeliumSpecific

#elif (HELIUM_VERSION == 1110) /* Helium 1.1.1 */
import Helium1v1v1.Src.Parsec.ParsecPos as HeliumSpecific (SourcePos(..))
import Helium1v1v1.Src.HeliumNT.Src.Parser.LexerMessage as HeliumSpecific
import Helium1v1v1.Src.Parsec.ParsecError as HeliumSpecific (ParseError(..), Message (..))
import Helium1v1v1.Src.HeliumNT.Src.Parser.ResolveOperators as HeliumSpecific -- (ResolveError (..), )
import Helium1v1v1.Src.HeliumNT.Src.Parser.OperatorTable as HeliumSpecific (Assoc (..))
import Helium1v1v1.Src.HeliumNT.Src.Syntax.UHA_Syntax as HeliumSpecific (Name (..), Names, Position (..), Range(..))
import Helium1v1v1.Src.HeliumNT.Src.Staticanalysis.Messages.StaticErrors as HeliumSpecific(Error (..)) -- should export own module, and reexport Entity (..) (from Messages)

-- Type error related imports
import Helium1v1v1.Src.HeliumNT.Src.Staticanalysis.Messages.TypeErrors as HeliumSpecific
import Helium1v1v1.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as HeliumSpecific hiding (Message (..)) -- does import (MessageLine (..), MessageBlocks, MessageBlock(..))
import qualified Helium1v1v1.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as Helium1v1TypeError (Message (..)) -- to avoid name clash with Message from other datatype
import Helium1v1v1.Src.HeliumNT.Src.Staticanalysis.Types.TypeRepresentation as HeliumSpecific (Tp (..), TpScheme (..))
import Helium1v1v1.Src.HeliumNT.Src.Utils.OneLiner as HeliumSpecific (Tree(..))

import Helium1v1v1.Src.HeliumNT.Src.Parser.LexerToken as HeliumSpecific

#elif (HELIUM_VERSION == 1120) /* Helium 1.1.2 */
import Helium1v1v2.Src.Parsec.ParsecPos as HeliumSpecific (SourcePos(..))
import Helium1v1v2.Src.HeliumNT.Src.Parser.LexerMessage as HeliumSpecific
import Helium1v1v2.Src.Parsec.ParsecError as HeliumSpecific (ParseError(..), Message (..))
import Helium1v1v2.Src.HeliumNT.Src.Parser.ResolveOperators as HeliumSpecific -- (ResolveError (..), )
import Helium1v1v2.Src.HeliumNT.Src.Parser.OperatorTable as HeliumSpecific (Assoc (..))
import Helium1v1v2.Src.HeliumNT.Src.Syntax.UHA_Syntax as HeliumSpecific (Name (..), Names, Position (..), Range(..))
import Helium1v1v2.Src.HeliumNT.Src.Staticanalysis.Messages.StaticErrors as HeliumSpecific(Error (..)) -- should export own module, and reexport Entity (..) (from Messages)

-- Type error related imports
import Helium1v1v2.Src.HeliumNT.Src.Staticanalysis.Messages.TypeErrors as HeliumSpecific
import Helium1v1v2.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as HeliumSpecific hiding (Message (..)) -- does import (MessageLine (..), MessageBlocks, MessageBlock(..))
import qualified Helium1v1v2.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as Helium1v1TypeError (Message (..)) -- to avoid name clash with Message from other datatype
import Helium1v1v2.Src.HeliumNT.Src.Staticanalysis.Types.TypeRepresentation as HeliumSpecific (Tp (..), TpScheme (..))
import Helium1v1v2.Src.HeliumNT.Src.Utils.OneLiner as HeliumSpecific (Tree(..))

import Helium1v1v2.Src.HeliumNT.Src.Parser.LexerToken as HeliumSpecific
#endif

-- Helper functions
toUniversalError datatype = error ("Error converting to universal " ++ datatype ++ " datatype")

-------------------------------------------
-- Universable Overall used (basic) datatypes
-------------------------------------------

instance Universable UniversalDatatypes.SourcePos SourcePos where
   fromUniversal (UniversalDatatypes.SourcePos sourceName line column) = (SourcePos sourceName line column)
   toUniversal (SourcePos sourceName line column) = (UniversalDatatypes.SourcePos sourceName line column)

instance Universable UniversalDatatypes.Name Name where
   toUniversal name =
      case name of
         (Name_Identifier range strings str) -> (UniversalDatatypes.Name_Identifier (toUniversal range) strings str)
         (Name_Operator  range strings str) -> (UniversalDatatypes.Name_Operator (toUniversal range) strings str)
         (Name_Special range strings str) -> (UniversalDatatypes.Name_Special (toUniversal range) strings str)
   fromUniversal _ = undefined

instance Universable UniversalDatatypes.Names Names where
   toUniversal names = map toUniversal names
   fromUniversal _ = undefined

instance Universable UniversalDatatypes.Range Range where
   toUniversal range =
      case range of
         (Range_Range pos1 pos2) -> UniversalDatatypes.Range_Range (toUniversal pos1) (toUniversal pos2)
         otherwise -> toUniversalError "Range"
   fromUniversal _ = undefined

instance Universable UniversalDatatypes.Position Position where
   toUniversal pos =
      case pos of
         (Position_Position str pos1 pos2) -> UniversalDatatypes.Position_Position str pos1 pos2
         Position_Unknown -> UniversalDatatypes.Position_Unknown
         otherwise -> toUniversalError "Position"
   fromUniversal _ = undefined

-------------------------------------------
-- Universable Lexer Error datatype
-------------------------------------------

instance Universable UniversalDatatypes.LexerError LexerError where
   toUniversal (LexerError sourcePos lexerErrorInfo) = UniversalDatatypes.LexerError (toUniversal sourcePos) (toUniversal lexerErrorInfo)
   fromUniversal _ = undefined

instance Universable UniversalLexerError.LexerErrorInfo LexerErrorInfo where
   fromUniversal = undefined
   toUniversal universalLexerErrorInfo =
      case universalLexerErrorInfo of
         UnterminatedComment -> UniversalLexerError.UnterminatedComment
         MissingExponentDigits -> UniversalLexerError.MissingExponentDigits
         UnexpectedChar char -> UniversalLexerError.UnexpectedChar char
         IllegalEscapeInChar -> UniversalLexerError.IllegalEscapeInChar
         EmptyChar -> UniversalLexerError.EmptyChar
         IllegalCharInChar -> UniversalLexerError.IllegalCharInChar
         NonTerminatedChar -> UniversalLexerError.NonTerminatedChar Nothing -- the maybe string is added since Helium 1.2
         EOFInChar -> UniversalLexerError.EOFInChar

         EOFInString -> UniversalLexerError.EOFInString
         IllegalEscapeInString -> UniversalLexerError.IllegalEscapeInString
         NewLineInString -> UniversalLexerError.NewLineInString
         IllegalCharInString -> UniversalLexerError.IllegalCharInString

         TooManyClose char -> UniversalLexerError.TooManyClose char
        -- In UnexpectedClose, first char is the closing bracket we see,
        -- second char is theHelium1v5v1.Src.HeliumNT.Src.Syntax.UHA_Syntax closing bracket we would like to see first
        -- e.g. [(1,3]  =>  UnexpectedClose ']' ... ')'
         UnexpectedClose char1 sourcePos char2  -> UniversalLexerError.UnexpectedClose char1 (toUniversal sourcePos) char2
         StillOpenAtEOF list {-[(sourcePos, char)] -} -> UniversalLexerError.StillOpenAtEOF (mapTuple (toUniversal, id) list)
         otherwise             -> toUniversalError "LexerErrorInfo"


-------------------------------------------
-- Universable Parse Error datatype
-------------------------------------------

instance Universable UniversalDatatypes.ParseError ParseError where
   toUniversal (ParseError sourcePos messages) = UniversalDatatypes.ParseError (toUniversal sourcePos) (map toUniversal messages)
   fromUniversal _ = undefined

instance Universable UniversalParseError.Message Message where
   toUniversal msg =
      case msg of
         SysUnExpect val -> UniversalParseError.SysUnExpect val
         UnExpect val -> UniversalParseError.UnExpect val
         Expect val -> UniversalParseError.Expect val
         Message val -> UniversalParseError.Message val
         otherwise -> toUniversalError "Message"
   fromUniversal _ = undefined

-------------------------------------------
-- Universable Import / Resolve Error datatype
-------------------------------------------

instance Universable UniversalDatatypes.ResolveError ResolveError where
   toUniversal (Ambiguous assoc1 name1 assoc2 name2) = UniversalDatatypes.Ambiguous (toUniversal assoc1) (toUniversal name1) (toUniversal assoc2) (toUniversal name2)
   fromUniversal _ = undefined

instance Universable UniversalImportError.Assoc Assoc where
   toUniversal assoc =
      case assoc of
         AssocNone -> UniversalImportError.AssocNone
         AssocLeft -> UniversalImportError.AssocLeft
         AssocRight -> UniversalImportError.AssocRight
   fromUniversal _ = undefined

-- other datatypes unversable defined with HeliumBasicTypes on top

-------------------------------------------
-- Universable Static Error datatype
-------------------------------------------

instance Universable UniversalDatatypes.StaticError {-Static-}Error where
   -- toUniversal staticError = genericUniversal staticError-- requires datatypes to be Typeable: Error, Entity, Range, Name etc
   
   toUniversal staticError =
      case staticError of
         NoFunDef entity name names -> UniversalStaticError.NoFunDef (toUniversal entity) (toUniversal name) (toUniversal names)
         Undefined entity name names strs -> UniversalStaticError.Undefined (toUniversal entity) (toUniversal name) (toUniversal names) strs
         Duplicated entity names -> UniversalStaticError.Duplicated (toUniversal entity) (toUniversal names)
         LastStatementNotExpr range -> UniversalStaticError.LastStatementNotExpr (toUniversal range)
         WrongFileName str1 str2 range -> UniversalStaticError.WrongFileName str1 str2 (toUniversal range)
         TypeVarApplication name -> UniversalStaticError.TypeVarApplication (toUniversal name)
         ArityMismatch entity name nop nofp -> UniversalStaticError.ArityMismatch (toUniversal entity) (toUniversal name) nop nofp
         DefArityMismatch name maybeval range -> UniversalStaticError.DefArityMismatch (toUniversal name) maybeval (toUniversal range)
         RecursiveTypeSynonyms names -> UniversalStaticError.RecursiveTypeSynonyms (toUniversal names)
         PatternDefinesNoVars range -> UniversalStaticError.PatternDefinesNoVars (toUniversal range)
    
        -- IntLiteralTooBig Range String              -- pvk Added in helium 1.1.2
         -- NegateNeeded Range                         -- pvk Added in helium 1.2, removed in helium 1.5
         -- OverloadingDisabled Range Static                 -- pvk Added in helium 1.2
         -- OverloadedRestrPat Name                    -- pvk added in helium 1.5
         -- WrongOverloadingFlag Bool{- flag? -}       -- pvk added in helium 1.5
         -- AmbiguousContext Name                      -- pvk Added in helium 1.2
         -- UnknownClass Name                          -- pvk Added in helium 1.2
         -- NonDerivableClass Name                     -- pvk added in helium 1.5
         -- CannotDerive Name Tps                      -- pvk added in helium 1.5, (Tps not available in helium 1.1.2, only from 1.2 and upwards
         -- TupleTooBig Range                          -- pvk added in helium 1.5
      
         otherwise -> toUniversalError "StaticError"

   fromUniversal _ = undefined

instance Universable UniversalStaticError.Entity Entity where
   toUniversal entity =
      case entity of
         TypeSignature -> UniversalStaticError.TypeSignature
         TypeVariable -> UniversalStaticError.TypeVariable
         TypeConstructor -> UniversalStaticError.TypeConstructor
         Definition -> UniversalStaticError.Definition
         Constructor -> UniversalStaticError.Constructor
         Variable -> UniversalStaticError.Variable
         Import -> UniversalStaticError.Import
         ExportVariable -> UniversalStaticError.ExportVariable
         ExportModule -> UniversalStaticError.ExportModule
         ExportConstructor -> UniversalStaticError.ExportConstructor
         ExportTypeConstructor -> UniversalStaticError.ExportTypeConstructor
         Fixity -> UniversalStaticError.Fixity
         otherwise -> toUniversalError "Entity"
   fromUniversal _ = undefined

-------------------------------------------
-- Universable Type Error datatype, helium1.1 type error only
-------------------------------------------
instance Universable UniversalDatatypes.TypeError TypeError where
   toUniversal error =
      case error of
         TypeError range messageBlock typeErrorTable typeErrorInfos -> UniversalTypeError.Helium1v1LikeTypeError (
            UniversalTypeError.TypeError (toUniversal range) (toUniversal messageBlock) (toUniversal typeErrorTable) (toUniversal typeErrorInfos)
            )
         CustomTypeError ranges message -> UniversalTypeError.Helium1v1LikeTypeError (
            UniversalTypeError.CustomTypeError (map toUniversal ranges) (toUniversal message)
            )
         otherwise -> toUniversalError "TypeError"
   fromUniversal = undefined

instance Universable UniversalTypeError.MessageBlock MessageBlock where
   toUniversal msgblock =
      case msgblock of
         MessageString string -> UniversalTypeError.MessageString string
         MessageRange range -> UniversalTypeError.MessageRange (toUniversal range)
         MessageType tp -> UniversalTypeError.MessageType (toUniversal tp)
         MessageTypeScheme tpScheme -> UniversalTypeError.MessageTypeScheme (toUniversal tpScheme)
         MessageInfoLink string -> UniversalTypeError.MessageInfoLink string
         MessageOneLineTree tree -> UniversalTypeError.MessageOneLineTree (toUniversal tree)
         MessageCompose messageBlocks -> UniversalTypeError.MessageCompose (map toUniversal messageBlocks)
         otherwise -> toUniversalError "MessageBlock"
   fromUniversal = undefined

instance Universable UniversalTypeError.Tp Tp where
   toUniversal tp =
      case tp of
         TVar int -> UniversalTypeError.TVar int
         TCon string -> UniversalTypeError.TCon string
         TApp tp1 tp2 -> UniversalTypeError.TApp (toUniversal tp1) (toUniversal tp2)
         otherwise -> toUniversalError "Tp"
   fromUniversal = undefined

instance Universable UniversalTypeError.TpScheme TpScheme where
   toUniversal tpscheme =
      case (tpscheme) of
         Scheme ints list tp -> UniversalTypeError.Scheme ints list (toUniversal tp)
         otherwise -> toUniversalError "Scheme"
   fromUniversal = undefined

instance Universable UniversalTypeError.Tree Tree where
   toUniversal tree =
      case tree of
         Node trees -> UniversalTypeError.Node (map toUniversal trees)
         Text string -> UniversalTypeError.Text string
         otherwise -> toUniversalError "Tree"
   fromUniversal = undefined
   
instance Universable UniversalTypeError.TypeErrorTable TypeErrorTable where
   toUniversal typeET =
      case typeET of  -- UnificationErrorTable
         (UnificationErrorTable lst {- [(string, messageBlock)] -} either1 either2) -> UniversalTypeError.UnificationErrorTable (mapTuple (id, toUniversal) lst) (eitherToUniversal either1) (eitherToUniversal either2)
         NotGeneralEnoughTable tree tpScheme1 tpScheme2 ->
            UniversalTypeError.NotGeneralEnoughTable (toUniversal tree) (toUniversal tpScheme1) (toUniversal tpScheme2)
         otherwise -> (toUniversalError "TypeErrorTable")
   fromUniversal = undefined

eitherToUniversal :: (Universable c a, Universable d b) => Either a b -> Either c d
eitherToUniversal eitherval = -- either toUniversal toUniversal
   case eitherval of
      Left val -> Left (toUniversal val)
      Right val -> Right (toUniversal val)

instance Universable UniversalTypeError.TypeErrorInfos TypeErrorInfos where
   toUniversal = map toUniversal
   fromUniversal = undefined

instance Universable UniversalTypeError.TypeErrorInfo TypeErrorInfo where
   toUniversal info =
      case info of
         TypeErrorHint string messageBlock -> UniversalTypeError.TypeErrorHint string (toUniversal messageBlock)
         IsFolkloreTypeError -> UniversalTypeError.IsFolkloreTypeError
         HasDocumentationLink string -> UniversalTypeError.HasDocumentationLink string
         otherwise -> toUniversalError "TypeErrorInfo"
   fromUniversal = undefined

instance Universable UniversalTypeError.Message Helium1v1TypeError.Message where
   toUniversal = map toUniversal
   fromUniversal = undefined

instance Universable UniversalTypeError.MessageLine MessageLine where
   toUniversal msgline =
      case msgline of
         MessageOneLiner messageBlock -> UniversalTypeError.MessageOneLiner (toUniversal messageBlock)
         MessageTable lst {-[(messageBlock, messageBlock2)]-} -> UniversalTypeError.MessageTable (mapTuple (toUniversal,toUniversal) lst)
         MessageHints string messageBlocks -> UniversalTypeError.MessageHints string (map toUniversal messageBlocks)
         otherwise -> toUniversalError "MessageLine"
   fromUniversal = undefined

-------------------------------------------
-- Universable Lexer Token datatype
-------------------------------------------
instance Universable UniversalDatatypes.Token Token where
   toUniversal (sourcePos,lexeme) = (toUniversal sourcePos, toUniversal lexeme)
   fromUniversal = undefined

instance Universable UniversalDatatypes.Lexeme Lexeme where
   toUniversal token = -- undefined
      case token of
         LexChar            string -> UniversalDatatypes.LexChar            string
         LexString          string -> UniversalDatatypes.LexString          string
         LexInt             string -> UniversalDatatypes.LexInt             string
         LexFloat           string -> UniversalDatatypes.LexFloat           string
         
         LexVar             string -> UniversalDatatypes.LexVar             string
         LexVarSym          string -> UniversalDatatypes.LexVarSym          string
         LexCon             string -> UniversalDatatypes.LexCon             string
         LexConSym          string -> UniversalDatatypes.LexConSym          string
         
         LexKeyword         string -> UniversalDatatypes.LexKeyword         string
         LexResVarSym       string -> UniversalDatatypes.LexResVarSym       string
         LexResConSym       string -> UniversalDatatypes.LexResConSym       string
         LexSpecial         char -> UniversalDatatypes.LexSpecial         char 
      
         LexInsertedOpenBrace   -> UniversalDatatypes.LexInsertedOpenBrace
         LexInsertedCloseBrace  -> UniversalDatatypes.LexInsertedCloseBrace
         LexInsertedSemicolon   -> UniversalDatatypes.LexInsertedSemicolon
      
         LexEOF  -> UniversalDatatypes.LexEOF
   fromUniversal = undefined