module HeliumInstrumentation.Hierar.Utils where

import HeliumInstrumentation.Hierar.Universable
import HeliumUniversal.Args as UniversalDatatypes
import qualified Thesis.HeliumUniversal.HeliumErrors as UniversalDatatypes
import qualified Thesis.HeliumUniversal.LexerToken as UniversalDatatypes
import Data.IORef 
import GHC.IOBase (unsafePerformIO)
import Control.Exception --(NoMethodError )
import System.Exit

-- for debugging
import Auxilary(traceVal)

-- compiler specific :(
import HeliumInstrumentation.Hierar.DataTypeConversion
import Helium1v1.Src.HeliumNT.Src.Staticanalysis.Messages.Messages as Messages --(HasMessage, sortMessages)
import Helium1v1.Src.HeliumNT.Src.Staticanalysis.Messages.HeliumMessages as HeliumMessages
 --(sortAndShowMessages)
import Helium1v1.Src.HeliumNT.Src.Main.Args as HeliumArgs


noAnalysis :: [Option] -> Bool
noAnalysis options = not . or $ map (flip elem options) analysisOptions

instrumentedErrorMessage :: String
instrumentedErrorMessage = "Analysis subject retrieved... "

justExit :: IO a
justExit = exitWith (ExitFailure 1)


-- pvk, old
{-
-accolade-# NOINLINE analysisSubject #-accolade
analysisSubject :: IORef (Maybe a)
analysisSubject = unsafePerformIO (newIORef Nothing)


-- hook functions, with return to code
--analyseLexErrors :: Helium1v5.Src.HeliumNT.Src.Parser.LexerMessage.LexerError -> IO a --Maybe LexerError
analyseLexErrorsIORef lexError =
   do --putStr "lexerror: \n" noAnalysis--showErrorsAndExit [lexError] 1 options
      writeIORef analysisSubject (Just lexError)
      throwIO (NoMethodError (instrumentedErrorMessage)
-}

-- hook functions, with return to code
-- analyseLexErrors :: Helium1v5.Src.HeliumNT.Src.Parser.LexerMessage.LexerError -> IO a --Maybe LexerError
--analyseLexErrors :: (HasMessage a, Universable UniversalDatatypes.LexerError a) => [a] -> IO a1
analyseLexErrors errors =
   do --putStr ("lexererror: \"" ++ show uniErrors ++ "\"")
      throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
      --return ()
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.LexerError

analyseParseErrors errors = 
   do throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.ParseError

-- importing/resolving operators
analyseImportErrors errors = 
   do throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.ResolveError

-- static
analyseStaticErrors errors = 
   do throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.StaticError
      
-- type
analyseTypeErrors errors = 
   do throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.TypeError

analyseWarnings warnings = undefined

analyseAST ast = undefined

analyseTokens tokens tokensWithLayout =
   do throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniTokenResult)))
   where
      uniTokenResult = UniversalDatatypes.TokenAnalysisResult (show tokensWithLayout) (map uniToken tokens) (map uniToken tokensWithLayout)
      uniToken token = (toUniversal token)::UniversalDatatypes.Token
