module Thesis.HeliumInstrumentation.Utils where

import Thesis.HeliumInstrumentation.Universable
import Thesis.HeliumInstrumentation.Message --Error
import Thesis.HeliumUniversal.Args as UniversalDatatypes
import qualified Thesis.HeliumUniversal.HeliumErrors as UniversalDatatypes
import qualified Thesis.HeliumUniversal.LexerToken as UniversalDatatypes
import Data.IORef 
import GHC.IOBase (unsafePerformIO)
import Control.Exception --(NoMethodError )
import System.Exit

-- for debugging
import Auxilary(traceVal)

-- compiler specific :(
import Thesis.HeliumInstrumentation.DataTypeConversion
import Messages --(HasMessage, sortMessages)
import HeliumMessages
 --(sortAndShowMessages)
import Args -- as HeliumArgs


noAnalysis :: [Option] -> Bool
noAnalysis options = not . or $ map (flip elem options) analysisOptions

instrumentationExit :: IO a
instrumentationExit = exitWith ExitSuccess -- (ExitFailure 2)

justExit = exitWith (ExitFailure 1)

showAndExit val =
   do putStr (instrumentationMessage ++ show val)
      instrumentationExit

-- pvk, old
{-
-accolade-# NOINLINE analysisSubject #-accolade
analysisSubject :: IORef (Maybe a)
analysisSubject = unsafePerformIO (newIORef Nothing)


-- hook functions, with return to code
--analyseLexErrors :: Helium1v5.Src.HeliumNT.Src.Parser.LexerMessage.LexerError -> IO a --Maybe LexerError
analyseLexErrorsIORef lexError =
   do --putStr "lexerror: \n" noAnalysis--showErrorsAndExit [lexError] 1 options
      writeIORef analysisSubject (Just lexError)
      throwIO (NoMethodError (instrumentedErrorMessage)
-}

-- hook functions, with return to code
-- analyseLexErrors :: Helium1v5.Src.HeliumNT.Src.Parser.LexerMessage.LexerError -> IO a --Maybe LexerError
--analyseLexErrors :: (HasMessage a, Universable UniversalDatatypes.LexerError a) => [a] -> IO a1

{-
old way:

analyseLexErrors errors =
   do -- putStr ("lexererror: \"" ++ show uniErrors ++ "\"")
      throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
      return ()
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.LexerError
-}

analyseLexErrors errors =
   do showAndExit uniErrors 
      -- putStr ("lexererror: \"" ++ show uniErrors ++ "\"")
      -- throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
      -- return ()
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.LexerError

analyseParseErrors errors = 
   do showAndExit uniErrors
         {- showAndExit
            UniversalDatatypes.toHeliumErrorData
               (map
                  (\error -> UniversalDatatypes.HeliumErrorComplete
                                 (showMessage error)
                                 ((toUniversal error)::UniversalDatatypes.ParseError)
                             )
                  errors)

         -}
      --throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.ParseError

-- importing/resolving operators
analyseImportErrors errors = 
   do showAndExit uniErrors
      --throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.ResolveError

-- static
analyseStaticErrors errors = 
   do showAndExit uniErrors
      -- throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.StaticError
      
-- type
analyseTypeErrors errors = 
   do showAndExit uniErrors
      --throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniErrors)))
   where
      uniErrors = UniversalDatatypes.toHeliumErrorData (map heliumError errors)
      heliumError error = UniversalDatatypes.HeliumErrorComplete (showMessage error) (phaseError error)
      phaseError error = (toUniversal error)::UniversalDatatypes.TypeError

analyseWarnings warnings = undefined

analyseAST ast = undefined

analyseTokens tokens tokensWithLayout =
   do showAndExit uniTokenResult
      --throwIO (NoMethodError (instrumentedErrorMessage ++ (show uniTokenResult)))
   where
      uniTokenResult = UniversalDatatypes.TokenAnalysisResult (show tokensWithLayout) (map uniToken tokens) (map uniToken tokensWithLayout)
      uniToken token = (toUniversal token)::UniversalDatatypes.Token
