module Thesis.HeliumUniversal.HeliumErrors (
     module Thesis.HeliumUniversal.HeliumErrors
   , module UniversalLexerError -- does not export the LexerError components, strange error in compiling Thesis.HeliumInstrumentation.DataTypeConversion, in ghc 6.4.2
   , UniversalLexerError.LexerError (LexerError)
   , UniversalLexerError.LexerErrorInfo (..)
   , module UniversalImportError -- UniversalImportError 
   , UniversalImportError.ResolveError (..)
   , module UniversalParseError  -- 
   , UniversalParseError.ParseError (..)
   , module UniversalStaticError 
   , UniversalStaticError.StaticError (..)
   , module UniversalTypeError 
   , UniversalTypeError.TypeError (..)
   ) where

-- import qualified Helium1v5v1.Src.HeliumNT.Src.Syntax.UHA_Syntax as UHA_Syntax (Name, Names, Strings, Position, Range)

import qualified Thesis.HeliumUniversal.LexerError as UniversalLexerError
import qualified Thesis.HeliumUniversal.ParseError as UniversalParseError
import qualified Thesis.HeliumUniversal.ImportError as UniversalImportError
import qualified Thesis.HeliumUniversal.StaticError as UniversalStaticError
import qualified Thesis.HeliumUniversal.TypeError as UniversalTypeError


import Data.Typeable
import Data.Maybe

data HeliumErrorComplete a = HeliumErrorComplete {
                                originalMessage :: String,
                                heliumErrorData :: a
                             }
          deriving (Show,Read, Eq)

-- abstract helium error data type, the error from a certain phase
data HeliumError
                 = LexerPhError   [HeliumErrorComplete UniversalLexerError.LexerError]
                 | ParsePhError   [HeliumErrorComplete UniversalParseError.ParseError]
                 | ImportPhError  [HeliumErrorComplete UniversalImportError.ResolveError]
                 | StaticPhError  [HeliumErrorComplete UniversalStaticError.StaticError]
                 | TypePhError    [HeliumErrorComplete UniversalTypeError.TypeError]
          deriving (Show, Read, Eq)

-- importing errors, not necessary for 1.1

class ToHeliumErrorData a where
   toHeliumErrorData :: [(HeliumErrorComplete a)] -> HeliumError

instance ToHeliumErrorData (UniversalLexerError.LexerError) where
   toHeliumErrorData heliumErrors = LexerPhError heliumErrors

instance ToHeliumErrorData UniversalParseError.ParseError where
   toHeliumErrorData parseErrors = ParsePhError parseErrors

instance ToHeliumErrorData UniversalImportError.ResolveError where
   toHeliumErrorData resolveErrors = ImportPhError resolveErrors

instance ToHeliumErrorData UniversalStaticError.StaticError where
   toHeliumErrorData staticErrors = StaticPhError staticErrors

instance ToHeliumErrorData UniversalTypeError.TypeError where
   toHeliumErrorData typeErrors = TypePhError typeErrors

-- abstract helium error summeration of all errors in a set
data HeliumErrorFlat = LexerPhErrorSingle   (HeliumErrorComplete UniversalLexerError.LexerError)
                     | ParsePhErrorSingle   (HeliumErrorComplete UniversalParseError.ParseError)
                     | ImportPhErrorSingle  (HeliumErrorComplete UniversalImportError.ResolveError)
                     | StaticPhErrorSingle  (HeliumErrorComplete UniversalStaticError.StaticError)
                     | TypePhErrorSingle    (HeliumErrorComplete UniversalTypeError.TypeError)
   deriving (Show, Read, Eq)

data HeliumErrorFlatNew a = HeliumErrorCompleteFlat a
   deriving (Show, Read, Eq)

flattenHeliumError :: HeliumError -> [HeliumErrorFlat]
flattenHeliumError helerr =
   case helerr of
      LexerPhError   errors -> map (\heliumErrorComplete -> LexerPhErrorSingle heliumErrorComplete) errors
      ParsePhError   errors -> map (\heliumErrorComplete -> ParsePhErrorSingle heliumErrorComplete) errors
      ImportPhError  errors -> map (\heliumErrorComplete -> ImportPhErrorSingle heliumErrorComplete) errors
      StaticPhError  errors -> map (\heliumErrorComplete -> StaticPhErrorSingle heliumErrorComplete) errors
      TypePhError    errors -> map (\heliumErrorComplete -> TypePhErrorSingle heliumErrorComplete) errors

{- old

data HeliumErrorFlat = LexerPhErrorSingle   UniversalLexerError.LexerError
                     | ParsePhErrorSingle   UniversalParseError.ParseError
                     | ImportPhErrorSingle  UniversalImportError.ResolveError
                     | StaticPhErrorSingle  UniversalStaticError.StaticError
                     | TypePhErrorSingle    UniversalTypeError.TypeError
   deriving (Show, Read, Eq)

flattenHeliumError :: HeliumError -> [HeliumErrorFlat]
flattenHeliumError helerr =
   case helerr of
      LexerPhError   errors -> map (\(HeliumErrorComplete msg error) -> LexerPhErrorSingle error) errors
      ParsePhError   errors -> map (\(HeliumErrorComplete msg error) -> ParsePhErrorSingle error) errors
      ImportPhError  errors -> map (\(HeliumErrorComplete msg error) -> ImportPhErrorSingle error) errors
      StaticPhError  errors -> map (\(HeliumErrorComplete msg error) -> StaticPhErrorSingle error) errors
      TypePhError    errors -> map (\(HeliumErrorComplete msg error) -> TypePhErrorSingle error) errors


-}

--applyToHeliumErrorFLat :: (HeliumErrorComplete b -> a) -> HeliumErrorFlat -> a
showOriMsgHeliumErrorFlat helerr =
   case helerr of
      LexerPhErrorSingle heliumErrorComplete    -> originalMessage heliumErrorComplete
      ParsePhErrorSingle heliumErrorComplete    -> originalMessage heliumErrorComplete
      ImportPhErrorSingle heliumErrorComplete   -> originalMessage heliumErrorComplete
      StaticPhErrorSingle heliumErrorComplete   -> originalMessage heliumErrorComplete
      TypePhErrorSingle heliumErrorComplete     -> originalMessage heliumErrorComplete
