module Thesis.HeliumUniversal.LexerError (
         module Thesis.HeliumUniversal.LexerError
       ) where

import Thesis.HeliumUniversal.BasicTypes (SourcePos)
import Data.Typeable

-- based on Helium 1.5.1
data LexerError = LexerError SourcePos LexerErrorInfo
     deriving (Typeable, Show, Read, Eq, Ord)

-- based on Helium 1.5.1
data LexerErrorInfo
    = UnterminatedComment
    | MissingExponentDigits
    | UnexpectedChar Char

    | IllegalEscapeInChar
    | EmptyChar
    | IllegalCharInChar
    | NonTerminatedChar (Maybe String)   -- the maybe string is added since Helium 1.2
    | EOFInChar

    | EOFInString
    | IllegalEscapeInString
    | NewLineInString
    | IllegalCharInString

    | TooManyClose Char
        -- In UnexpectedClose, first char is the closing bracket we see, 
        -- second char is theHelium1v5v1.Src.HeliumNT.Src.Syntax.UHA_Syntax closing bracket we would like to see first
        -- e.g. [(1,3]  =>  UnexpectedClose ']' ... ')'
    | UnexpectedClose Char SourcePos Char
    | StillOpenAtEOF [(SourcePos, Char)]
    deriving (Show, Read, Eq, Typeable, Ord)

