module Thesis.HeliumUniversal.StaticError where

import Thesis.HeliumUniversal.BasicTypes
import Data.Typeable

-------------------------------------------------------------
-- (Static) Errors, based on Errors from StaticErrors.hs, helium 1.5.1
-------------------------------------------------------------
type StaticErrors = [StaticError]
data StaticError  =
              NoFunDef Entity Name {-names in scope-}Names
            | Undefined Entity Name {-names in scope-}Names {-similar name in wrong name-space hint-}[String] {- hints -}
            | Duplicated Entity Names
            | LastStatementNotExpr Range
            | WrongFileName {-file name-}String {-module name-}String Range {- of module name -}
            | TypeVarApplication Name
            | ArityMismatch {-type constructor-}Entity Name {-verwacht aantal parameters-}Int {-aangetroffen aantal parameters-}Int
            | DefArityMismatch Name (Maybe Int) {- verwacht -} Range
            | RecursiveTypeSynonyms Names
            | PatternDefinesNoVars Range
            | IntLiteralTooBig Range String              -- pvk Added in helium 1.1.2
            | NegateNeeded Range                         -- pvk Added in helium 1.2, removed in helium 1.5
            | OverloadingDisabled Range                  -- pvk Added in helium 1.2
            | OverloadedRestrPat Name                    -- pvk added in helium 1.5
            | WrongOverloadingFlag Bool{- flag? -}       -- pvk added in helium 1.5
            | AmbiguousContext Name                      -- pvk Added in helium 1.2
            | UnknownClass Name                          -- pvk Added in helium 1.2
            | NonDerivableClass Name                     -- pvk added in helium 1.5
            | CannotDerive Name Tps                      -- pvk added in helium 1.5, (Tps not available in helium 1.1.2, only from 1.2 and upwards
            | TupleTooBig Range                          -- pvk added in helium 1.5
            deriving (Eq, Show, Read, Typeable)

---------------------------------------------------------------
-- Static Errors sub types
-- Name, Names, Position, Range imported from Helium1v5v1.Src.HeliumNT.Src.Syntax.UHA_Syntax, similar in all heliums
---------------------------------------------------------------
{- Moved to HeliumUniversal.HeliumBasicTypes
-- Name --------------------------------------------------------
data Name = Name_Identifier (Range) (Strings) (String)
          | Name_Operator (Range) (Strings) (String)
          | Name_Special (Range) (Strings) (String)
            deriving (Eq, Show, Read, Typeable)
-- Names -------------------------------------------------------
type Names = [(Name)]
-- Range -------------------------------------------------------
data Range = Range_Range (Position) (Position)
            deriving (Eq, Show, Read, Typeable)
-- Position ----------------------------------------------------
data Position = Position_Position (String) (Int) (Int)
              | Position_Unknown
            deriving (Eq, Show, Read, Typeable)
-- Strings -----------------------------------------------------
type Strings = [(String)]
-}

-------------------------------------------------------------
-- Entity from Helium1v5v1.Src.HeliumNT.Src.Staticanalysis.Messages.StaticErrors 
-------------------------------------------------------------

data Entity = TypeSignature
            | TypeVariable
            | TypeConstructor
            | Definition
            | Constructor
            | Variable
            | Import
            | ExportVariable
            | ExportModule
            | ExportConstructor  -- same for Helium 1.1, 1.1A, 1.1.1, 1.1.2
            | ExportTypeConstructor
            | Fixity
      deriving (Eq, Show, Read, Typeable)

-------------------------------------------------------------
-- Tps/Tp from Helium1v5v1.Src.Top.Src.Top.Types.Basics -- only available in Helium 1.2 and up
-------------------------------------------------------------

type Tps      = [Tp]
-- |A data type to represent monotypes. Note that 'Type' is already in use
-- in the Unified Haskell Architecture (UHA) which is used in the Helium compiler
data Tp       = TVar Int      -- ^The type variables are numbered.
              | TCon String   -- ^A type constant is represented by a string.
              | TApp Tp Tp    -- ^The application of two Top.Types. Not all types that can be
                              -- constructed are well-formed.
  deriving (Eq, Show, Read, Typeable)

 