module Thesis.HeliumUniversal.TypeError.Helium1v1TypeError where

import Thesis.HeliumUniversal.BasicTypes
import Data.Typeable
-- Helium 1v1 like TypeError

-- TypeError is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Messages/TypeErrors.hs:19
data TypeError
    = TypeError Range MessageBlock TypeErrorTable TypeErrorInfos
    | CustomTypeError [Range] Message
     deriving (Eq, Show, Read, Typeable)
     
-- Range is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Syntax/UHA_Syntax.hs:166
{- data Range -- defined in HeliumBasicTypes
    = Helium1v1.Src.HeliumNT.Src.Syntax.UHA_Syntax.Range_Range Helium1v1.Src.HeliumNT.Src.Syntax.UHA_Syntax.Position Helium1v1.Src.HeliumNT.Src.Syntax.UHA_Syntax.Position
-}

-- TypeErrorTable is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Messages/TypeErrors.hs:29
data TypeErrorTable
    = UnificationErrorTable [(String, MessageBlock)] (Either Tp TpScheme) (Either Tp TpScheme) |
      NotGeneralEnoughTable Tree TpScheme TpScheme
     deriving (Eq, Show, Read, Typeable)
     
-- TypeErrorInfos is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Messages/TypeErrors.hs:39
type TypeErrorInfos = [TypeErrorInfo]

-- Message is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Messages/Messages.hs:25
type Message = [MessageLine]


-- Helium1v1.Src.HeliumNT.Src.Syntax.UHA_Syntax.Position is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Syntax/UHA_Syntax.hs:156
-- data Position -- defined in HeliumBasicTypes

-- MessageBlock is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Messages/Messages.hs:32
data MessageBlock
    = MessageString String |
      MessageRange Range |
      MessageType Tp |
      MessageTypeScheme TpScheme |
      MessageInfoLink String |
      MessageOneLineTree Tree |
      MessageCompose MessageBlocks
     deriving (Eq, Show, Read, Typeable)
     
-- Tp is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Types/TypeRepresentation.hs:18
data Tp = TVar Int | TCon String | TApp Tp Tp
     deriving (Eq, Show, Read, Typeable)
     
-- TpScheme is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Types/TypeRepresentation.hs:16
data TpScheme = Scheme [Int] [(Int,String)] Tp
     deriving (Eq, Show, Read, Typeable)

-- TypeErrorInfo is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Messages/TypeErrors.hs:40
data TypeErrorInfo
    = TypeErrorHint String MessageBlock |
      IsFolkloreTypeError |
      HasDocumentationLink String
     deriving (Eq, Show, Read, Typeable)
    
-- MessageLine is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Staticanalysis/Messages/Messages.hs:27
data MessageLine
    = MessageOneLiner MessageBlock |
      MessageTable [(MessageBlock, MessageBlock)] |
      MessageHints String MessageBlocks
     deriving (Eq, Show, Read, Typeable)

-- Tree is a type constructor, defined at /users/pkeeken/trunk/development/instrumented_helium_compilers/Helium1v1/Src/HeliumNT/Src/Utils/OneLiner.hs:5
data Tree
    = Node [Tree] |
      Text String
     deriving (Eq, Show, Read, Typeable)

type MessageBlocks = [MessageBlock]

-- comparison
{- Hel 1.1.X
:i TypeError
:i Range
:i TypeErrorTable
:i TypeErrorInfos
:i Message
-- hidden Position 
:i Helium1v1.Src.HeliumNT.Src.Syntax.UHA_Syntax.Position 
:i MessageBlock
:i Tp
:i TpScheme 
:i TypeErrorInfo
:i MessageLine 
:i Tree 
:i MessageBlocks


-- not in Helium 1.1
:i RangeLexer
:i TypeErrorHint
:i Universal
:i QType
:i Predicate
:i OneLineTree

-}