module Thesis.Metrics where

import Statistics.DescriptiveAnalysis
import Statistics.SummaryMeasures
import Statistics.BasicTypes
import Thesis.Settings
import Thesis.Select
import Auxilary

import Thesis.Helium.Logging
import Thesis.Helium.Parser.ResearchLexer as LineLexer

-- import Thesis.Research.ResearchGeneration


import qualified Thesis.HeliumUniversal.HeliumErrors as UniversalDatatypes
import qualified Thesis.HeliumUniversal.LexerToken as UniversalDatatypes
--import Thesis.HeliumInstrumentation.CallHeliumSrc (doHeliumAnalysis)
import Thesis.HeliumInstrumentation.CallHeliumBin (doHeliumAnalysis, tryConversion)
import Thesis.HeliumUniversal.Args (AnalysisOption(..))

import Data.Maybe
import Data.Char
import Data.List
import System.Time hiding (Day)
-- import Data.Time -- new timelib


metricNotApplicableError :: String -> Logging -> String -> a
metricNotApplicableError metric log reason = error $ "\nError: \nMetric " ++ metric ++ " not applicable on logging, because " ++ reason ++ "(when working with log: " ++ show log

--------------------------------------------------------------------
-- Goal 1: Program characteristics
--------------------------------------------------------------------

------------------------------------------
-- Question 1: Program elementaries     --
------------------------------------------

-- Source code
m1_1_1 :: Logging -> String
m1_1_1 = getSource  -- unsafePerformIO . getContentsStrict . logInputPath

-- Source AST
m1_1_2 :: Logging -> String
m1_1_2 = undefined -- buildAST . getSource

-- Compiler output std
m1_1_3 :: Logging -> String
m1_1_3 = getCompilerOutput

-- Compiler output verbose
m1_1_4 :: Logging -> String
m1_1_4 = getCompilerOutputVerbose

-- Identity / Student
m1_1_5 :: Logging -> Username
m1_1_5 = username

m1_1_5_Analysis :: DescriptiveKey key => AnalysisFK key Logging Username
m1_1_5_Analysis = basicAnalysis' "m1_1_5 Student" m1_1_5

-- Phase
m1_1_6 :: Logging -> Phase
m1_1_6 = phase


m1_1_6_Analysis :: AnalysisFK KeyHistory Logging Phase
m1_1_6_Analysis = basicAnalysis' "m1_1_6 Phase" phase
phaseAnalysis = m1_1_6_Analysis

-- Used compiler
m1_1_7 :: Logging -> HeliumVersion
m1_1_7 = heliumVersion

m1_1_7_Analysis :: DescriptiveKey key => AnalysisFK key Logging HeliumVersion
m1_1_7_Analysis = basicAnalysis' "m1_1_7 Used Compiler" m1_1_7

-- Logging date/time
-- use new timelib
m1_1_8 :: Logging -> CalendarTime
m1_1_8 ={-
   stringToCalTime . -}logDate

-- Logging date/time, as string
m1_1_8B :: Logging -> String
m1_1_8B = 
   unparsedLogDate
  
-- Logging location   
m1_1_9 :: Logging -> Path
m1_1_9 = logPath

-- During Lab Hours
m1_1_10 :: Logging -> Bool
m1_1_10 = 
   undefined
   
-- During Lab Holiday
m1_1_11 :: Logging -> Bool
m1_1_11 = 
   undefined

-- Course information -- course year
m1_1_12 :: Logging -> Maybe PeriodName
m1_1_12 = collegePeriod 
   

m1_1_13 :: Logging -> Year_Week
m1_1_13 log = weekOfYear $ logDate log

-- when using TimeLib
--   let result = weekOfYearInfo . logDate $ log
--   in (fst3 result, snd3 result)

m1_1_13_Analysis :: DescriptiveKey key => AnalysisFK key Logging Year_Week
m1_1_13_Analysis = basicAnalysis' "m1_1_13 Year_Week" m1_1_13

{-
-- Course week information (year, week of year, day of week)  -- using TimeLib
m1_1_13b :: Logging -> (Integer, Int, Int)
m1_1_13b = weekOfYearInfo . logDate
-}
------------------------------------------
-- Question 2: Program size             --
------------------------------------------

-- Lines of source code
m1_2_1 :: Logging -> [String]
m1_2_1 = lines . m1_1_1

m1_2_1b :: Logging -> [LineLexer.SourceSegment]
m1_2_1b = linelexer . m1_1_1

-- written lines
m1_2_2 :: Logging -> [String]
m1_2_2 = filter (not . emptyLine) . m1_2_1

-- code lines
m1_2_3 :: Logging -> [String]
m1_2_3 = undefined
   
-- filter isCodeLine (breakOnComment . m1_1_1)

{-

newtype BareProgam = BareProgram [BareProgramLexemes]

data BareProgramLexemes
                 = BareSource String
                 | EndOfLineComment String
                 | InLineComment String

pBareProgram :: Parser BareProgram
pBareProgram =
   do x <- many pBareProgramLexeme
      return x

pBareProgramLexem :: Parser pBareProgramLexeme
   do x <-(pBareSource <|> p <|> p)
               <?> "program lexeme"

{- -- Old
breakOnComment :: Parser BareSource
breakOnComment :: String -> [String]
breakOnComment "" = [""]
breakOnComment line =
   let
      fstLineComment = partitionAtString "--" line
      fstInLineCommentBegin = partitionAtString "{-" line
      --fstInLineCommentEndAfterBegin = partitionAtString "-}" (snd fstInLineCommentBegin)
      fstInLineCommentEnd = -- {- (\(before, after) -> (before ++ "-}", dropPrefix "{-" after)) $ -}
         (partitionAtString "-}" line) 
      minPlacement p1 p2 = compare (fst p1) (fst p2)
      parts = minimumBy minPlacement [fstLineComment, fstInLineCommentBegin, fstInLineCommentEnd]
   in
      if (fst parts == line)
         then [line]   -- no new comment found
         else
            case part of
               fstLineComment -> [fst parts] ++ (breakOnComment $ snd parts)

isCommentLine line =
      isPrefixOf "--" trimmedLine || (isPrefixOf "{-" trimmedLine) && (isSuffixOf "-}" trimmedLine)
   where trimmedLine = trim line

isCodeLine = notCommentLine

-}

-- Lines Of Code
m1_2_4 :: Logging -> Int
m1_2_4 = frequency . m1_2_1

m1_2_4_Analysis :: AnalysisFK KeyHistory Logging Int
m1_2_4_Analysis = basicAnalysis' "m1_2_4_Lines Of Code" m1_2_4

-- m1_2_5 .. m1_2_6 undefined
------------------------------------------
-- Question 3: Token interpretation     --
------------------------------------------

-- Words of source code
m1_3_1 :: Logging -> [String]
m1_3_1 = words . m1_1_1

{-
m1_3_1_Analysis :: AnalysisFK KeyHistory Logging Phase
m1_3_1_Analysis = basicAnalysis' "m1_3_1 Words of source code" m1_3_1

m1_3_1_Metric :: MetricAnalysis KeyHistory Logging   -- Words of source code
m1_3_1_Metric = OrdMetricAnalysis m1_3_1_Analysis
WOCMetric   = m1_3_1_Metric 
-}

data InstrumentedValue a =
     InstrumentedValue { getValue :: a }
   | NoMaybe String


-- Lexical interpretation
m1_3_1b :: Logging -> Maybe (UniversalDatatypes.TokenAnalysisResult)
m1_3_1b log =
   let errorSource = "m1_3_1b (Helium Token Source Representation)"
       tokenResult = doHeliumAnalysis (heliumVersion log) log [TokenAnalysis] errorSource 
   in
      if (phase log > Lexical) then -- (checkMinPhase Lexical log)
         -- tryConversion tokenResult errorSource
         Just $ tryConversion tokenResult errorSource
      else
         Nothing -- "metric is only usefull for loggings that pass the Lexical phase" --
         -- metricNotApplicableError errorSource log "metric is only usefull for loggings that pass the Lexical phase" -- (read tokenResult)

--m1_3_1c :: Logging -> Maybe ([UniversalDatatypes.Token])
m1_3_1c log = -- UniversalDatatypes.tokens $ m1_3_1b log
   maybe Nothing (Just . UniversalDatatypes.tokens) $ m1_3_1b log

m1_3_1d :: Logging -> Maybe [UniversalDatatypes.Lexeme]
m1_3_1d log = -- map tokenToLexeme $ m1_3_1c log
   maybe Nothing (Just . (map tokenToLexeme)) $ m1_3_1c log

-- helper
tokenToLexeme = snd

m1_3_2 :: Logging -> Maybe [UniversalDatatypes.Token]
m1_3_2 log = Just $ undefined -- filter (flip elem reservedWords) (m1_3_1d log)

-- helper, to be defined in LexerToken
reservedWords = undefined

-- Number of words of source code
m1_3_4 :: Logging -> Int
m1_3_4 = frequency . m1_3_1

m1_3_4_Analysis :: AnalysisFK KeyHistory Logging Int
m1_3_4_Analysis = basicAnalysis' "m1_3_4 Number of words of source code" m1_3_4

-- tmp metric
m1_3_4d :: Logging -> Maybe (Int)
m1_3_4d log = -- frequency . m1_3_1d
   maybe Nothing (Just . frequency) $ m1_3_1d log

m1_3_4d_Analysis :: AnalysisFK KeyHistory Logging (Maybe Int)
m1_3_4d_Analysis = basicAnalysis' "m1_3_4d Number of Universal Lexemes of source code" m1_3_4d

{-
-- Lexical interpretation
m1_3_1b :: Logging ->  (UniversalDatatypes.TokenAnalysisResult)
m1_3_1b log =
   let errorSource = "m1_3_1b (Helium Token Source Representation)"
       tokenResult = doHeliumAnalysis (heliumVersion log) log [TokenAnalysis] errorSource 
   in
      if (phase log > Lexical) then -- (checkMinPhase Lexical log)
         tryConversion tokenResult errorSource
      else
         metricNotApplicableError errorSource log "metric is only usefull for loggings that pass the Lexical phase" -- (read tokenResult)

m1_3_1c :: Logging -> [UniversalDatatypes.Token]
m1_3_1c log = UniversalDatatypes.tokens $ m1_3_1b log

m1_3_1d :: Logging -> [UniversalDatatypes.Lexeme]
m1_3_1d log = map tokenToLexeme $ m1_3_1c log

-- helper
tokenToLexeme = snd

m1_3_2 :: Logging -> [UniversalDatatypes.Token]
m1_3_2 log = undefined -- filter (flip elem reservedWords) (m1_3_1d log)

-- helper, to be defined in LexerToken
reservedWords = undefined

-- Number of words of source code
m1_3_4 :: Logging -> Int
m1_3_4 = frequency . m1_3_1

m1_3_4_Analysis :: AnalysisFK KeyHistory Logging Int
m1_3_4_Analysis = basicAnalysis' "m1_3_4 Number of words of source code" m1_3_4

m1_3_4_Metric :: MetricAnalysis KeyHistory Logging   -- Number of words of source code
m1_3_4_Metric = IntMetricAnalysis m1_3_4_Analysis
wOCMetric   = m1_3_4_Metric 

-- tmp metric
m1_3_4d :: Logging -> Int
m1_3_4d = frequency . m1_3_1d

m1_3_4d_Analysis :: AnalysisFK KeyHistory Logging Int
m1_3_4d_Analysis = basicAnalysis' "m1_3_4d Number of Universal Lexemes of source code" m1_3_4d

m1_3_4d_Metric :: MetricAnalysis KeyHistory Logging   -- Number of of Universal Lexemes of source code
m1_3_4d_Metric = IntMetricAnalysis m1_3_4d_Analysis
-}
------------------------------------------
-- Question 6: Output analysis          --
------------------------------------------
{-
m1_6_1 :: Logging -> IO UniversalDatatypes.HeliumError
m1_6_1 log = do x <- doHeliumAnalysis (heliumversion log) log [ErrorAnalysis]
                case x of
                  Nothing       -> ioError $ userError ("metric (1_6_1) not applyable to this logging (log: " ++ show log ++ ")")
                  Just retrievedError -> return (read retrievedError)
-}

m1_6_1 :: Logging -> UniversalDatatypes.HeliumError
m1_6_1 log =
   let
      errorSource = "m1_6_1 (HeliumErrors)"
      retrievedError = doHeliumAnalysis (heliumVersion log) log [ErrorAnalysis] errorSource
   in
      if (checkMaxPhase Typing log) then 
         tryConversion retrievedError errorSource
      else
         metricNotApplicableError errorSource log "metric is only usefull for loggings that end compilation in the typing phase or before this phase"


-- list of all errors, independent of phase, can be inconsistent in certain cases
m1_6_1b :: Logging ->  [UniversalDatatypes.HeliumErrorFlat]
m1_6_1b =
   UniversalDatatypes.flattenHeliumError . m1_6_1

-- number of errors
m1_6_2a =
   size . UniversalDatatypes.flattenHeliumError . m1_6_1

-- original error messages
m1_6_2b :: Logging -> [String]
m1_6_2b =
   map (UniversalDatatypes.showOriMsgHeliumErrorFlat) . UniversalDatatypes.flattenHeliumError . m1_6_1

{-
-- lines or errors
m1_6_2c :: Logging -> [[String]]
m1_6_2c =
   map (UniversalDatatypes.showOriMsgHeliumErrorFlat) . UniversalDatatypes.flattenHeliumError . m1_6_1
-}

----------------------------------------------
-- Question 6: Logging Difference/Coherence --
----------------------------------------------

m1_7_1 :: Logging -> Logging -> TimeDiff
m1_7_1 log1 log2 = diffClockTimes (toClockTime $ m1_1_8 log2) (toClockTime $ m1_1_8 log1)

calcCompilationTimeDiff :: Logging -> Maybe Logging -> Maybe TimeDiff
calcCompilationTimeDiff fst y =
         do
            snd <- y
            return (diffClockTimes (toClockTime $ m1_1_8 snd) (toClockTime $ m1_1_8 fst))

------------------------------------------------------------------------------------
-- Metric help data / functions
------------------------------------------------------------------------------------

-- used with m1_2_X
emptyLine :: String -> Bool
emptyLine line = (null . trim) line


------------------------------------------
-- College information and add. date info
------------------------------------------

collegePeriod :: Logging -> Maybe PeriodName
collegePeriod log =
   let getPeriod = (\period -> if (isDuringPer log period) then Just $ periodName period else Nothing)
       periodNames = mapMaybe getPeriod fpCourses
   in
      if (length periodNames > 0)
         then
            if (length periodNames == 1)
               then Just $ head periodNames
               else error $ "Provided periods are overlapping of logging: " ++ show log
         else
            Nothing

{- shorter but not faster
   if (isDuringAnyPer log collegePeriods)
      then Just (head (mapMaybe getPeriod collegePeriods))
      else Nothing
-}

isDuringPer :: Logging -> CollegePeriod -> Bool
isDuringPer log (CollegePeriod nm start end) = isCompiledBetween start end log

isDuringAnyPer :: Logging -> [CollegePeriod] -> Bool
isDuringAnyPer = any . isDuringPer

checkMaxPhase :: Phase -> Logging -> Bool
checkMaxPhase maxPhase log = (phase log <= maxPhase)

checkMinPhase :: Phase -> Logging -> Bool
checkMinPhase minPhase log = (phase log >= minPhase)

{-

TimeLib implementation

weekOfYearInfo :: CalendarDate -> (Integer, Int, Int)  -- (Year, week of Year, day of Week)
weekOfYearInfo = undefined
  -- toISO8601Week . {- localDay . -} stringToNewTimeLibLocalTime  -- using TimeLib

current: weekOfYearInfo:: Logging -> (Integer,Int)
  
stringToNewTimeLibLocalTime :: String -> Day -- LocalTime  -- losing the pico seconds, so losing uniqueness
stringToNewTimeLibLocalTime input =
   let yearAndRest = splitAt 4 input
       monAndRest  = splitAt 2 (dropWhile (=='-') $ snd yearAndRest)
       dayAndRest  = splitAt 2 (dropWhile (=='-') $ snd monAndRest)
       hourAndRest = splitAt 2 (dropWhile (=='@') $ snd dayAndRest)
       minAndRest  = splitAt 2 (dropWhile (=='_') $ snd hourAndRest)
       secAndRest  = splitAt 2 (dropWhile (=='_') $ snd minAndRest)
       msecAndRest = splitAt 3 (dropWhile (=='_') $ snd secAndRest)

       year = (read (fst yearAndRest)::Integer)
       month = (read (fst monAndRest)::Int)
       day = (read (fst dayAndRest)::Int)

       --localDay = fromGregorian year month day

       localTimeOfDay = TimeOfDay (read (fst hourAndRest)::Int)
                                  (read (fst minAndRest)::Int)
                                  0 --undefined -- (read (fst secAndRest)::Int)  how to read a pico value?
   in
      show year ++ show month ++ show day -- localDay
      -- LocalTime localDay localTimeOfDay
-}

--
-- Werk college selection, FP 0304
--

-- weeks of fp0203 course, week start on sunday
fp0203week11_begin = "2003-03-09@00_00_00_000"
fp0203week11_end   = "2003-03-15@12_99_99_999"
fp0203week12_begin = "2003-03-16@00_00_00_000"
fp0203week12_end   = "2003-03-22@12_99_99_999"
fp0203week13_begin = "2003-03-23@00_00_00_000"
fp0203week13_end   = "2003-03-29@12_99_99_999"
fp0203week14_begin = "2003-03-30@00_00_00_000"
fp0203week14_end   = "2003-04-05@12_99_99_999"
fp0203week15_begin = "2003-04-06@00_00_00_000"
fp0203week15_end   = "2003-04-12@12_99_99_999"
fp0203week16_begin = "2003-04-13@00_00_00_000"
fp0203week16_end   = "2003-04-19@12_99_99_999"
fp0203week17_begin = "2003-04-20@00_00_00_000"
fp0203week17_end   = "2003-04-26@12_99_99_999"
fp0203week18_begin = "2003-04-27@00_00_00_000"   -- geen practicum meer
fp0203week18_end   = "2003-05-03@12_99_99_999"   -- geen practicum meer

-- weeks of FP0304 course, no month correction
fp0304week6_begin = "2004-02-01@00_00_00_000"
fp0304week6_end   = "2004-02-07@12_99_99_999"
fp0304week7_begin = "2004-02-08@00_00_00_000"
fp0304week7_end   = "2004-02-14@12_99_99_999"
fp0304week8_begin = "2004-02-15@00_00_00_000"
fp0304week8_end   = "2004-02-21@12_99_99_999"
fp0304week9_begin = "2004-02-22@00_00_00_000"    -- Voorjaarsvakantie
fp0304week9_end   = "2004-02-28@12_99_99_999"    -- Voorjaarsvakantie
fp0304week10_begin = "2004-02-29@00_00_00_000"
fp0304week10_end   = "2004-03-06@12_99_99_999"
fp0304week11_begin = "2004-03-07@00_00_00_000"
fp0304week11_end   = "2004-03-13@12_99_99_999"
fp0304week12_begin = "2004-03-14@00_00_00_000"
fp0304week12_end   = "2004-03-20@12_99_99_999"
fp0304week13_begin = "2004-03-21@00_00_00_000"
fp0304week13_end   = "2004-03-27@12_99_99_999"
-- last exercise was using Hugs

-- weeks of FP0405 course, no month correction
fp0405week6_begin = "2005-02-06@00_00_00_000"
fp0405week6_end   = "2005-02-12@12_99_99_999"
fp0405week7_begin = "2005-02-13@00_00_00_000"
fp0405week7_end   = "2005-02-19@12_99_99_999"
fp0405week8_begin = "2005-02-20@00_00_00_000"
fp0405week8_end   = "2005-02-26@12_99_99_999"
fp0405week9_begin = "2005-02-27@00_00_00_000" 
fp0405week9_end   = "2005-03-05@12_99_99_999" 
fp0405week10_begin = "2005-03-06@00_00_00_000"
fp0405week10_end   = "2005-03-12@12_99_99_999"
fp0405week11_begin = "2005-03-13@00_00_00_000"
fp0405week11_end   = "2005-03-19@12_99_99_999"
fp0405week12_begin = "2005-03-20@00_00_00_000"
fp0405week12_end   = "2005-03-26@12_99_99_999"
fp0405week13_begin = "2005-03-27@00_00_00_000"
fp0405week13_end   = "2005-04-02@12_99_99_999"
fp0405week14_begin = "2005-04-03@00_00_00_000"
fp0405week14_end   = "2005-04-09@12_99_99_999"
-- last exercise was using Ghc

-- weeksFP0203  -- :: [Logging -> Bool]
fp0203week11 = isCompiledBetween fp0203week11_begin fp0203week11_end
fp0203week12 = isCompiledBetween fp0203week12_begin fp0203week12_end
fp0203week13 = isCompiledBetween fp0203week13_begin fp0203week13_end
fp0203week14 = isCompiledBetween fp0203week14_begin fp0203week14_end
fp0203week15 = isCompiledBetween fp0203week15_begin fp0203week15_end
fp0203week16 = isCompiledBetween fp0203week16_begin fp0203week16_end
fp0203week17 = isCompiledBetween fp0203week17_begin fp0203week17_end
fp0203week18 = isCompiledBetween fp0203week18_begin fp0203week18_end

-- weeksFP0304 :: [Logging -> Bool]
fp0304week6 = isCompiledBetween fp0304week6_begin fp0304week6_end
fp0304week7 = isCompiledBetween fp0304week7_begin fp0304week7_end
fp0304week8 = isCompiledBetween fp0304week8_begin fp0304week8_end
fp0304week9 = isCompiledBetween fp0304week9_begin fp0304week9_end
fp0304week10 = isCompiledBetween fp0304week10_begin fp0304week10_end
fp0304week11 = isCompiledBetween fp0304week11_begin fp0304week11_end
fp0304week12 = isCompiledBetween fp0304week12_begin fp0304week12_end
fp0304week13 = isCompiledBetween fp0304week13_begin fp0304week13_end

--weeksFP0405 :: [Logging -> Bool]
fp0405week6   = isCompiledBetween fp0405week6_begin fp0405week6_end
fp0405week7   = isCompiledBetween fp0405week7_begin fp0405week7_end
fp0405week8   = isCompiledBetween fp0405week8_begin fp0405week8_end
fp0405week9   = isCompiledBetween fp0405week9_begin fp0405week9_end
fp0405week10  = isCompiledBetween fp0405week10_begin fp0405week10_end
fp0405week11  = isCompiledBetween fp0405week11_begin fp0405week11_end
fp0405week12  = isCompiledBetween fp0405week12_begin fp0405week12_end
fp0405week13  = isCompiledBetween fp0405week13_begin fp0405week13_end
fp0405week14  = isCompiledBetween fp0405week14_begin fp0405week14_end

{-

-- a very in efficient week of year calc, probl. due to IO calling / unsafe c calls

weekOfYearInfo_old:: Logging -> Year_Week -- (Integer, Int)
weekOfYearInfo_old log =
   case (collegePeriod log) of
      Nothing -> error "logging is not in predefined set of weeks or not in the predef courses, use timelib"
      Just Fp0203 -> Year_Week $ findWfp0203 log
      Just Fp0304 -> Year_Week $ findWfp0304 log
      Just Fp0405 -> Year_Week $ findWfp0405 log

findWfp0203 log
   | fp0203week11 log = (2003, 11)
   | fp0203week12 log = (2003, 12)
   | fp0203week13 log = (2003, 13)
   | fp0203week14 log = (2003, 14)
   | fp0203week15 log = (2003, 15)
   | fp0203week16 log = (2003, 16)
   | fp0203week17 log = (2003, 17)
   | fp0203week18 log = (2003, 18)
   | otherwise        = error "error week is not specified for fp0203"

findWfp0304 log
   | fp0304week6 log = (2004, 6)
   | fp0304week7 log = (2004, 7)
   | fp0304week8 log = (2004, 8)
   | fp0304week9 log = (2004, 9)
   | fp0304week10 log = (2004, 10)
   | fp0304week11 log = (2004, 11)
   | fp0304week12 log = (2004, 12)
   | fp0304week13 log = (2004, 13)
   | otherwise        = error "error week is not specified for fp0304"

findWfp0405 log
   | fp0405week6 log = (2005, 6)
   | fp0405week7 log = (2005, 7)
   | fp0405week8 log = (2005, 8)
   | fp0405week9 log = (2005, 9)
   | fp0405week10 log = (2005, 10)
   | fp0405week11 log = (2005, 11)
   | fp0405week12 log = (2005, 12)
   | fp0405week13 log = (2005, 13)
   | fp0405week14 log = (2005, 14)
   | otherwise        = error "error week is not specified for fp0405"
-}
{-

--
-- FP weeks per course
--

weeksFP0203 :: [Logging -> Bool]
weeksFP0203   = [ isCompiledBetween fp0203week11_begin fp0203week11_end
                , isCompiledBetween fp0203week12_begin fp0203week12_end
                , isCompiledBetween fp0203week13_begin fp0203week13_end
                , isCompiledBetween fp0203week14_begin fp0203week14_end
                , isCompiledBetween fp0203week15_begin fp0203week15_end
                , isCompiledBetween fp0203week16_begin fp0203week16_end
                , isCompiledBetween fp0203week17_begin fp0203week17_end
                , isCompiledBetween fp0203week18_begin fp0203week18_end
                ]

weeksFP0304 :: [Logging -> Bool]
weeksFP0304   = [ isCompiledBetween fp0304week6_begin fp0304week6_end
                , isCompiledBetween fp0304week7_begin fp0304week7_end
                , isCompiledBetween fp0304week8_begin fp0304week8_end
                , isCompiledBetween fp0304week9_begin fp0304week9_end
                , isCompiledBetween fp0304week10_begin fp0304week10_end
                , isCompiledBetween fp0304week11_begin fp0304week11_end
                , isCompiledBetween fp0304week12_begin fp0304week12_end
                , isCompiledBetween fp0304week13_begin fp0304week13_end
                ]

weeksFP0405 :: [Logging -> Bool]
weeksFP0405   = [ isCompiledBetween fp0405week6_begin fp0405week6_end
                , isCompiledBetween fp0405week7_begin fp0405week7_end
                , isCompiledBetween fp0405week8_begin fp0405week8_end
                , isCompiledBetween fp0405week9_begin fp0405week9_end
                , isCompiledBetween fp0405week10_begin fp0405week10_end
                , isCompiledBetween fp0405week11_begin fp0405week11_end
                , isCompiledBetween fp0405week12_begin fp0405week12_end
                , isCompiledBetween fp0405week13_begin fp0405week13_end
                , isCompiledBetween fp0405week14_begin fp0405week14_end
                ]


--
-- Werk college selection, FP 0304
--

-- start of periode fp 0304
startDateofFP0304 :: CalendarTime
startDateofFP0304 = stringToCalTime "2004-01-01@00_00_00_000"

-- offset of week numer of fp0304
weekOffsetFP0304 :: Int
weekOffsetFP0304 = 6

-- werkcollSchedule 
werkcollSchedule :: [(String, Int, Int, [Int])]
werkcollSchedule = [
    ("di",13,15,[6..8]++[10..15])
   ,("di",15,17,[6..8]++[10..15])
   ,("do",11,13,[6..8]++[10..15])
   ,("wo",11,13,[6..8]++[10..15])
   ,("wo",13,15,[6..8]++[10..15])
   ,("do",13,15,[6..8]++[10..15])
   ,("vr",11,13,[6..8]++[10..12]++[15])
   ,("vr",13,15,[6..8]++[10..12]++[15])
   ]

-- generate (from time, to time) for each werkcollege in werkcollege schedule
genWCollege :: Int -> [(String, Int, Int, [Int])] -> [(String, String)]
genWCollege weekOffset schedule = concatMap genWCDates schedule
   where 
      genWCDates :: (String, Int, Int, [Int]) -> [(String, String)]
      genWCDates (day, starthour, endhour, weeks@(x:xs)) = map (giveWCOfWeek) weeks
         where
            giveWCOfWeek week = let
                                   diff1 = TimeDiff {tdYear=0, tdMonth=0, tdDay=daydiff+daydiff2, tdHour=starthour, tdMin=0, tdSec=0, tdPicosec=0}
                                   diff2 = TimeDiff {tdYear=0, tdMonth=0, tdDay=daydiff+daydiff2, tdHour=endhour, tdMin=0, tdSec=0, tdPicosec=0}
                                in  
                                   (calTimeToString . toUTCTime . addToClockTime diff1 $ (toClockTime startDateofFP0304), 
                                    calTimeToString . toUTCTime . addToClockTime diff2 $ (toClockTime startDateofFP0304))
               where 
                  daydiff :: Int
                  daydiff = (week - weekOffset) * 7
                  daydiff2 :: Int
                  daydiff2 = case day of
                                "di" -> 2
                                "wo" -> 3
                                "do" -> 4
                                "vr" -> 5

-- selection of werkcollges, in tuple form
werkcollegesFP0304 :: [(String, String)]
werkcollegesFP0304 = genWCollege weekOffsetFP0304 werkcollSchedule

-- selection of werkcollges, in function form
werkcollegesFP0304selections :: [Logging -> Bool]
werkcollegesFP0304selections = map betweenDates werkcollegesFP0304 
   where
      betweenDates (start, end) = isCompiledBetween start end

-- selection of werkcolleges loggings, in list
loggingselectionWCFP0304 :: [Logging]
loggingselectionWCFP0304 = (concat $ selectLoggingSets werkcollegesFP0304selections)

-- write selection to file
writeLoggingSelectionToFile :: [Logging] -> String -> IO ()
writeLoggingSelectionToFile selection filename
   = do  let output = unlines $ map ppLogging selection
         output `seq` writeFile (programdir ++ "/" ++ filename ++ ".csv") (output)
         putStr ("Klaar! Selectie weggeschreven in "++ programdir ++ "/" ++ filename ++ ".csv")

-- write fp0304 loggings to file
writeLoggingSelectionToFilefp0304 :: IO ()
writeLoggingSelectionToFilefp0304 = writeLoggingSelectionToFile loggingselectionWCFP0304 "selectionfp0304"

-}



-- examples
{-
do1 = analyseSimple m1_2_4
do2 = giveBasicStats . analyseSimple $ m1_2_4
do3 = frequencies . analyseSimple $ m1_2_4
do4 = writeFile "test.csv" (verticalList do3)
-}

------------------------------------------------------------------
--- Old style error recognition
------------------------------------------------------------------
{-

--old way of counting a certain selection
numberOfLoggingForSelection :: (Logging -> Bool) -> Int
numberOfLoggingForSelection selection = frequency $ selectLoggings selection

--old way of counting a selection for identity == ...
nOLForIdentity   identity = numberOfLoggingForSelection (isIdentity identity)
nOLForIdentity'  identity = frequency $ analyse (isIdentity identity) id
nOLForIdentity'' identity = frequency $ selectLoggings (isIdentity identity)
-}

{-
-- Analysis on compiler output, old way in tuple form
aPresentedErrors = analyseCO presentedErrors
aTotalAOfErrors  = analyseCO totalAmountOfErrors
aCompleteOutput  = analyseCO id
aReturnLog   log = (log, log)
aSelect fun log  = (fun log, log)


-- old way of analysing errors, using regex's
-- Aux. analysis function on compiler output (String -> a)
presentedErrors :: String -> [String]
presentedErrors = multipleMatchRegex errorRegexNLA
   where 
      errorRegexNLA, errorRegexLA, errorRegexSimple :: Regex    -- multiple implementation depending on regex capabilities
      errorRegexSimple = mkRegexWithOpts "(\\([0-9]+,[0-9]+\\): .*(\n [ ]+.*)*)" True True  --Warnings are not filters, lookahead is not part of e-grep
      errorRegexLA     = mkRegexWithOpts "(\\([0-9]+,[0-9]+\\): (?!Warning).*(\n [ ]+.*)*)" True True
      errorRegexNLA    = mkRegexWithOpts "(\\([0-9]+,[0-9]+\\): ([^W]|.[^a]|..[^r]|...[^n]|....[^i]|.....[^n]|......[^g]).*(\n [ ]+.*)*)" True True

totalAmountOfErrors :: String -> Int
totalAmountOfErrors output = 
  case (matchRegexAll (totalAErrorsRegex) output) of
     Just x -> case (fourth x) of 
                  [] -> error "Error in totalAmountOfErrors: match succeeded but without any sub expresssions as result"
                  otherwise -> ((read . head . fourth) $ x) -- catch error if read fails
     Nothing -> 0 
   where
      totalAErrorsRegex :: Regex
      totalAErrorsRegex = mkRegexWithOpts "^[ ]*Compilation failed with ([0-9]+) error.*$" True True
-}

-- amount of errors of loggings in phase T
-- analysis :: [Logging] -> ([a], [Logging])
{-
analysis ds = 
   map (mean . frequency . (doSingleAnalysis aPresentedErrors)) ds
   where selection1 = filter (isPhase (Typing))  
-}
