module OldStuff where

programmingdir :: String
programmingdir = "/users/pkeeken/trunk/writings/programming/heliumprganalysis/"

-- in / out of results
writeDSToFile :: Show a => String -> String -> [a] -> IO ()
writeDSToFile filename listname dataset = writeFile (programmingdir ++ filename) (show dataset)

readDSFromFile fn = 
  do geg <- readFile (programmingdir ++fn)
     let log = (read geg) :: [Logging]
     print (log !! 5)

--show per line, unused, use show to write it to a file
ppList :: Show a => String -> [a] -> String
ppList nm l = nm ++ " = [" ++ (init $ content l) ++ "]"
  where content = foldr (\elm rest -> " " ++ (show elm) ++ "," ++ rest) ""

--split list in fixed sizes
partitionAt :: Int -> [a] -> [[a]]
partitionAt am ds | (length ds) > am = (take am ds) : (partitionAt am (drop am ds))
                  | otherwise     = [(take am ds)]

--simular
blocks blockSize = 
   takeWhile ( not . null) . map fst. iterate (splitAt blockSize . snd) . splitAt blockSize in blocks 2 [0..9]

{- 

line wise reading of a file 
main = do handle <- openFile logfile ReadMode  -- open the file for reading
          str <- hGetLine handle               -- get the first line out of it
          let n = read str                     -- parse the number to be found on the first line
          if n > 10
              then do putStr "HIGH SCORE! Enter your name: " -- note the use of do, since we're chaining together more than one action here
                      name <- getLine                        -- get a line from the user, then print a result based on it.
                      putStrLn ("Congratulations " ++ name ++ ", on your recent success!")
              else putStrLn "Get a better file, you lose." -- here we just insult the user for their poor choice of files,
                                                           -- and as this is just one thing, we don't need a do block.
          hClose handle                           -- clean up the handle


old way of doing analyses for the data set per phase
loggingStats :: [Logging] -> [(String, Integer, Float)]
loggingStats logs
   = let  -- numLoggings :: Integer
         numLoggings = toEnum . length $ logs
--         numberOfPhase :: Phase -> DataSet -> Float
         numberOfPhase ph = toEnum . length . filter (isPhase ph)
     in [ ("Amount Lexical Logs", numberOfPhase Lexical logs , (numberOfPhase Lexical logs) /(numLoggings*100.0))
          ,("Amount Parser  Logs", numberOfPhase Parsing logs, (numberOfPhase Parsing logs)/(numLoggings*100))
          ,("Amount ResolOp Logs", numberOfPhase ResolOp logs, (numberOfPhase ResolOp logs)/(numLoggings*100))
          ,("Amount Static Logs", numberOfPhase (Static []) logs, (numberOfPhase (Static []) logs)/(numLoggings*100))
          ,("Amount Typing Logs", numberOfPhase Typing logs, numberOfPhase Typing logs/(numLoggings*100))
          ,("Amount CodeGen Logs", numberOfPhase CodeGen logs, numberOfPhase CodeGen logs/(numLoggings*100))
          ,("Amount InternalErr Logs", numberOfPhase InternalErr logs, numberOfPhase InternalErr logs/(numLoggings*100))
          ,("Total Amount Logs", numLoggings, 100.0)
        ]

-}


{- lazy try, parsec is based on a IO monad, dus zal alleen werken, als ik de sequence herschrijf in een lazy variant
getLogLines :: IO [ String ]
getLogLines = 
   do handle <- openFile logfile1 ReadMode
      loglines <- hGetContents handle
      return (lines loglines)

getLoggings :: IO [ Logging ]
getLoggings = do xs <- getLogLines
                 let parsedLines = (map parseLine xs)
                 ys <- sequence (parsedLines)  -- loggings should be lazy evaluated :(
                 return (concat ys)            -- remove empty loggings (comments)

app :: (Logging -> a) -> IO [ IO [ Logging ] ]
app f = do xs <- getLogLines
           return (map (parseLine) xs)

parseLine :: String -> IO [Logging]
parseLine line = 
   do case (parseLogfileLine line) of 
         Left err  -> do print err
                         return []
         Right xs  -> return xs
-}
-- werkt in ghci
-- let g = let f = do y <- getLogLines; return y in  do x <- f ; return ((map parseLine x)) in do x <-g ; y <- (mapM sequence x) ; return y

{-
appOnLogLines fun = 
 do result <- parseLogfile logfile2
    case (result) of
      Left err  -> print err
      Right xs  -> print (fun xs)

-}

helErrorRegex :: HeliumVersion -> Regex
helErrorRegex helver = case (helver) of 
                      HeliumVersion "1.0" "" -> errorVersion1
                      otherwise -> mkRegexWithOpts "^[ ]*Compilation failed with ([0-9]+) error.*$" True True  -- like totalAErrors
                        --error ("no regular expression available for " ++ (show helver))
 where
    errorVersion1 = mkRegexWithOpts "(\\([0-9]+,[0-9]+\\):)" True True -- match with lines like "(23,23): blab
    errorVersion2 = mkRegex "(\\([0-9]+,[0-9]+\\): .*)"  -- match with lines like "(23,23): blabla"
