module Thesis.Research.AbstractIntMetricResearch where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.Presentation
-- import Statistics.BasicTypes

import Thesis.Research.Common
import Thesis.Helium.Logging

import Auxilary
import Grouping

import Data.Maybe
import System.Time

{-

Abstract research to study metric in time, based on ideas from Modulelength research / line based interpretation res

-}

type Metric a = AnalysisFK KeyHistory Logging a

metricAnalysisPerWeek :: Metric a -> AnalysisFK KeyHistory [Logging] [a]
metricAnalysisPerWeek metric =
       mapAnalysis metric
   <.> groupPerWeek

metricAnalysisPerWeekDay :: Metric a -> AnalysisFK KeyHistory [Logging] [a]
metricAnalysisPerWeekDay metric =
       mapAnalysis metric
   <.> groupPerWeekDay
   <.> groupPerWeek

-- DataInfo a req should be removed, by new intSummarytable impl
abstractIntMetricResearchInTime :: (Num a, Real a, DataInfo a) =>
   AnalysisFK KeyHistory Logging a -> String -> OutputFormat -> [(KeyHistory, [Logging])] -> FilePath -> IO MarkUpDoc
abstractIntMetricResearchInTime metric mname format input outputpath = do
   let
      name = -- "Abstract Int Metric Research In Time" ++ " (" ++ mname ++ ")"
         mname
      researchdir = joinPaths outputpath (stringToFilePath name)
--       outputfile  = joinDirFileExt researchdir (stringToFilePath name) $ showFormatExt format

   createDirectoryIfMissing True researchdir

   boxPlotPerWeek <- renderBoxPlot researchdir $
                              metricAnalysisPerWeek metric <$> input
   barChartAvgPerWeek <- renderBarChart researchdir $
                           averageAnalysis <.> metricAnalysisPerWeek metric <$> input
   barChartMedPerWeek <- renderBarChart researchdir $
                           medianAnalysis <.> metricAnalysisPerWeek metric <$> input
   barPlotAvgPerDay <- -- renderBarPlot1D (showMaybeData) researchdir $ analysisAvgLOCPerWeekDay metric <$> input
         renderBarChart researchdir $ averageAnalysis <.> metricAnalysisPerWeekDay metric <$> input  -- new
   barPlotMedPerDay <- 
         renderBarChart researchdir $ medianAnalysis <.> metricAnalysisPerWeekDay metric <$> input

   -- writeFile outputfile $
   --                renderMarkUpDoc format $
   let metricResult = 
                  
                    (subsection Nothing ! [formatonly $ show HtmlFormat] << name)
                     +++
                    plotToFigure' outputpath boxPlotPerWeek
                     +++
                    plotToFigure' outputpath barChartAvgPerWeek
                     +++
                    plotToFigure' outputpath barChartMedPerWeek
                     +++
                    plotToFigure' outputpath barPlotAvgPerDay
                     +++
                    plotToFigure' outputpath barPlotMedPerDay
--                      +++
--                     (intSummarytable $
--                          (mapAnalysis metric)
--                      <.> groupPerWeekDay
--                      <.> groupPerWeek
--                      <$> input)
   -- return noMarkUp
   return metricResult

