module Thesis.Research.CheckLogDate where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.Presentation

import Thesis.Metrics
import Thesis.Select
import Thesis.Research.Common
import Thesis.Helium.Logging
import Data.Typeable

import Statistics.BasicTypes

import Auxilary
import Grouping

import Data.Maybe
import System.Time

{-

Compare date of logfile to the parsed and pp date, to avoid date problems

also some week calc checks

mentioned runtime is for using the analyses on the complete fp0304 logfile

Mind: Analysis used when logDate Logging data type was returning CalenderDate (String), now is System.Time.CalendarTime

-}


default_logfile    = "datasets/demo/demo.log"
default_outputpath = "analysis_results/demo"

-- parse and unparse log date
getDate = calTimeToString {-. stringToCalTime -}. logDate

getWeekDay = ctWDay . {-stringToCalTime . -}logDate

compareDate datestr = ((calTimeToString $ stringToCalTime datestr) == datestr)

-- check whether the calculated week of year info is really in the predefined week
chk_weekNum :: Logging -> Year_Week -> Bool
chk_weekNum log yw
                     | year yw == 2003 && week yw == 11 =
                           isCompiledBetween fp0203week11_begin fp0203week11_end log
                     | year yw == 2003 && week yw == 12 =
                           isCompiledBetween fp0203week12_begin fp0203week12_end log
                     | year yw == 2003 && week yw == 13 =
                           isCompiledBetween fp0203week13_begin fp0203week13_end log
                     | year yw == 2003 && week yw == 14 =
                           isCompiledBetween fp0203week14_begin fp0203week14_end log
                     | year yw == 2003 && week yw == 15 =
                           isCompiledBetween fp0203week15_begin fp0203week15_end log
                     | year yw == 2003 && week yw == 16 =
                           isCompiledBetween fp0203week16_begin fp0203week16_end log
                     | year yw == 2003 && week yw == 17 =
                           isCompiledBetween fp0203week17_begin fp0203week17_end log
                     | year yw == 2003 && week yw == 18 =
                           isCompiledBetween fp0203week18_begin fp0203week18_end log

                     -- fp0304
                     | year yw == 2004 && week yw == 6 =
                           isCompiledBetween fp0304week6_begin fp0304week6_end log
                     | year yw == 2004 && week yw == 7 =
                           isCompiledBetween fp0304week7_begin fp0304week7_end log
                     | year yw == 2004 && week yw == 8 =
                           isCompiledBetween fp0304week8_begin fp0304week8_end log
                     | year yw == 2004 && week yw == 9 =
                           isCompiledBetween fp0304week9_begin fp0304week9_end log
                     | year yw == 2004 && week yw == 10 =
                           isCompiledBetween fp0304week10_begin fp0304week10_end log
                     | year yw == 2004 && week yw == 11 =
                           isCompiledBetween fp0304week11_begin fp0304week11_end log
                     | year yw == 2004 && week yw == 12 =
                           isCompiledBetween fp0304week12_begin fp0304week12_end log
                     | year yw == 2004 && week yw == 13 =
                           isCompiledBetween fp0304week13_begin fp0304week13_end log

                     -- fp0405
                     | year yw == 2005 && week yw == 6 =
                           isCompiledBetween fp0405week6_begin fp0405week6_end log
                     | year yw == 2005 && week yw == 7 =
                           isCompiledBetween fp0405week7_begin fp0405week7_end log
                     | year yw == 2005 && week yw == 8 =
                           isCompiledBetween fp0405week8_begin fp0405week8_end log
                     | year yw == 2005 && week yw == 9 =
                           isCompiledBetween fp0405week9_begin fp0405week9_end log
                     | year yw == 2005 && week yw == 10 =
                           isCompiledBetween fp0405week10_begin fp0405week10_end log
                     | year yw == 2005 && week yw == 11 =
                           isCompiledBetween fp0405week11_begin fp0405week11_end log
                     | year yw == 2005 && week yw == 12 =
                           isCompiledBetween fp0405week12_begin fp0405week12_end log
                     | year yw == 2005 && week yw == 13 =
                           isCompiledBetween fp0405week13_begin fp0405week13_end log
                     | year yw == 2005 && week yw == 14 =
                           isCompiledBetween fp0405week14_begin fp0405week14_end log

main = runNewResearchFromLogFile chkLogDateResearch startkey default_logfile default_outputpath HtmlFormat

chkLogDateResearch :: OutputFormat -> [(KeyHistory, [Logging])] -> FilePath -> IO ()
chkLogDateResearch format input outputpath = do
   let
      -- ana = runAnalysisGeneric logs key $ groupPerWeek
      name = "LogDate Research"
      researchdir = joinDirAndFileName outputpath (stringToFilePath name) ++ [pathSeparator]
      outputfile = joinDirFileExt researchdir (stringToFilePath name) $ showFormatExt format
      compareDates = (basicAnalysis "ori date & parsed + pp date" (\l -> (unparsedLogDate l, calTimeToString $ logDate l, unparsedLogDate l == (calTimeToString $ logDate l))))
      mappedCompareDates = mapAnalysis compareDates :: AnalysisFK KeyHistory [Logging] [(String, String, Bool)]
      chkLogDateSeq = basicAnalysis "filter on false elms" (filter ((==False). thd3)) 
                           <.> basicAnalysis "compare" (map (\(l1,l2) -> (unparsedLogDate l1, unparsedLogDate l2, (logDate l1) < (logDate l2)))) -- (map (uncurry (<)))
                           <.> basicAnalysis "twin loggings" (\logs -> zip (init logs) (tail logs))
                           <$> input
 

   createDirectoryIfMissing True researchdir

   -- show dates in table list

   writeFile outputfile
                  $ renderMarkUpDoc format $ document noHtml $
                     (section Nothing << (name ++ " Fp 0304 old data logfile"))
                     +++
                     (showAsMarkUpTable1D $
                           -- groupPerElement "log date" <.>
                           basicAnalysis "wrongly parsed logdates" (filter ((==False) . thd3)) <.>
                           mappedCompareDates 
                              <$> input
                     )  -- 7 sec
                     +++
                     (showAsMarkUpTable1D $ chkLogDateSeq -- 14 sec
                     )
                     +++
                     (showAsMarkUpTable1D $ chkLogDateSeq -- 14 sec
                     )
{-                     +++
                     (showAsMarkUpTable1D $
                           basicAnalysis "diff week infos" (filter ((==False) . thd3))
--                           <.> basicAnalysis "compare" (map (\(l1,l2) -> (weekOfYearInfo l1, weekOfYearInfo l2, (weekOfYearInfo l1) < (weekOfYearInfo l2)))) -- (map (uncurry (<)))
                           <.> basicAnalysis "compare" (map (\(l1,l2) -> ("nohting", "nohting", (weekOfYearInfo l1) < (weekOfYearInfo l2)))) -- (map (uncurry (<)))

                           <.> basicAnalysis "twin loggings" (\logs -> zip (init logs) (tail logs))
                           <$> input
                     )  -- 4m40 sec, in tuple, 2m27 with only the thd (ord cmp)
                     -}
{-                     +++
                     (showAsMarkUpTable1D $
                           basicAnalysis "diff week infos" (filter ((==False) . thd3))
                           <.> basicAnalysis "compare" (map (\(l1,l2) -> (weekOfYear $ stringToCalTime $ logDate l1, weekOfYear $ stringToCalTime $ logDate l2, (stringToCalTime $ logDate l1) == (stringToCalTime $ logDate l2))))
--                           <.> basicAnalysis "compare" (map (\(l1,l2) -> ("nohting", "nohting", (weekOfYear $ stringToCalTime $ logDate l1) < (weekOfYear $ stringToCalTime $ logDate l2)))) -- (map (uncurry (<)))
                           <.> basicAnalysis "twin loggings" (\logs -> zip (init logs) (tail logs))
                           <$> input
                     )  -- only thd cmp, 22 sec, 3 tuple. 29 sec
 -}
                     +++
                     (showAsMarkUpTable1D $
                               basicAnalysis "Only incorrect week numbers" (filter ((==False) . thd3))
                           <.> basicAnalysis "calc and cmp weekinfo" (map (\l -> (l, weekOfYear $ logDate l, chk_weekNum l (weekOfYear $ logDate l))))
                           <$> input
                     )  -- 44 s
                     +++
                     (showAsMarkUpTable1D $
                               basicAnalysis "no of logs" length
                           <.> groupPerWeek
                           <$> input
                     )  -- grpPerWeek with new 1m25, old 25m18

                     -- modlength research now takes 1m39 instead of 25 min.
                     
   putStrLn "parse date + 2x chk logdate seq + cal & cmp week info"

   return ()

