{-# OPTIONS_GHC -fallow-undecidable-instances #-}
module Thesis.Research.Common (
           module Thesis.Research.Common 
         , module Statistics.SummaryMeasures
         )  where

----
-- Common research actions, thesis related
----

import Statistics.DescriptiveAnalysis
import Statistics.Presentation
import Statistics.SummaryMeasures

import Thesis.Helium.LogfileParser
import Thesis.Helium.Logging
import TestLoggings2

import Thesis.Helium.Logging
import Grouping
import qualified Thesis.Grouping.Logging as LoggingGrouping (
                       --   groupPerSession_
                          groupPerWeekFromSeq_
                        , groupPerWeek_
--                        , groupPerPhase
                        , groupPerPhase_
                        )
import Thesis.Metrics
import Data.Typeable

-- for IO like operations
-- import Thesis.Research.ResearchGeneration hiding (generateResearches)
-- import qualified Thesis.Research.ResearchGeneration

import Auxilary
import Distribution.Compat.FilePath
import System.Directory(createDirectoryIfMissing)

import System.Time

-- Test Analysis data

testAnaData :: [(KeyHistory, [Logging])]  -- AnalysisInOut
testAnaData = kickStartAnalysis (log1:log2:logs) startkey

-- key tst
keytest = ordValueCount <.> mapAnalysis phaseAnalysis $ testAnaData
keytest2 = ordValueCount <.> mapAnalysis phaseAnalysis <.> groupPerStudent $ testAnaData
keytest3 = ordValueCount <.> mapAnalysis phaseAnalysis <.> groupPerStudent <.> groupPerPhase $ testAnaData
keytest4 = mapAnalysis phaseAnalysis <.> groupPerStudent <.> groupPerPhase $ testAnaData

k0 =  (map fst keytest4) !! 0  -- user peter
k1 =  (map fst keytest4) !! 1  -- user peter
k2 =  (map fst keytest4) !! 2

[(AggregateAnalysis str keys)] = getKHL $ fst $ head keytest
[(AggregateAnalysis str2 keys2)] = getKHL $ fst $ head keytest2

-- aggrkeys = getKHL $ fst $ head keytest

----
-- Usefull Analyses / Helper functions
----

type AnalysisKH a b = AnalysisFK KeyHistory a b 

nameDataSet :: String -> AnalysisKH [a] [a]
nameDataSet name = basicAnalysis' name id

concatAnalysis :: AnalysisFK KeyHistory [[a]] [a]
concatAnalysis = basicAnalysis "" concat

concatMapAnalysis :: AnalysisFK KeyHistory a [b] -> AnalysisFK KeyHistory [a] [b]
concatMapAnalysis fun =
         concatAnalysis
   <.>   mapAnalysis fun

----
-- Grouping helper
----

groupPerElement :: (Show a, DescriptiveKey key) => String -> AnalysisFK key [a] a
groupPerElement descr = basicAnalysis' "" head <.> groupAnalysis_  (\fstelm -> (descr, show fstelm)) (groupBy (\_ -> const False))

groupPerValue :: (DataInfo a, DescriptiveKey key) => AnalysisFK key [a] [a]
groupPerValue =
   groupAnalysis (id) (groupAllByEq (==))

groupPerValueRight :: (DescriptiveKey key, Eq a) => (a -> (String, String)) -> AnalysisFK key [a] [a]
groupPerValueRight grpdescrfun =
   groupAnalysis_ grpdescrfun (groupAllByEq (==))

-- does not encapsulate the grouping value
{-
groupPerValueSimple :: (DescriptiveKey key, DataInfo a) => AnalysisFK key [a] [a]
groupPerValueSimple =
   groupAnalysis_ (\elms -> (showType elms, showValue elms)) (groupAllByEq (==))
-}
groupPerValueUnder ::(DataInfo b, DescriptiveKey key) => (a -> b) -> AnalysisFK key [a] [a]
groupPerValueUnder elmchar {- elementairy elmcharacteristic -} =
   -- groupAnalysis_ (\fstelm -> (showType $ elmchar fstelm, showValue $ elmchar fstelm)) (groupAllUnder elmchar)
   groupAnalysis elmchar (groupAllUnder elmchar)

----

----
-- Helium logging attribute grouping
----

groupPerLogging :: (DescriptiveKey key) => AnalysisFK key [Logging] Logging
groupPerLogging = groupPerElement "Logging"

{-
-- groupPerSession :: (DescriptiveKey key) => Statistics.Analysis.DescriptiveAnalysis.Prim.AnalysisFK key [Logging] [Logging]
groupPerSession :: AnalysisFK KeyHistory [Logging] [Logging]
groupPerSession =
   groupAnalysis''_ (\grps -> zip (repeat "Session") (map (\no -> "Session" ++ show no) [0..])) groupPerSession_
-}
groupPerStudent :: AnalysisKH [Logging] [Logging]
groupPerStudent =
   groupAnalysis_ (\fstelm -> ("Username", show $ username fstelm)) (groupAllUnder username)
--   groupAnalysis (username) (groupAllUnder username)

-- group per student, supressing to influence the title generation
groupPerStudentSilent :: AnalysisKH [Logging] [Logging]
groupPerStudentSilent =
   groupAnalysisSilent (groupAllUnder username)


groupPerUsername = groupPerStudent

groupPerPhase :: (DescriptiveKey key) => AnalysisFK key [Logging] [Logging]
groupPerPhase = 
   groupPerValueUnder phase

-- end helium logging attribute grouping

----
-- Time based grouping
----

groupPerWeek :: DescriptiveKey key => AnalysisFK key [Logging] [Logging]
groupPerWeek =
   groupAnalysis (weekOfYear . logDate) (LoggingGrouping.groupPerWeek_)

-- group per week, but include missing intermediate values
groupPerWeek' :: DescriptiveKey key => AnalysisFK key [Logging] [Logging]
groupPerWeek' =
   groupAnalysis'' -- :: (DescriptiveKey key, DataInfo b) => ([[a]] -> [b]) -> ([a] -> [[a]]) -> AnalysisFK key [a] [a]
      (undefined :: ([[Logging]]->[Year_Week]))
      (\logs -> if null logs then []
                else
                  let grpPerWk = LoggingGrouping.groupPerWeek_ logs
                      fstwk = (weekOfYear . logDate . head) $ head grpPerWk
                      lastwk = (weekOfYear . logDate . head) $ last grpPerWk
                  in undefined
      )


-- groupPerWeek = groupPerWeek_

-- group on string value
groupPerWeek_ :: ({-DataInfo a, -}DescriptiveKey key) => AnalysisFK key [Logging] [Logging]
groupPerWeek_ =
   groupAnalysis_ (\elms -> (showType $ m1_1_13 elms, showValue $ m1_1_13 elms)) (LoggingGrouping.groupPerWeek_)

groupPerWeekFromSeq :: ({-DataInfo a, -}DescriptiveKey key) => AnalysisFK key [Logging] [Logging]
groupPerWeekFromSeq =
   groupAnalysis_ (\elms -> (showType $ m1_1_13 elms, showValue $ m1_1_13 elms)) (LoggingGrouping.groupPerWeekFromSeq_)

instance DataInfo Day where
   dataKind _ = Ordinal
   showType _ = "weekday"
   shortname = showType
   ordIndex = fromEnum

instance Typeable Day where
   typeOf = error "no type rep defined for System.Time.Day"

getWeekDay = ctWDay . logDate

groupByWeekDay :: [Logging] -> [[Logging]]
groupByWeekDay = 
      groupAllByOrd (\l1 l2 -> compare (getWeekDay l1) (getWeekDay l2))

groupPerWeekDay :: AnalysisFK KeyHistory [Logging] [Logging]
groupPerWeekDay = 
   groupAnalysis getWeekDay groupByWeekDay

---- end time based grouping

----
-- Basic calculations analyses
----

calculateSize :: (DescriptiveKey key) => AnalysisFK key [a] Int
calculateSize = basicAnalysis "group size" length

calculateFrequency :: (DescriptiveKey key) => AnalysisFK key [a] Int
calculateFrequency = basicAnalysis "frequency" length

-- calculateNumber_OfElements :: (DescriptiveKey key) => AnalysisFK key [a] Int
calculateNumber_OfElements :: AnalysisFK KeyHistory [a] Int
calculateNumber_OfElements = basicAnalysis "number" length

countNumberOfLoggings   ::    DescriptiveKey key => AnalysisFK key [a] Int
countNumberOfLoggings   =     basicAnalysis "number of loggings" length

-- flat error count
countFrequency :: (DataInfo a, DescriptiveKey key) => AnalysisFK key [a] Int
countFrequency = calculateSize
                    <.>
                 groupPerValue

-- Count the amount of occurences of a metric result in a input set
countMetricFrequency :: (DataInfo b, DescriptiveKey key) => AnalysisFK key a b -> AnalysisFK key [a] Int
countMetricFrequency metric = countFrequency <.> mapAnalysis metric

-- Summary Statistics

setSizeAnalysis :: (DescriptiveKey key) => AnalysisFK key [a] Int
setSizeAnalysis = basicAnalysis "size of set" length

modeNomAnalysis :: (DescriptiveKey key, Eq a) => AnalysisFK key [a] (OptData [a])
modeNomAnalysis = basicAnalysis "mode" modeNomMultiple

modeAnalysis :: (DescriptiveKey key, Ord a) => AnalysisFK key [a] (OptData a)
modeAnalysis = basicAnalysis "mode" mode

-- SingleOrBetween a, defined in DataInfo.hs
medianOrdAnalysis :: (DescriptiveKey key, Ord a) => AnalysisFK key [a] (OptData (SingleOrBetween a))
medianOrdAnalysis =
   basicAnalysis' "" (optdata NaN (Value . fromEitherToSingleOrBetween)) <.> basicAnalysis' "median" medianOrd

medianAnalysis :: (DescriptiveKey key, Real a) => AnalysisFK key [a] (OptData Float)
medianAnalysis = basicAnalysis "median" median

meanAnalysis :: (DescriptiveKey key, Real a {-, DataInfo a -}) => AnalysisFK key [a] (OptData Float)
meanAnalysis = basicAnalysis "mean" mean

averageAnalysis :: (DescriptiveKey key, Real a) => AnalysisFK key [a] (OptData Float)
averageAnalysis = basicAnalysis "average" mean

----
-- Combined overview statistic analyses
----

-- nominal values are not stricly in Ord, but for standard presentation (ordering) it is usefull anyway
nomSummaryStats :: (DescriptiveKey key, Nominal a {-, Ord a -}) => AnalysisFK key [a] (Int) -- , OptData [a])
nomSummaryStats = setSizeAnalysis --  <&> modeNomAnalysis

ordSummaryStats :: (DescriptiveKey key, Ordinal a) => AnalysisFK key [a] ((Int, OptData a), OptData (SingleOrBetween a))
ordSummaryStats = setSizeAnalysis <&> modeAnalysis <&> medianOrdAnalysis 

intSummaryStats :: (DescriptiveKey key, RatioValue a) => AnalysisFK key [a] (((Int, OptData a),  OptData Float), OptData Float)
intSummaryStats = setSizeAnalysis <&> modeAnalysis <&> medianAnalysis <&> meanAnalysis 

ratSummaryStats :: (DescriptiveKey key, RatioValue a) => AnalysisFK key [a] (((Int, OptData a),  OptData Float), OptData Float)
ratSummaryStats = intSummaryStats

-- to calculate summary statistics over all grouped values, untested
nomAllGroupsSummaryStats :: (DescriptiveKey key, Nominal a {-, Ord a -}) => AnalysisFK key a (Int) --, OptData [a])
nomAllGroupsSummaryStats = nomSummaryStats <.> aggregateAnalysis'' "" id

-- allGroupsSummaryStats :: (DescriptiveKey key, Nominal a, Ord a) => AnalysisFK key a (Int, OptData a)
-- allGroupsSummaryStats statana = statana <.> aggregateAnalysis'' "" id

-- etc

-- to Wiki: old, then use description / shortname to set column name, or analyse keyhistory, parsing... :(
calculateBasicStats :: (Real a, Fractional b) => AnalysisFK KeyHistory [a] (Int, OptData (b, a, a, Float, OptData a))
calculateBasicStats = basicAnalysis' "Basic Statistics: (freq, mean, min, max, median, mode)" giveBasicStats

--

----
-- Combined overview statistic analyses presentations
----

-- KeyHistory dependant display
nomSummarytable :: (Nominal a, DataInfo (Int, OptData [a])) => [(KeyHistory, [a])] -> MarkUpDoc
nomSummarytable result = 
               (showAsMarkUpTable1D $ nomSummaryStats result)

ordSummarytable :: (Ordinal a, DataInfo a) => [(KeyHistory, [a])] -> MarkUpDoc
ordSummarytable result = 
               (showAsMarkUpTable1D $ ordSummaryStats result)

intSummarytable :: (Interval a, DataInfo (Int, OptData a)) => [(KeyHistory, [a])] -> MarkUpDoc
intSummarytable result = 
               (showAsMarkUpTable1D $ intSummaryStats result)

ratSummarytable :: (RatioValue a, DataInfo (Int, OptData a)) => [(KeyHistory, [a])] -> MarkUpDoc
ratSummarytable result = 
               (showAsMarkUpTable1D $ ratSummaryStats result)

---


----
-- Combined analyses
----

-- tupling examples:
-- tupling can be used to mimick grouping, but the result is to be handled explicitely by the next analysis
-- so taking avg, etc. more work to implement

-- not as usefull, as the above, since that one, will more likely to produce standard output
-- this is because the grouping info/subject is encapsulated in the key, and used for later reference
numberOfLoggingsPerPhase :: (DescriptiveKey key) => AnalysisFK key [Logging] (Phase,Int)
numberOfLoggingsPerPhase = 
    (basicAnalysis "phase" (phase . head) <&> calculateSize) <.> groupPerPhase



-- calculating avg (if tupled only once) now should go like:
averageOverall :: (DescriptiveKey key, DataInfo a) => AnalysisFK key (a,Int) (OptData Float)
averageOverall = aggregateAnalysis "avg" (mean . complementData . (map snd))

-- from Datainfo
complementData = id -- undefined

avgNoLogsPerPhase :: (DescriptiveKey key) => AnalysisFK key [Logging] (OptData Float)
avgNoLogsPerPhase = averageOverall <.> numberOfLoggingsPerPhase

numberOfLoggingsPerPhasePerGrp :: (DescriptiveKey key) => AnalysisFK key [Logging] [(Phase,Int)]
numberOfLoggingsPerPhasePerGrp = 
    mkAnaGrpIndependant $ aggregateAnalysis "" id <.> (basicAnalysis "phase" (phase . head) <&> calculateSize) <.> groupPerPhase
{-
numberOfLoggingsPerPhasePerGrp ::
   (DescriptiveKey key, Ord a) => AnalysisFK key Logging a -> AnalysisFK key [Logging] [(Phase,Int)]
numberOfLoggingsPerPhasePerGrp ana = 
       basicAnalysis "phase" (map (\logsph -> (phase $ head logsph, length logsph)))
   <.> basicAnalysis "group per phase" groupPerPhase_
  undefined --  (basicAnalysis "phase" (map ana . head) <&> basicAnalysis "size" (map length)) <.> basicAnalysis "group per phase" groupPerPhase_
-}

ordValueCountFromAnalysis :: (DescriptiveKey key, DataInfo a, Ord a) => AnalysisFK key Logging a -> AnalysisFK key [Logging] [(a,Int)]
ordValueCountFromAnalysis ordana = ordValueCount <.> mapAnalysis ordana

ordValueCount  ::    (DescriptiveKey key, Ord a, DataInfo a) => AnalysisFK key [a] [(a,Int)]
ordValueCount  = 
       isolateAnalysis $
         aggregateAnalysis "value frequency" id <.> (basicAnalysis "" head <&> calculateSize) <.> groupPerValue

ordValuePercentages :: (DescriptiveKey key, Ord a, DataInfo a) => AnalysisFK key [a] [(a,Float)]
ordValuePercentages = relativeValueCount <.> ordValueCount

-- relativeValueCount :: (DescriptiveKey key) => AnalysisFK key [(a, Int)] [(a,Float)]
-- Real and Frac classes to generalize the function
relativeValueCount   ::    (DescriptiveKey key, Real b, Real c, Fractional c) => AnalysisFK key [(a, b)] [(a, c)]
relativeValueCount   =     basicAnalysis "percentages" relativeCount

----
-- Experimental combinators
----

-- moved to PrimAC, combinators for handling isolated analysis
mkAnaGrpIndependant :: Analysis keyb b keyc c -> Analysis keyb b keyc c
mkAnaGrpIndependant = isolateAnalysis 

-- flattenAnalysis for DescriptiveKey, a key type independant variant is in PrimAC.hs
flattenAnalysisDK :: (DataInfo b, DescriptiveKey key) => AnalysisFK key [(b, a)] a
flattenAnalysisDK = basicAnalysis "" (snd .  head) <.> groupAnalysis fst (split)
   where
      split :: [x] -> [[x]]
      split list = [[x] | x <- list]

flattenAnalysis' :: (DataInfo b, DescriptiveKey key) => AnalysisFK key [(b, a)] a
flattenAnalysis' = flattenAnalysisDK

----
-- Presentation / analyses helpers
----

-- key dependant show
showLoggingListTables :: (Show a, DataInfo b) => AnalysisFK KeyHistory a b -> [(KeyHistory, [a])] -> [MarkUpDoc]
showLoggingListTables metric = map (showAsMarkUpTable1D . (metric <.> groupPerElement "Logging")) . splitAnalysis

-- to be removed
researchPerStudent :: (OutputFormat -> [(KeyHistory, [Logging])] -> FilePath -> IO ()) -> (OutputFormat -> [(KeyHistory, [Logging])] -> FilePath -> IO ())
researchPerStudent research format input outputpath = do
   let
      students = splitAnalysis $ groupPerStudent <$> input
   mapM_ (\(studentdata, num) -> research format studentdata (joinPaths outputpath (show num))) $ zip students [0..]

researchPerDataSet ::
   (String -> OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO a) ->
      String -> OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO [a] 
researchPerDataSet research grpname format outputpath input = do
   mapM (\(dataset, num) ->
            research (unwords (grpname:[show num])) format outputpath dataset) $
               zip (splitAnalysis $ input) [0..]

-- to be removed
-- map an presentation function, each in a new directory, as safeguard against over writing
-- particular usefull, when rendering rel. stacked bar charts, since the title rendering is not complete
mapRendering :: (FilePath -> [(KeyHistory, a)] -> IO (FilePath, String)) -> FilePath -> [[(KeyHistory, a)]] -> IO [(FilePath,String)]
mapRendering renderfun path inputs =
   mapM (\(num, input) -> do let newpath = (path ++ show num)
                             createDirectoryIfMissing True newpath
                             renderfun newpath input) $
        zip [0..length inputs] inputs

        

----
-- Usefull Research Actions
----

-- for running plain IO like researches

-- BasicTypes: type Research key a = ([a] -> key -> FilePath -> IO ())
-- runAnalysis ::  a -> keya -> Analysis keya a keyb b -> [(keyb, b)]
testAnalysisWithLogFile :: ([(KeyHistory, [Logging])] -> [(KeyHistory, b)]) -> FilePath -> FilePath -> IO [(KeyHistory,b)]
testAnalysisWithLogFile ana logfile outputpath =
   do
      logs <- parseLogfile logfile
      return $ runAnalysis logs ana

runResearchFromLogFile :: Research key Logging -> key -> FilePath -> FilePath -> IO ()
runResearchFromLogFile research key logfile outputpath =
   do
      logs <- parseLogfile logfile
      research logs key outputpath

runResearchFromLogFileKeyable :: (Keyable key) => Research key Logging -> FilePath -> FilePath -> IO ()
runResearchFromLogFileKeyable research logfile outputpath =
   runResearchFromLogFile research startkey logfile outputpath

runResearchOld :: ([Logging] -> key -> FilePath -> IO ()) -> [Logging] -> key -> FilePath -> IO ()
runResearchOld research = research -- logs key

runResearchKeyable :: (Keyable key) => ([Logging] -> key -> FilePath -> IO ()) -> [Logging] -> FilePath -> IO ()
runResearchKeyable research logs = runResearchOld  research logs startkey

runResearchKeyHistory :: Research KeyHistory Logging -> [Logging] -> FilePath -> IO ()
runResearchKeyHistory research logs fp = runResearchKeyable research logs fp

type NewResearch key a = (OutputFormat -> [(key,[a])] -> FilePath -> IO ())

runNewResearchFromLogFile :: NewResearch key Logging -> key -> FilePath -> FilePath -> OutputFormat -> IO ()
runNewResearchFromLogFile research key logfile outputpath format =
   do
      logs <- parseLogfile logfile
      research format (kickStartAnalysis logs key) outputpath

runResearch = runNewResearchFromLogFile

type NewResearch' key a = (OutputFormat -> FilePath -> [(key,[a])] -> IO ())

runNewResearchFromLogFile' ::
   NewResearch' KeyHistory Logging -> KeyHistory -> FilePath -> FilePath -> OutputFormat -> IO ()
runNewResearchFromLogFile' research key logfile outputpath format =
   do
      logs <- parseLogfile logfile
      research format outputpath (kickStartAnalysis logs key) 

inputAnalysis :: AnalysisFK KeyHistory a ()
inputAnalysis = basicAnalysis "data set" (const ())

----
-- Markuphelpers
----

plotToFigure :: (FilePath , String) -> MarkUpDoc
plotToFigure plotdetails =
      figure Nothing (snd plotdetails)
             (fst $ splitFileName $ fst plotdetails, snd $ splitFileName $ fst plotdetails)

plotToFigure' :: FilePath -> (FilePath , String) -> MarkUpDoc
plotToFigure' basedir (fullpath, descr) =
      figure Nothing descr
             (basedir, rel_img_dir fullpath basedir)
   -- where

rel_img_dir fpath basdir =
         dropAbsolutePrefix $ dropPrefix (removePostFixedSlashes $ correctPath basdir) $ correctPath fpath

-- test = dropPrefix

