module Thesis.Research.CompilationDistribution where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.SummaryMeasures
import Statistics.Presentation

import Thesis.Metrics
import Thesis.Research.Common
import Thesis.Helium.Logging
import Thesis.Grouping.Logging (sameSourceFile)

import Grouping
import Data.Typeable

import Auxilary

import Data.Maybe
import Data.List
import System.Time

import Text.Regex

{-

verdeling van tijd tussen compilaties (gemiddeld per student)

-}

default_logfile    = "datasets/demo/demo.log"
default_outputpath = "analysis_results/demo"

-- logTimeDiff :: Logging -> Logging -> TimeDiff
-- logTimeDiff l1 l2 = diffCalendarTimes (stringToCalTime $ logDate l2) (stringToCalTime $ logDate l1)

logTimeDiff :: Logging -> Logging -> TimeDiff
logTimeDiff l1 l2 = diffCalendarTimes (logDate l2) (logDate l1)

-- to set an upper bound
calcTimeCoherentSessions :: Int -> AnalysisFK KeyHistory [Logging] [[Logging]]
calcTimeCoherentSessions min =
   basicAnalysis ("time coherence of " ++ show min ++ " min")
                 (groupByCoherence  (\l1 l2 -> logTimeDiff l1 l2 < minutesOfTD min))

tuplePerPair :: [a] -> [(a,a)]
tuplePerPair input = zip (init input) (tail input)

tuplePerTwoLoggings        ::    [Logging] -> [(Logging, Logging)]
tuplePerTwoLoggings input  =     zip (init input) (tail input)

calculateCompilationTimeDiffs :: Int -> AnalysisFK KeyHistory [Logging] [Float]
calculateCompilationTimeDiffs min =
                mapAnalysis (
                   basicAnalysis  "relative time difference"
                                  (timeDiffToMin . uncurry logTimeDiff)
                )
         <.>   concatMapAnalysis (basicAnalysis "tupling per two loggings" tuplePerPair)
         <.>   calcTimeCoherentSessions min

-- Abstraction used in TypingTillCodeGen analysis
compilationTimeDiff ::   Int ->   Int  ->  AnalysisFK KeyHistory [Float]   (OptData b)
                                       ->  AnalysisFK KeyHistory [Logging] [OptData b]
compilationTimeDiff minsize min aggregateTDAnaPerStudent  =
               aggregateAnalysis "" id
         <.>   aggregateTDAnaPerStudent
         <.>   calculateCompilationTimeDiffs min
         <.>   selectGrpAnalysis'_ ((> minsize) . length)
         <.>   groupPerStudent

-- <.>   selectGrpAnalysis ("min size: " ++ show minsize) ((> minsize) . length)

avgCompilationTimeDiff     =  compilationTimeDiff 10 60 averageAnalysis
medianCompilationTimeDiff  =  compilationTimeDiff 10 60 medianAnalysis

avgCompilationTimeDiffPerWeek =
               avgCompilationTimeDiff
       <..>    groupPerWeek

medianCompilationTimeDiffPerWeek =
               medianCompilationTimeDiff
       <..>    groupPerWeek

main = runNewResearchFromLogFile complDistResearch startkey default_logfile default_outputpath HtmlFormat

-- its also easy to generate a bee swarm of this data:
--  pl -prefab vdist data=data6 y=1  mediansym=dot  dist=hist \
--        showoutliers=yes

complDistResearch :: OutputFormat -> [(KeyHistory, [Logging])] -> FilePath -> IO ()
complDistResearch format input outputpath = do
   let
      name = "2. Compilation Distribution"
      researchdir = joinDirAndFileName outputpath (stringToFilePath name) ++ [pathSeparator]
      outputfile = joinDirFileExt researchdir (stringToFilePath name) $ showFormatExt format

   createDirectoryIfMissing True researchdir

   boxplotAvg <- renderBoxPlot researchdir $ avgCompilationTimeDiff <$> input
   boxplotAvgPerWeek <- renderBoxPlot researchdir $ avgCompilationTimeDiffPerWeek <$> input

   boxplotMedian <- renderBoxPlot researchdir $ medianCompilationTimeDiff <$> input
   boxplotMedianPerWeek <- renderBoxPlot researchdir $ medianCompilationTimeDiffPerWeek <$> input
   
   -- show result data in tables
   -- putStrLn  (unlines $ map (showMaybeData . snd) $ avgCompilationDiff <$> input)
   writeFile (researchdir++"/distr_data.txt") (unlines $ map (show {-showMaybeData -}. snd) $ avgCompilationTimeDiff <$> input)

   writeFile outputfile
      $ renderMarkUpDoc format $ document noHtml $
         (section Nothing << name)
         +++
         plotToFigure boxplotAvg
         +++
         plotToFigure boxplotAvgPerWeek
         +++
         plotToFigure boxplotMedian
         +++
         plotToFigure boxplotMedianPerWeek
--          (showAsMarkUpTable1D $ avgCompilationDiff <$> input)
--          +++
--          (showAsMarkUpTable1D $ (mapAnalysis (mapAnalysis (
--                basicAnalysis "relative time diff"
--                   (\(l1,l2) -> (l1, l2, timeDiffToMin $ normalizeTimeDiff $ diffCalendarTimes (stringToCalTime $ logDate $ l2) (stringToCalTime $ logDate $ l1)))  -- [[TD]]
--                )
--              ) ) -- :: AnalysisFK KeyHistory [[(Logging, Logging)]] [[Float]])
--            <.>             (mapAnalysis $ basicAnalysis "tupling per two loggings" tuplePerTwo)
--                <.> groupPerCoherenceTime  -- [[L]]
--                <.> groupPerStudent
--                <.> basicAnalysis "onlye group63" (filter ((==Username "group63").username))
--                <$> input
--          )
--          +++
--          (showAsMarkUpTable1D $
--             (mapAnalysis $ basicAnalysis "tupling per two loggings" tuplePerTwo)
--                <.> groupPerCoherenceTime  -- [[L]]
--                <.> groupPerStudent
--                <.> basicAnalysis "onlye group63" (filter ((==Username "group63").username))
--                <$> input
--          )
--          +++
--          (showAsMarkUpTable1D $
--             (groupPerCoherenceTime :: AnalysisFK KeyHistory [Logging] [[Logging]])
--             <.> groupPerStudent
--             <.> basicAnalysis "onlye group63" (filter ((==Username "group63").username))
--             <$> input)
   return ()

logy = Log {username = Username "group71", phase = CodeGen, heliumVersion = HeliumVersion {heliumVersionNumber = "1.5", buildDateRef = "Thu-Jan-27-15_37_27-RST-2005"}, logPath = Path {baseDir = "/scratch/data/fp/archives/fp-0405-p3", userDir = "group71", loggingDir = "2005-01-23@14_14_36_192", sourceFile = "Hello.hs"}, logDate = stringToCalTime "2005-01-23@14_14_36_192", staticErrorCodes = [], extraLoggingInfo = []}

logx = Log {username = Username "group71", phase = CodeGen, heliumVersion = HeliumVersion {heliumVersionNumber = "1.5", buildDateRef = "Thu-Jan-27-15_37_27-RST-2005"}, logPath = Path {baseDir = "/scratch/data/fp/archives/fp-0405-p3", userDir = "group71", loggingDir = "2005-01-07@16_06_42_557", sourceFile = "Hello.hs"}, logDate = stringToCalTime "2005-01-07@16_06_42_557", staticErrorCodes = [], extraLoggingInfo = []}
