module Thesis.Research.ModuleLengthResearch where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.Presentation

import Thesis.Metrics
import Thesis.Research.Common  -- getWeekDay, groupByWeekDay, groupPerWeekDay
import Thesis.Research.AbstractIntMetricResearch
import Thesis.Helium.Logging
import Data.Typeable

import Statistics.BasicTypes

import Auxilary
import Grouping

import Data.Maybe
import System.Time

{-

Length of Module analysis or Lines of Code (LOC/m1_1_1)

-}
        
default_logfile    = "datasets/demo/demo.log"
default_outputpath = "analysis_results/demo"

locMetric   ::    AnalysisFK KeyHistory Logging Int
locMetric   =     basicAnalysis "module length (LOC)" (length . lines . sourcecode)

main = runNewResearchFromLogFile' moduleLengthResearchOverall startkey default_logfile default_outputpath HtmlFormat

researchname = "Module Length Research"

moduleLengthResearchOverall :: OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO ()
moduleLengthResearchOverall format outputpath input = do
   let
      researchref = stringToFilePath researchname
      researchdir = outputpath ++ "/" ++ researchref  ++ "/"
      outputfile = researchdir ++ researchref  ++ "." ++ (showFormatExt format)

   createDirectoryIfMissing True researchdir

   research <- moduleLengthResearch [] format researchdir input
   
   writeFile outputfile
             (renderMarkUpDoc format $ document noHtml $ research)
   return ()

moduleLengthResearchPerStudent :: OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO ()
moduleLengthResearchPerStudent format outputpath input = do
   let
      researchref = stringToFilePath (researchname ++ "perstudent")
      researchdir = outputpath ++ "/" ++ researchref  ++ "/"
      outputfile = researchdir ++ researchref  ++ "." ++ (showFormatExt format)

   createDirectoryIfMissing True researchdir

   researchPerStudent <-
      researchPerDataSet moduleLengthResearch "student" format researchdir ({-take 10 $ -}groupPerStudent <$> input)

   writeFile outputfile
             (renderMarkUpDoc format $ document noHtml $
                  concatMarkUp $ researchPerStudent)

   return ()

-- actual research
moduleLengthResearch :: String -> OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO MarkUpDoc
moduleLengthResearch partname format outputpath input = do
   let
--       pathref = stringToFilePath partname
--       researchdir = (joinPaths outputpath pathref) ++ "/"
      
   -- createDirectoryIfMissing True researchdir

   locResearch <- abstractIntMetricResearchInTime locMetric partname format input outputpath
   
   return $
                    (section Nothing ! [formatonly $ show HtmlFormat] << (unwords (researchname:[partname])))
                     +++
                    locResearch
