module Thesis.Research.PhaseResearch where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.Presentation

import Thesis.Metrics
import Thesis.Research.Common
import Thesis.Helium.Logging
import Data.Typeable

import Auxilary

import Data.Maybe
import Control.Monad

{-

5. Bepaal het percentage van logs in iedere afzonderlijke fase op het  
totale aantal logs per week.
      Doe dit voor de jaren afzonderlijk, zodat we kunnen zien of de  
trends hetzelfde zijn.

-}

default_logfile    = "datasets/demo/demo.log"
default_outputpath = "analysis_results/demo"

main = runNewResearchFromLogFile' phaseResearchOverall startkey default_logfile default_outputpath LaTeXFormat

anyfrom :: Eq a => [a] -> a -> Bool
anyfrom = flip elem

mainPhasesAnalysis :: AnalysisFK KeyHistory [Logging] [Logging]
mainPhasesAnalysis = basicAnalysis "" (filter (anyfrom mainphases . phase))
   where
      mainphases = [Lexical, Parsing, Static, Typing, CodeGen]

errorphases = [Lexical, Parsing, Static, Typing]

errorPhasesAnalysis :: AnalysisFK KeyHistory [Logging] [Logging]
errorPhasesAnalysis = basicAnalysis "" (filter ((`elem` errorphases) . phase))

phaseMetric :: AnalysisFK KeyHistory Logging Phase
phaseMetric = basicAnalysis "phase" phase

loggingsPerPhase :: AnalysisFK KeyHistory [Logging] Int
loggingsPerPhase =
        countNumberOfLoggings
    <.> groupPerPhase
    <.> mainPhasesAnalysis 

loggingsPerPhasePerWeek :: AnalysisFK KeyHistory [Logging] Int
loggingsPerPhasePerWeek =
        loggingsPerPhase
    <.> groupPerWeek

-- gen. inproper titles
{-
loggingsPerPhasePerWeek' :: AnalysisFK KeyHistory [Logging] [(Phase, Int)]
loggingsPerPhasePerWeek' =
        basicAnalysis "number of" (map $ fmapTupleSnd length)
    <.> basicAnalysis "phases" Thesis.Grouping.Logging.groupPerPhase
    <.> groupPerWeek-}

-- see thesis
-- relativeValueCountAnalysis :: (DataInfo a, Ord a) => AnalysisFK KeyHistory Logging a -> AnalysisFK KeyHistory [Logging] [(a, Float)]
-- relativeValueCountAnalysis metric = relativeValueCount
--                             <.> basicAnalysis ""
--                             <.> mapAnalysis metric

-- Using Ord instance of Phase data type, to calculate the phase ratio
relLoggingsPerOrdPhasePerWeek :: AnalysisFK KeyHistory [Logging] [(Phase, Float)]
relLoggingsPerOrdPhasePerWeek = relativeValueCount
                            <.> ordValueCount
                            <.> mapAnalysis phaseAnalysis
                            <.> groupPerWeek
                            <.> mainPhasesAnalysis 

-- combined rel and abs
data AbsRelValue = AbsRel Int Float

instance Show AbsRelValue where
   show (AbsRel abs rel) = show abs ++ " (" ++ show rel ++ "%)"

-- for table with combined abs/rel values
absRelAnalysis :: AnalysisFK KeyHistory [Logging] [(Phase, AbsRelValue)]
absRelAnalysis =
       (basicAnalysis "" (map (\(absT,relT) -> (fst absT, AbsRel (snd absT) (snd relT))) . uncurry zip))
                        <.>  ((basicAnalysis "" id) <&> relativeValueCount)
                        <.>  ordValueCount  -- [(a,Int)]
                        <.>  mapAnalysis phaseAnalysis
                        <.>  groupPerWeek
                        <.>  mainPhasesAnalysis

researchname = "Phase Research"

phaseResearchOverall :: OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO ()
phaseResearchOverall format outputpath input = do
   let
      researchref = stringToFilePath researchname
      researchdir = outputpath ++ "/" ++ researchref  ++ "/"
      outputfile = researchdir ++ researchref  ++ "." ++ (showFormatExt format)

   createDirectoryIfMissing True researchdir

   research <- phaseResearch [] format researchdir input
   
   writeFile outputfile
             (renderMarkUpDoc format $ document noHtml $ research)
   return ()

phaseResearchPerStudent format outputpath input = do
   let
      researchref = stringToFilePath (researchname ++ "perstudent")
      researchdir = outputpath ++ "/" ++ researchref  ++ "/"
      outputfile = researchdir ++ researchref  ++ "." ++ (showFormatExt format)

   createDirectoryIfMissing True researchdir

   researchPerStudent <-
      researchPerDataSet phaseResearch "student" format researchdir $ groupPerStudent <$> input

   writeFile outputfile
             (renderMarkUpDoc format $ document noHtml $
                  concatMarkUp $ researchPerStudent)

   return ()

-- actual research
phaseResearch :: String -> OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO MarkUpDoc
phaseResearch partname format outputpath input = do
   let
      pathref = stringToFilePath partname
      researchdir = (joinPaths outputpath pathref) ++ "/"

   createDirectoryIfMissing True researchdir

-- slechte title door flattenAnalysis, maar laat tochniet zo veel zinnigs zien,
--    barPlotPhaseOverall <-
--          renderBarChart researchdir $ flattenAnalysis' <.> ordValueCount <$> anaResult
--    barPlotPhaseRelOverall <- renderBarChart researchdir $ flattenAnalysis' <.> relativeValueCount <.> ordValueCount <$> anaResult

   barPlotPhaseOverall <- renderBarChart researchdir $ loggingsPerPhase <$> input
   barPlotPhasesPerWeek <- renderBarChartDynamic researchdir (loggingsPerPhasePerWeek input)
--    barPlotPhasesPerWeekRelative <- renderStackedBarChart researchdir $ relLoggingsPerOrdPhasePerWeek <$> input
--    barPlotErrorPhasesPerWeekRelative <-
--       renderStackedBarChart researchdir $ relLoggingsPerOrdPhasePerWeek <.> errorPhasesAnalysis <$> input
   barPlotPhasesPerWeekRelative <- renderRelativeStackedBarChart researchdir $ loggingsPerPhasePerWeek <$> input

   -- for inclusion in thesis
   when (format == LaTeXFormat || format == LaTeXFormatInclude) $
      do {-writeFile (outputpath ++ "/tableAbsoluteLoggingsPerPhase.tex") $ renderLateX (showAsTable1D $ loggingsPerPhase <$> input)
         writeFile (outputpath ++ "/tableAbsoluteLoggingsPerPhasePerWeek.tex") $ renderLateX (showAsTableDynamic $ loggingsPerPhasePerWeek <$> input)
         writeFile (outputpath ++ "/tableRelativeLoggingsPerPhasePerWeek.tex") $ renderLateX (showAsTable2D $ relLoggingsPerOrdPhasePerWeek <$> input)-}
         writeFile (outputpath ++ "/tableAbsRelLoggingsPerPhasePerWeek.tex") $ renderLateX (showAsTable2D $ absRelAnalysis <$> input)
         return ()
      
   return      $
                     (section Nothing << (unwords (researchname:[partname])))
                        +++
                     plotToFigure' outputpath barPlotPhaseOverall
                        +++
                     (showAsTable1D $ loggingsPerPhase <$> input)
                        +++
                     plotToFigure' outputpath barPlotPhasesPerWeek
                     +++
                     (showAsTableDynamic $ loggingsPerPhasePerWeek <$> input)
                     +++
                     plotToFigure' outputpath  barPlotPhasesPerWeekRelative
                     +++
                     (showAsTable2D $ relLoggingsPerOrdPhasePerWeek <$> input)
                     +++
                     (showAsTable2D $ absRelAnalysis <$> input)
--                      +++
--                      plotToFigure' outputpath barPlotErrorPhasesPerWeekRelative
--                      +++
--                      (showAsTable2D $ relLoggingsPerOrdPhasePerWeek <.> errorPhasesAnalysis <$> input)
--                      +++
--                      (ordSummarytable $ mapAnalysis phaseAnalysis <.> groupPerWeek <$> input)

phaseResearch_ :: String -> OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO ()
phaseResearch_ partname format outputpath input = do
   let
      pathref = stringToFilePath partname
      researchdir = (joinPaths outputpath pathref) ++ "/"
   barPlotPhasesPerWeek <- renderBarChartDynamic researchdir (loggingsPerPhasePerWeek input)
   return ()


