module Thesis.Research.ThesisACExample where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.Presentation
import Statistics.BasicTypes
import Statistics.SummaryMeasures

import Thesis.Research.Common
import Thesis.Helium.Logging

import Grouping
import Auxilary

import Data.Maybe

{-

Example for using analysis combinators

The thesisACExample function is a template, to extend for new analyses.

The thesisACExample result is used in the thesis document in \subsection{Analysing using Statistical Combinators}, also described in 01-CombinatorExample.lhs as part of thesis

-}


name = "Thesis AC Example"
outputpath = "analysis_results/demo"

default_logfile    = "datasets/demo/demo.log"

thesisACExample :: {- (DataInfo key, DescriptiveKey key) => -} [Logging] -> KeyHistory -> FilePath -> IO ()
thesisACExample logs key outputpath =
   do
      let
         runAna = runAnalysisGeneric logs key
         logFreqPerStudentPerPhase =
            runAna $
               calculateNumber_OfElements <.> groupPerPhaseAna <.>
                  groupPerStudent <.> nameDataSet "Loggings (from FP 0203)"

      -- putStrLn $ showAsTable2DSimple "No of loggings per student / phase" ["No. of Loggings"] showPrettyKHL2R show logFreqPerStudentPerPhase
      let researchdir = outputpath ++ "/" ++ stringToFilePath name ++ "/"
      createDirectoryIfMissing True researchdir

      -- let x = renderLateX $ showAsMarkUpTable1D logFreqPerStudentPerPhase
      
      writeToFile (researchdir ++ stringToFilePath name ++ ".tex")
         $ renderLateX $ showAsMarkUpTable1D logFreqPerStudentPerPhase
      writeToFile (researchdir ++ stringToFilePath name ++ ".html")
         $ renderHtml $ showAsMarkUpTable1D logFreqPerStudentPerPhase
      putStrLn $ renderLateX $ showAsMarkUpTable1D logFreqPerStudentPerPhase

main = thesisACExample logs (emptykey::KeyHistoryList) outputpath

