module Thesis.Research.ThesisACExample2 where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.Presentation

import Thesis.Helium.Logging
import Thesis.Helium.LogfileParser(parseLogfile)
import Thesis.Metrics(m1_1_1, phaseAnalysis)
import Thesis.Research.Common -- (groupPerPhase, groupPerWeek, plotToFigure)

import qualified Thesis.Grouping.Logging

import Auxilary(fmapTupleSnd)
-- import Data.Maybe
-- import Control.Monad

{-

   Example analyses used in impl chapter, also described in 02-CombinatorExample2.lhs as part of thesis

-}

default_logfile    = "datasets/demo/demo.log"
default_outputpath = "analysis_results/demo"

-- main = runNewResearchFromLogFile phaseResearch startkey default_logfile default_outputpath HtmlFormat


-- Simple / minimalistic example

-- first way of calculating the number of phases per week
loggingsPerPhasePerWeek :: AnalysisFK KeyHistory [Logging] Int
loggingsPerPhasePerWeek =
        countNumberOfLoggings
    <.> groupPerPhase
    <.> groupPerWeek


-- snd way of calculating the number of phases per week
-- numberOfLoggingsPerPhase :: (DescriptiveKey key) => AnalysisFK key [Logging] (Phase,Int)
-- numberOfLoggingsPerPhase = 
--     (basicAnalysis "phase" (phase . head) <&> calculateSize) <.> groupPerPhase

-- from Thesis.Grouping.Logging
-- groupPerPhase            :: [Logging] -> [(Phase, [Logging])]
-- groupPerPhase            =  enTupleGroup phase          . groupAllByOrd phaseOrd

-- poor descr for it
loggingsPerPhasePerWeek' :: AnalysisFK KeyHistory [Logging] [(Phase, Int)]
loggingsPerPhasePerWeek' =
        basicAnalysis "number of" (map $ fmapTupleSnd length)
    <.> basicAnalysis "phases" Thesis.Grouping.Logging.groupPerPhase
    <.> groupPerWeek

-- using Ord instance of data type
loggingsPerOrdPhase :: AnalysisFK KeyHistory [Logging] [(Phase,Int)]
loggingsPerOrdPhase = ordValueCount <.> mapAnalysis phaseAnalysis

relLoggingsPerOrdPhase :: AnalysisFK KeyHistory [Logging] [(Phase, Float)]
relLoggingsPerOrdPhase = relativeValueCount <.> ordValueCount <.> mapAnalysis phaseAnalysis


presentLoggingPerPhasePerWeek :: FilePath -> FilePath -> IO ()
presentLoggingPerPhasePerWeek logfile outputfp =
   do
      -- parse the logfile
      loggings <- parseLogfile logfile
      
      -- execute analysis
      let analysisResult = runAnalysis loggings loggingsPerPhasePerWeek
          analysisResult' = runAnalysis loggings loggingsPerPhasePerWeek'
          simpleAnalysisResult = runAnalysis loggings $ countNumberOfLoggings
          simpleAnalysisResult' = runAnalysis loggings $ 
                                          countNumberOfLoggings <.> groupPerPhase
          quantativeResult = runAnalysis loggings $ 
                  mapAnalysis (basicAnalysis "LOC" (length . lines . m1_1_1)) <.> groupPerWeek

          loggingsPerOrdPhaseResult = runAnalysis loggings loggingsPerOrdPhase

      -- present analysis results

      -- conflicting names for barPlot, barSChart
      -- barplot first implementation, compiletion on a low level
      -- barPlot <- renderBarPlot1D show outputfp analysisResult
      -- bar chart, thesis style, completion and ordering in presentation combinators
      barChart <- renderBarChart outputfp analysisResult

      -- bar chart, thesis style, completion and ordering in presentation combinators
      barChartSectionPresComb <- renderBarChart outputfp $
               countNumberOfLoggings <.> groupPerPhase <$> kickStartAnalysis loggings startkey
      
      -- bar chart, thesis style, user calc 2nd dim, completion and ordering in presentation combinators
      stackedBarChart <- renderStackedBarChart outputfp analysisResult'
      -- bar chart, thesis style, dynamic presentation rendering, completion and ordering in presentation combinators
      stackedBarChart' <- renderBarChartDynamic outputfp analysisResult
      -- bar chart, example 2, thesis style, dynamic presentation rendering, completion and ordering in presentation combinators
      stackedBarChart'' <- renderBarChartDynamic outputfp simpleAnalysisResult
      -- bar chart, example 3, thesis style, dynamic presentation rendering, completion and ordering in presentation combinators
      stackedBarChart''' <- renderBarChartDynamic outputfp simpleAnalysisResult'

      -- plotbox plot
      boxPlot <- renderBoxPlot outputfp quantativeResult

      -- barChart, using ordValueAnalysis, a isolated, aggregated, tuple analysis
      -- ana included in ana execute 
      barChartOrd <- renderBarChart outputfp $ runAnalysis loggings $
                                 flattenAnalysis' <.> loggingsPerOrdPhase
      barChartOrdStacked <- renderStackedBarChart outputfp $ runAnalysis loggings $
                                 loggingsPerOrdPhase
      barChartOrdStackedPerStudentPerPhase <- renderStackedBarChart outputfp $ runAnalysis loggings $
                                 loggingsPerOrdPhase <.> groupPerStudent

      -- barChart, using ordValueAnalysis, with relativeOrdCalc
      barChartOrdRel <- renderBarChart outputfp $ runAnalysis loggings $
                                 flattenAnalysis' <.> relativeValueCount <.> loggingsPerOrdPhase
      barChartOrdStackedRel <- renderStackedBarChart outputfp $ runAnalysis loggings $
                                 relativeValueCount <.> loggingsPerOrdPhase

      barChartOrdStackedRelPerWeekPerPhase <- renderStackedBarChart outputfp $ runAnalysis loggings $
                                 relativeValueCount <.> loggingsPerOrdPhase <.> groupPerWeek

      -- split per student
      -- for direct application
      let input = kickStartAnalysis loggings startkey
      barChartSet_relativePhasesPerWeekSplitPerStudent <-
            mapM (renderBarChart outputfp . loggingsPerPhasePerWeek)
                 (take 5 $ splitAnalysis $ groupPerStudent <$> input)

      -- rendering rel, barplot fails, due to incorrect title rendering, use mapRendering

      -- write output to a Html file
      writeFile (outputfp ++ "/overview.html")
                (renderHtml $
                    section Nothing "Overview of graphs"
                       +++
                    showAsTable1D analysisResult
                       +++
                    showAsTable2D analysisResult'
                       +++
                    showAsTableDynamic analysisResult
                       +++
                    showAsTableDynamic simpleAnalysisResult
                       +++
                    showAsTableDynamic simpleAnalysisResult'
--                        +++
--                     barPlot  -- name clash with barChart
                       +++
                    plotToFigure barChart
                       +++
                    plotToFigure barChartSectionPresComb  -- for in thesis, section prescomb
                    
                       +++
                    plotToFigure stackedBarChart
                       +++
                    plotToFigure stackedBarChart'
                       +++
                    plotToFigure stackedBarChart''
                       +++
                    plotToFigure stackedBarChart'''
                       +++
                    section Nothing "Box plot"

                       +++
                    plotToFigure boxPlot
                       
                       +++
                    section Nothing "Ord counted frequencies"
                       
                       +++
                    plotToFigure barChartOrd
                       +++
                    strong << "Note: malformed title: wrong order, twice \"of\" (in ordCounts only) and \"per Thesis.Helium.Logging.Phase per Thesis.Helium.Logging.Phase\""

                       +++
                    plotToFigure barChartOrdStacked 

                       +++
                    plotToFigure barChartOrdStackedPerStudentPerPhase
                    
                       +++
                    plotToFigure barChartOrdRel
--                        +++
--                     strong << "Note: malformed title: wrong ordering on description elems"

                       +++
                    plotToFigure barChartOrdStackedRel

                       +++
                    plotToFigure barChartOrdStackedRelPerWeekPerPhase

                       +++
                    (section Nothing "Split Per student")
                       +++
                       concatHtml
                    (map (\plot -> plotToFigure plot) barChartSet_relativePhasesPerWeekSplitPerStudent)
                    )
   
   -- optional other way to render plots
   -- writePlotFiles (researchdir++"/") (stringToFilePath name) PNGFormat $
   --               barPlot name $ flattenAnalysis <.> ordValueCount <$> anaResult
--   renderPNGPlot  (researchdir++"/") (stringToFilePath name) $ 
--                  barPlot name $ flattenAnalysis <.> ordValueCount <$> anaResult


main = presentLoggingPerPhasePerWeek default_logfile default_outputpath

