module Thesis.Research.TypingTillCodeResearch where

-- next to should be combined, in single default AC import
import Statistics.DescriptiveAnalysis
import Statistics.SummaryMeasures
import Statistics.Presentation
import Statistics.Completion

import Thesis.Metrics
import Thesis.Research.Common
import Thesis.Helium.Logging
import Thesis.Grouping.Logging (sameSourceFile, groupPerUsername_)

import Grouping
import Data.Typeable

import Auxilary

import Data.Maybe
import Data.List
import System.Time

import Text.Regex

import Thesis.Research.HintRateResearch hiding (main, default_logfile, default_outputpath)

{-

Wat mij betreft pas je 1 en 2 alleen toe op 04/05, misschien ter  
controle (wat betreft de uitkomsten) ook op 03/04.
1 Bereken per student, per week, de (gemiddelde tijd, gemiddelde  
aantal compiles) per sessie, benodigd om van een T
    compilatie te komen tot een C compilatie.
    [ C P T T T P T C L  L P P T C ] wordt [T T T P T C], [ T C ]

Session = is op tijd en filenaam
generated for student 29, 71 and 86 from the fp0203 data set
-}

default_logfile    = "datasets/demo/demo.log"
default_outputpath = "analysis_results/demo"

mainOverall = runNewResearchFromLogFile' typingTillCodeResearchOverall startkey default_logfile default_outputpath HtmlFormat
mainPerStudent = runNewResearchFromLogFile' typingTillCodeResearchPerStudent startkey default_logfile default_outputpath HtmlFormat

researchname   = "1. Typing Research"
timecoherence  = 10 -- min
minlogsize     = 10

calculateTCSequencesAnalysis  ::  AnalysisFK KeyHistory [Logging] [[Logging]]
calculateTCSequencesAnalysis  =
   basicAnalysis "Group of loggings from TtoC" calculateTCSequences

-- Grouping algorithm:
-- group loggings between typing till codegen, in between loggings are dropped
-- [ C P T T T P T C L  L P P T C ] becomes [T T T P T C], [ T C ]a

-- to check:
-- groupByOrdinalAttribute id Typing CodeGen [Lexical, Parsing, Typing, Typing, Parsing, Typing, CodeGen, Static,Typing, Typing, CodeGen, Parsing, Lexical, Static, Typing, Parsing, CodeGen]

calculateTCSequences  ::  [Logging] -> [[Logging]]
calculateTCSequences  =   groupingBetweenPhases Typing CodeGen

groupingBetweenPhases :: Phase -> Phase -> [Logging] -> [[Logging]]
groupingBetweenPhases = groupByOrdinalAttribute phase

groupByOrdinalAttribute :: (Ord b) => (a -> b) -> b -> b -> [a] -> [[a]]
groupByOrdinalAttribute attr from till xs =
      filter ((== till) . attr . last) $ filter (not . null) $ foldl (select) [[]] xs
   where
      -- select :: [[c]] -> c -> [[c]], strictly only works for T to C
                     -- init grouping
      select env x | (null $ last env) && (attr x == from) = init env ++ [[x]]
                     -- extend previous group
                   | (not . null $ last env) && (attr (last . last $ env) /= till) = init env ++ [(last env ++ [x])]
                     -- start new group, if x has from value
                   | (not . null $ last env) && (attr (last . last $ env) == till) =
                              if (attr x == from) then env ++ [[x]]  -- add new group
                                                  else env ++ [[ ]]  -- add empty group
                   | otherwise = env


innerGrpSize :: AnalysisFK KeyHistory [[a]] [Int]
innerGrpSize = basicAnalysis "number of loggings" (map length)
-- mapAnalysis numberOfLoggings

averageTtoCLength :: AnalysisFK KeyHistory [[a]] (OptData Float)
averageTtoCLength = averageAnalysis <.> innerGrpSize

--  return (diffClockTimes (toClockTime $ m1_1_8 snd) (toClockTime $ m1_1_8 fst))

-- "time between T to C"
timeTCSequences  ::  AnalysisFK KeyHistory [Logging] TimeDiff
timeTCSequences  =   basicAnalysis "" (uncurry logTimeDiff . headlast)

headlast :: [a] -> (a,a)
headlast list = (head list, last list)

timeDiffToSecAnalysis  ::  AnalysisFK KeyHistory TimeDiff Integer
timeDiffToSecAnalysis =
   basicAnalysis "" timeDiffToSec

timeDiffToMinAnalysis  ::  AnalysisFK KeyHistory TimeDiff Float
timeDiffToMinAnalysis =
   basicAnalysis "" timeDiffToMin

avgNoOfMinFromTtoC :: AnalysisFK KeyHistory [TimeDiff] (OptData Float)
avgNoOfMinFromTtoC = basicAnalysis "average no of min." (mean . map (timeDiffToMin))

avgNoOfSecFromTtoC :: AnalysisFK KeyHistory [TimeDiff] (OptData Float)
avgNoOfSecFromTtoC = basicAnalysis "average no of sec." (mean . map (timeDiffToSec))

groupOverview :: AnalysisFK KeyHistory [[Logging]] [[((Phase, CalendarTime), FilePath)]]
groupOverview = basicAnalysis "(phase, logtime/date, Filename)"  (map (map phaseAndCalTime))

phaseAndCalTime :: Logging -> ((Phase, CalendarTime), FilePath)
phaseAndCalTime log = ((phase log, logDate log), sourceFile $ logPath log)

filterOnStudent :: Username -> AnalysisFK KeyHistory [Logging] [Logging]
filterOnStudent student = basicAnalysis (" of " ++ show student) (filter ((student==) . username))

-- example to analyze per student, thesis purpose only
analysisPerStudent    :: AnalysisFK KeyHistory [Logging] a -> [(KeyHistory, [Logging])] -> [[(KeyHistory, a)]]
analysisPerStudent ana = map ana . splitAnalysis . groupPerStudent

numOfCompilesPerStudent :: Int -> [(KeyHistory, [Logging])] -> [[(KeyHistory, [Int])]]
numOfCompilesPerStudent minsize = analysisPerStudent
   (    mapAnalysis countNumberOfLoggings
   <.>  concatMapAnalysis calculateTCSequencesAnalysis
   <.>  calcFilenameCoherentSequences
   <.>  selectGrpAnalysis'_ ((> minsize) . length)
   <.>  groupPerWeek)

-- abstract coherence function
groupOnCoherence :: String -> (Logging -> Logging -> Bool) -> AnalysisFK KeyHistory [Logging] [[Logging]]
groupOnCoherence descr coherence =
   basicAnalysis  ("coherence grouping on: " ++ descr)
                  (groupByCoherence coherence)

calcFilenameCoherentSequences :: AnalysisFK KeyHistory [Logging] [[Logging]]
calcFilenameCoherentSequences =
   basicAnalysis  "coherence grouping on: filename"
                  (groupByCoherence sameSourceFile)

logTimeDiff :: Logging -> Logging -> TimeDiff
logTimeDiff l1 l2 = diffCalendarTimes (logDate l2) (logDate l1)

-- groupPerCoherenceTimeAndFilename = calcTimeAndFilenameCoherentSequences
-- groupPerCoherenceFilename = calcFilenameCoherentSequences

calcTimeAndFilenameCoherentSequences :: Int -> AnalysisFK KeyHistory [Logging] [[Logging]]
calcTimeAndFilenameCoherentSequences min =
   basicAnalysis  ("coherence grouping on: filename and time range (" ++ show min ++ ")")
                  (groupByCoherence
                     (\l1 l2 -> sameSourceFile l1 l2
                                    &&
                                       logTimeDiff l1 l2 < minutesOfTD min
                     ))

basicConcat = basicAnalysis "" concat

-- sessions
sessionsFilenameCoh_LogsFromTtoC :: AnalysisFK KeyHistory [Logging] [[Logging]]
sessionsFilenameCoh_LogsFromTtoC =
         concatMapAnalysis calculateTCSequencesAnalysis
   <.>   calcFilenameCoherentSequences

sessionsFilenameAndTimeCoh_LogsFromTtoC :: Int -> AnalysisFK KeyHistory [Logging] [[Logging]]
sessionsFilenameAndTimeCoh_LogsFromTtoC min =
         concatMapAnalysis calculateTCSequencesAnalysis
   <.>   calcTimeAndFilenameCoherentSequences min



overallAnalysisPerStudent ::
   AnalysisFK KeyHistory [Logging] [a] -> Int ->
   AnalysisFK KeyHistory [a] (OptData b) -> AnalysisFK KeyHistory [Logging] [OptData b]
overallAnalysisPerStudent ana minsize aggregateTDAnaPerStudent  =
               aggregateAnalysis "" id
         <.>   aggregateTDAnaPerStudent
         <.>   ana
         <.>   selectGrpAnalysis'_ ((> minsize) . length)
         <.>   groupPerStudent

minTtoCsize = 0

overallAvgNumOfCompilesTtoCPerStudent :: AnalysisFK KeyHistory [Logging] [OptData Float]
overallAvgNumOfCompilesTtoCPerStudent =
   overallAnalysisPerStudent
      (    mapAnalysis countNumberOfLoggings
      <.>  selectGrpAnalysis'_ ((> minTtoCsize) . length)
      <.>  sessionsFilenameCoh_LogsFromTtoC)
      minlogsize
      averageAnalysis

overallMedianNumOfCompilesTtoCPerStudent =
   overallAnalysisPerStudent
      (    mapAnalysis countNumberOfLoggings
      <.>  selectGrpAnalysis'_ ((> minTtoCsize) . length)
      <.>  sessionsFilenameCoh_LogsFromTtoC)
      minlogsize
      medianAnalysis

-- overallAverageTimeTtoCPerStudent :: AnalysisFK KeyHistory [Logging] [OptData Float]
overallAvgTimeTtoCPerStudent =
   overallAnalysisPerStudent
      (   mapAnalysis timeDiffToSecAnalysis
      <.> mapAnalysis timeTCSequences
      <.> selectGrpAnalysis'_ ((> minTtoCsize) . length)
      <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timecoherence)
      minlogsize
      averageAnalysis

overallMedianTimeTtoCPerStudent =
   overallAnalysisPerStudent
      (   mapAnalysis timeDiffToSecAnalysis
      <.> mapAnalysis timeTCSequences
      <.> selectGrpAnalysis'_ ((> minTtoCsize) . length)
      <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timecoherence)
      minlogsize
      medianAnalysis

----
-- Type hint effectiveness Analysis
----

-- hintEffectivenessAnalysis :: Int -> AnalysisFK KeyHistory [Logging] [(Int, [Integer])]
-- hintEffectivenessAnalysis min =
--                   categoriesValuesAna
--            <.>    mapAnalysis (percentageOfTypeHintsAnalysis' <&> (timeDiffToSecAnalysis <.> timeTCSequences))
--            <.>    sessionsFilenameAndTimeCoh_LogsFromTtoC min

-- hint effectiveness, expressed in time to compile
hintEffectivenessTimeAnalysis :: Int -> AnalysisFK KeyHistory [Logging] [(Double, Integer)]
hintEffectivenessTimeAnalysis min =
                mapAnalysis (percentageOfTypeHintsAnalysis' <&> (timeDiffToSecAnalysis <.> timeTCSequences))
         <.>    concatMapAnalysis calculateTCSequencesAnalysis
         <.>    calcTimeAndFilenameCoherentSequences min

-- (for thesis) previous 2 lines, were:
--           <.>  sessionsFilenameAndTimeCoh_LogsFromTtoC min

categorizeValuesAnalysis  ::  AnalysisFK KeyHistory [(Double, b)] [(Int, [b])]
categorizeValuesAnalysis  =   basicAnalysis "" categorizeValues

categorizeValues  ::  [(Double, b)] -> [(Int, [b])]
categorizeValues =      complementPerc
                    .   groupPerCategory 
                    .   map (\(perc, time) -> (calcCategory' perc, time))
   where
      complementPerc = complementGen (take 10 $ iterate (+10) 5) []
      -- group per 10 pct points, and return centerpoint (5*x)
      calcCategory'  100  =  (div (fromEnum  99) 10) * 10 + 5
      calcCategory'  p    =  (div (fromEnum   p) 10) * 10 + 5

groupPerCategory :: Eq a => [(a,b)] -> [(a,[b])]
groupPerCategory = map (\list -> (fst $ head list, map snd list)) . groupAllUnder fst

-- groupPerCategory' :: Eq a => [(a,[b])] -> [(a,[b])]
-- groupPerCategory' = map (\list -> (fst $ head list, concatMap snd list)) . groupAllUnder fst


calculateHintEffectivenessInTime = calculateHintEffectiveness
 
calculateHintEffectiveness :: Int -> Int -> AnalysisFK KeyHistory [Logging] [Integer]
calculateHintEffectiveness min minsize =
               flattenAnalysis'
         <.>   categorizeValuesAnalysis
         <.>   aggregateAnalysis "hint effectiveness in time" concat
         <.>   hintEffectivenessTimeAnalysis min
         <.>   selectGrpAnalysis'_ ((> minsize) . length)
         <.>   groupPerStudent
         <.>   groupPerWeek

-- hint effectiveness, expressed in number of compiles
hintEffectivenessCompilesAnalysis :: AnalysisFK KeyHistory [Logging] [(Double, Int)]
hintEffectivenessCompilesAnalysis =
                mapAnalysis (percentageOfTypeHintsAnalysis' <&> countNumberOfLoggings)
           <.>  concatMapAnalysis calculateTCSequencesAnalysis
           <.>  calcFilenameCoherentSequences

-- (for thesis) previous 2 lines, were:
--           <.>  sessionsFilenameAndTimeCoh_LogsFromTtoC min

calculateHintEffectivenessInCompiles :: Int -> AnalysisFK KeyHistory [Logging] [Int]
calculateHintEffectivenessInCompiles minsize =
               flattenAnalysis'
         <.>   categorizeValuesAnalysis
         <.>   aggregateAnalysis "hint effectiveness in number of compiles" concat
         <.>   hintEffectivenessCompilesAnalysis
         <.>   selectGrpAnalysis'_ ((> minsize) . length)
         <.>   groupPerStudent
         <.>   groupPerWeek

typingTillCodeResearchOverall :: OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO ()
typingTillCodeResearchOverall format outputpath input = do
   let
      name = researchname
      researchdir = joinDirAndFileName outputpath (stringToFilePath name) ++ [pathSeparator]
      outputfile  = joinDirFileExt     researchdir (stringToFilePath name) $ showFormatExt format
      timecoh = timecoherence

   createDirectoryIfMissing True researchdir

   -- do research overall, but silently grouped per student
--    tToCOverallForAllStudents <-
--       typingTillCodeResearch "" format researchdir (groupPerStudentSilent <$> input)
{-
   writeFile outputfile
      $ renderMarkUpDoc format $ document noHtml $
         (section Nothing << name )
         +++
         tToCOverallForAllStudents-}

   let ana2 =
             mapAnalysis timeDiffToSecAnalysis
         <.> mapAnalysis timeTCSequences
         <.> basicConcat
         <.> mapAnalysis calculateTCSequencesAnalysis
         <.> basicConcat
         <.> mapAnalysis (calcTimeAndFilenameCoherentSequences timecoh)
         <.> basicAnalysis "" groupPerUsername_
         <.> groupPerWeek

   timeToCOverallForAllStudents <-
      renderBoxPlot researchdir (ana2 <$> input)

   numCompilesToCOverallForAllStudents <-
      renderBoxPlot researchdir (
                mapAnalysis countNumberOfLoggings
            <.> concatMapAnalysis calculateTCSequencesAnalysis
            <.> concatMapAnalysis calcFilenameCoherentSequences
            <.> basicAnalysis "" groupPerUsername_
            <.> groupPerWeek
            <$> input)

   numAvgCompilesToCOverallPerStudents <-
      renderBoxPlot researchdir (overallAvgNumOfCompilesTtoCPerStudent <..> groupPerWeek <$> input)

   numMedianCompilesToCOverallPerStudents <-
      renderBoxPlot researchdir (overallMedianNumOfCompilesTtoCPerStudent <..> groupPerWeek <$> input)

   avgTimeToCOverallPerStudents <-
      renderBoxPlot researchdir (overallAvgTimeTtoCPerStudent <..> groupPerWeek <$> input)

   medianTimeToCOverallPerStudents <-
      renderBoxPlot researchdir (overallMedianTimeTtoCPerStudent <..> groupPerWeek <$> input)

   -- hint effectiveness analysis
   timeHintTimeT2CCorrelation <-
      renderBoxPlot researchdir ((calculateHintEffectivenessInTime timecoh minlogsize) <$> input)

   compilesHintTimeT2CCorrelation <-
      renderBoxPlot researchdir (calculateHintEffectivenessInCompiles minlogsize <$> input)

   writeFile outputfile
      $ renderMarkUpDoc format $ document noHtml $
         (section Nothing << (name ++ " time-coherence: " ++ show timecoherence ++ " min logs per week: " ++ show minlogsize))
         +++
         -- overall, not aggregated per student
         plotToFigure' researchdir timeToCOverallForAllStudents
         +++
         plotToFigure' researchdir numCompilesToCOverallForAllStudents
         +++
         -- overall, aggregated per student
         plotToFigure' researchdir numAvgCompilesToCOverallPerStudents
         +++
         plotToFigure' researchdir numMedianCompilesToCOverallPerStudents
         +++
         plotToFigure' researchdir avgTimeToCOverallPerStudents
         +++
         plotToFigure' researchdir medianTimeToCOverallPerStudents
         +++
         (showAsTableDynamic $
                countNumberOfLoggings
            <.> groupPerWeek
            <.> groupPerStudent
            <$> input)
--          +++  -- table are very slow to render
--          (showAsTable1D $
--                mapAnalysis (basicAnalysis "" (map (\l-> (username l, phase l, logDate l, logPath l))))
--            <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timecoh
--            <.> groupPerWeek
--            <.> groupPerStudent
--            <$> input)
--          +++
--          (showAsTable1D $
--                mapAnalysis (basicAnalysis "" (map (\l-> (username l, phase l, logDate l, logPath l))))
--            <.> concatMapAnalysis calculateTCSequencesAnalysis
--            <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timecoh
--            <.> groupPerWeek
--            <.> groupPerStudent
--            <$> input)
         +++
         (section Nothing << "Hint Effectiveness - correlation time to solve vs perc of type hints" )
         +++
         plotToFigure' researchdir timeHintTimeT2CCorrelation
         +++
         plotToFigure' researchdir compilesHintTimeT2CCorrelation
         
   return ()

typingTillCodeResearchPerStudent :: OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO ()
typingTillCodeResearchPerStudent format outputpath input = do
   let
      name = researchname ++ "perstudent"
      researchdir = joinDirAndFileName outputpath (stringToFilePath name) ++ [pathSeparator]
      outputfile = joinDirFileExt researchdir (stringToFilePath name) $ showFormatExt format

   -- do research per student
   tToCPerStudent <-
      researchPerDataSet typingTillCodeResearch "student" format researchdir ({-take 10 $ -}groupPerStudent <$> input)

   writeFile outputfile
      $ renderMarkUpDoc format $ document noHtml $
         (section Nothing << name )
         +++
         concatMarkUp
            tToCPerStudent

typingTillCodeResearch :: String -> OutputFormat -> FilePath -> [(KeyHistory, [Logging])] -> IO MarkUpDoc
typingTillCodeResearch student format outputpath input = do
   let
      name = student
      researchdir = joinDirAndFileName outputpath (stringToFilePath name) ++ [pathSeparator]
      outputfile = joinDirFileExt researchdir (stringToFilePath name) $ showFormatExt format
      timesessionlen = timecoherence -- min

   createDirectoryIfMissing True researchdir
 
   barChartAvgNoLogsPerGroupTtoC <-
      renderBarChart researchdir $
               averageTtoCLength
         <.>   sessionsFilenameCoh_LogsFromTtoC
         <.>   selectGrpAnalysis'_ ((> minlogsize) . length)
         <.>   groupPerWeek
         <$>   input

   barChartAvgNoOfMinPerGroupTtoC <-
      renderBarChart researchdir $
               avgNoOfSecFromTtoC
         <.>   mapAnalysis timeTCSequences
         <.>   sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
         <.>   selectGrpAnalysis'_ ((> minlogsize) . length)
         <.>   groupPerWeek
         <$>   input

   boxPlotNoLogsPerSessionTtoC <- renderBoxPlot  researchdir $
               innerGrpSize
           <.> sessionsFilenameCoh_LogsFromTtoC
           <.> selectGrpAnalysis'_ ((> minlogsize) . length)
           <.> groupPerWeek
           <$> input

   boxPlotNoSecPerSessionTtoC <- renderBoxPlot  researchdir $
               basicAnalysis "" (map timeDiffToSec)
           <.> mapAnalysis timeTCSequences
           <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
           <.> selectGrpAnalysis'_ ((> minlogsize) . length)
           <.> groupPerWeek
           <$> input

   -- hint effectiveness, unfortunate not enough data per student,
   -- throws error is no data is available at all:
   -- pl proc rangebar: error 601: datafield not specified or out of range
   hintEffectiveness <- renderBoxPlot researchdir $
           flattenAnalysis' $
               (basicAnalysis
                  ""
                  (map (\list -> ((calcCategory 10) $ fst $ head list, map snd list))
                  . groupAllUnder (calcCategory 10 . fst))
               )
           <.> mapAnalysis (percentageOfTypeHintsAnalysis' <&> ((basicAnalysis "" timeDiffToSec) . timeTCSequences))  -- [(Double, Int)]
           <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
           <$> input

   -- to have a min. number of loggings, see commented lines
   hintEffectiveness' <- renderBoxPlot researchdir $
                              flattenAnalysis'
                          <.> categorizeValuesAnalysis
                          -- <.> aggregateAnalysis "" concat
                          <.> hintEffectivenessTimeAnalysis timesessionlen -- minlogsize
                          -- <.> selectGrpAnalysis ((> minlogsize) . length)
                          -- <.> groupPerWeek
                          <$> input

   -- return result data
   return $
         (subsection Nothing << (name ++ " time-coherence: " ++ show timesessionlen ++ " min logs per week: " ++ show minlogsize))
         +++
         (showAsTable1D $
                countNumberOfLoggings
            <.> groupPerWeek
            <$> input)
         +++
         (showAsTableDynamic $
               countNumberOfLoggings <.> groupPerPhase <.> groupPerWeek <$> input)
         +++
         subsection Nothing << "Number of loggings from T to C"
         +++
         plotToFigure' outputpath boxPlotNoLogsPerSessionTtoC
         +++
         (showAsTableDynamic $
               innerGrpSize
           <.> sessionsFilenameCoh_LogsFromTtoC
           <.> groupPerWeek
           <$> input)
         +++
         plotToFigure' outputpath barChartAvgNoLogsPerGroupTtoC
         +++
         (showAsMarkUpTable1D $
               averageTtoCLength
           <.> sessionsFilenameCoh_LogsFromTtoC
           <.> groupPerWeek
           <$> input)
         +++
         subsection Nothing << "Time from T to C"
         +++
         (showAsTableDynamic $
               basicAnalysis "" (map timeDiffToSec)
           <.> mapAnalysis timeTCSequences
           <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
           <.> groupPerWeek
           <$> input)
         +++
         plotToFigure' outputpath boxPlotNoSecPerSessionTtoC
         +++
         plotToFigure' outputpath barChartAvgNoOfMinPerGroupTtoC
         +++
         (showAsMarkUpTable1D $
               avgNoOfSecFromTtoC
           <.> mapAnalysis timeTCSequences
           <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
           <.> groupPerWeek
           <$> input)
         +++
         (showAsMarkUpTable1D $
               avgNoOfMinFromTtoC
           <.> mapAnalysis timeTCSequences
           <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
           <.> groupPerWeek
           <$> input)

-- checks:
--          +++
--          (showAsMarkUpTable1D $ groupOverview <.> sessionsFilenameCoh_LogsFromTtoC <.> groupPerWeek <$> input)
--          +++
--          (showAsMarkUpTable1D $
--                groupOverview
--            <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
--            <.> groupPerWeek
--            <$> input)
          
          +++
          (subsection Nothing << "Hint effectiveness")
          +++
          plotToFigure' outputpath hintEffectiveness
          +++
          plotToFigure' outputpath hintEffectiveness'
          +++ -- tests
          (showAsTable1D $
               (basicAnalysis
                  ""
                  (map (\list -> ((calcCategory 10) $ fst $ head list, list))
                  . groupAllUnder (calcCategory 10 . fst))
               )
           <.> mapAnalysis (percentageOfTypeHintsAnalysis' <&> ((basicAnalysis "" timeDiffToSec) . timeTCSequences))  -- [(Double, Int)]
           <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
--           <.> groupPerWeek
           <$> input)

                     +++
          (showAsTable1D $
               (basicAnalysis
                  ""
                  ({-map (\list -> ((10*) $ (calcCategory 10) $ fst $ head list), length list) . -}groupUnder (calcCategory 10 . fst))
               )
           <.> mapAnalysis (percentageOfTypeHintsAnalysis' <&> ((basicAnalysis "" timeDiffToSec) . timeTCSequences))
           <.> sessionsFilenameAndTimeCoh_LogsFromTtoC timesessionlen
           <.> groupPerWeek
           <$> input)

-- To divide values into a categorial system
-- calcCategory :: (Real a1, Real a, Integral b) => a -> a1 -> b
calcCategory :: (Real a1, Real a) => a -> a1 -> Int
calcCategory size = (round . (/(realToFrac size))) . realToFrac

--calcCategory :: (Real a1, Real a) => a -> a1 -> Int
calcCategory' size value = div value size