module Thesis.Select where

import Thesis.Settings
import Thesis.Helium.Logging
import Text.Regex

import Auxilary

import Data.List
import Data.Maybe


-- selection combinators

select = filter

andSelect   :: (a -> Bool) -> (a -> Bool) -> a -> Bool
andSelect s1 s2 = (\log  -> s1 log || s2 log)

orSelect    :: (a -> Bool) -> (a -> Bool) -> a -> Bool
orSelect s1 s2 log = (s1 log) || (s2 log)

-- logging specific combinators

-- should use logging metric m1_1_?
isPhase :: Phase -> Logging -> Bool
isPhase ph log = phase log == ph

isUsername :: String -> Logging -> Bool
isUsername un log = usInLog == un
   where (Username usInLog) = username log 

isVersion :: HeliumVersion -> Logging -> Bool
isVersion version log = heliumVersion log == version
--isVersion version (Log _ _ v _ _) = (version == v)

isTrue :: a -> Bool
isTrue _ = True

-- isCompiledBetween :: Ord a => a -> a -> Logging -> Bool
isCompiledBetween :: String -> String -> Logging -> Bool
isCompiledBetween date1 date2 log =
   isCompiledBefore date2 log && isCompiledAfter date1 log

-- isCompiledBefore :: Ord a => a -> Logging -> Bool
isCompiledBefore bnd log = ({-stringToCalTime $ -}logDate log) < (stringToCalTime bnd)

-- isCompiledAfter :: Ord a => a -> Logging -> Bool
isCompiledAfter bnd log = ({-stringToCalTime $-} logDate log) > (stringToCalTime bnd)

like :: String -> String -> Bool
like str1 str2 = str1 `elem` (inits str2)

likeRE :: Regex -> String -> Bool
likeRE re = isJust . matchRegexAll re

-- finding a following logging in time in a set of loggings
nextLogging :: Logging -> DataSet -> Maybe Logging
nextLogging l logs =
   let nextLogs = filter  (> l) logs
   in
      case (length nextLogs >= 1) of
         True -> Just (minimum nextLogs)
         False -> Nothing

-- finding a previous logging in time in a set of loggings
prevLogging :: Logging -> DataSet -> Maybe Logging
prevLogging l logs =
   let prevLogs = filter  (< l) logs
   in
      case (length prevLogs >= 1) of
         True -> Just (minimum prevLogs)
         False -> Nothing

allPhases :: [Phase]
allPhases = [Lexical, Parsing, ResolOp, Static {- [] -}, Typing, CodeGen, InternalErr]

--function for selection of loggings
{-
toselectDSPerPhase :: [DataSet -> DataSet]
toselectDSPerPhase = map (filter . isPhase) allPhases

selectDSPerPhase :: DataSet -> [DataSet]
selectDSPerPhase logs = map ($logs) toselectDSPerPhase
-}

selectDSPerPhase :: DataSet -> [DataSet]
selectDSPerPhase logs = map (($ logs) . filter . isPhase) $ allPhases

mapPerPhase :: (DataSet->a) -> DataSet -> [a]
mapPerPhase fun logs = map fun (selectDSPerPhase logs)

-- select all loggings ending in Lexical phase
selectLexPhaseLoggings :: DataSet -> DataSet
selectLexPhaseLoggings = filter (isPhase (Lexical))
