module Test where

import Auxilary (mapTuple)
import Statistics.Analysis
import Statistics.DataInfo
import Statistics.BasicTypes
import Statistics.Presentation
--import Statistics.MeasurementLevels

import Logging
import Thesis.Grouping.Logging
import Metrics
import Select

-- { - # OPTIONS_GHC -fallow-overlapping-instances -fallow-undecidable-instances -fallow-incoherent-instances # - }

data KeyKind =
           CompleteSelection String
         | UnsupportedAnaOperation String
         | SelectionOfLoggingsPerIdentity Identity
         | SelectionOfLoggingsPerPhase Phase
         | SelectionOfLoggingsPerWeek (Integer, Int)
         | SelectionOfLoggingsPerDay
         | SelectionOfLoggingsPerSession Session
         | SelectionOfLoggingsWithMin Int
         | CalculatedValueOverLoggingset String [Logging]  -- description, logs
   deriving (Show, Read, Eq)

newtype Key = Key [KeyKind] -- list of keys, descriptions of succesly applied analyses
   deriving (Show, Read, Eq)

data Result a =
     DataSingle Key [a]
   | DataResultSet [Result a]
   deriving (Show, Read, Eq)

type GroupSelection a b = ([a] -> [(b, [a])])
type AnaFunction    a b = (a -> b)

addKey :: KeyKind -> Key -> Key
addKey key (Key keys) = Key (keys ++ [key])

combKey :: Key -> Key -> Key
combKey (Key k1) (Key k2) = Key (k1 ++ k2)

---
--groupAna :: Statistics.Analysis (Key, DataResult a) (Key, DataResult a)

startAnalysis :: a -> [(Key, a)]
startAnalysis input = undefined --[(Key [CompleteSelection], input)]


-- Abstract selection function, could be implemented as well as fold
anaGroupPerSelection :: GroupSelection a b -> (b -> KeyKind) -> [(Key, [a])] -> [(Key, [a])]
anaGroupPerSelection groupfunction keykind [] = []
anaGroupPerSelection groupfunction keykind ((keys, elms) : xs) =
   let
      groups = groupfunction elms
   in
      map (\(grpElm, selection) -> (addKey (keykind grpElm) keys, selection)) groups
         ++ anaGroupPerSelection groupfunction keykind xs

-- (<|>) ::
-- filterana =

anaGroupPerIdentity :: [(Key, [Logging])] -> [(Key, [Logging])]
anaGroupPerIdentity = anaGroupPerSelection groupPerIdentity SelectionOfLoggingsPerIdentity
anaGroupPerPhase    = anaGroupPerSelection groupPerPhase    SelectionOfLoggingsPerPhase
anaGroupPerWeek     = anaGroupPerSelection groupPerWeek     SelectionOfLoggingsPerWeek
anaGroupPerSession  = anaGroupPerSelection groupPerSession  SelectionOfLoggingsPerSession

-- Test
-- test :: [(Key, [Logging])]
-- test = anaGroupPerPhase $ startAnalysis [log1, log2, log3, log2]
            

{- alternative impl
anaGroupPerSession' [] = []
anaGroupPerSession' ((keys, logs) : xs) =
   let
      groups = groupPerSession logs
   in
      map (\(session, selection) -> (addKey (SelectionOfLoggingsPerSession session) keys, selection)) groups
         ++ anaGroupPerSession' xs

-- analogue for anaGroupPerPhase
-}


-- a more abstract combinator/helper function
liftFun :: AnaFunction a b -> (b -> KeyKind) -> [(Key, [a])] -> [(Key, [b])]
liftFun anaFun keykind input =
      map (\(keys, selection) -> (addKey (keykind undefined) keys, map anaFun selection)) input
-- map in snd van tuple, ook een parameter maken van anafunction


--anaGroupPerPhaseAna :: Statistics.Analysis [(Key, [a])] [(Key, [a])]
anaGroupPerPhaseAna = Statistics.Analysis analysisID descr subj outc ana pres
   where
      analysisID = "Ana"
      descr = ""
      subj = ""
      outc = ""
      ana = anaGroupPerSelection groupPerPhase   SelectionOfLoggingsPerPhase
      pres = undefined {- addPresentationIO
         (toGNUPlotScatter descr (subj,outc))
         basicPresentation
         -}CompleteSelection

protocolProgChar progrchar {- or metric -}= undefined


------------------
-- Run Research, Research Combinators
------------------

data DataValue a = SingleData Key a
                 | DataSet    Key [(DataValue a)]
   deriving (Show, Read, Eq)

type DataValueAlgebra a b = (
     (Key -> a   -> b)
   , (Key -> [b] -> b)
   )

foldDVAlg :: DataValueAlgebra a b -> DataValue a -> b
foldDVAlg (singleD, setD) (SingleData key value) = singleD key value
foldDVAlg (singleD, setD) (DataSet key dsvalues) = setD key $ map (foldDVAlg (singleD, setD)) dsvalues

calcDepthDVs :: DataValue a -> Int
calcDepthDVs = foldDVAlg (\_ _ -> 0, \_ dsdpths -> maximum dsdpths + 1)

--flattenDV :: DataValue a -> [[(Key,a)]]
flattenDV = foldDVAlg
               (\key value -> [(key,value)],
                \pkey valueset -> map (\(ckey,value) -> (combKey pkey ckey,value)) $ concat valueset
               )

testData = DataSet (Key [CompleteSelection ""]) [SingleData (Key []) Lexical]

newtype Ana2 a b = Statistics.Analysis2 ([(Key,a)] -> [(Key, b)])
newtype Ana3 a b = Statistics.Analysis3 (DataValue a -> DataValue b)

type Metric2 b = Ana2 Logging b

-- (<.>) :: Ana2 a b -> Ana2 b c -> Ana2 a c
-- (<.>) = undefined



data ShowData = ShowAsTable String String String

(<$>) :: (ShowData -> Key) -> Ana2 b c -> Ana2 b c
(<$>) = undefined

grpAna :: ([a] -> [[a]]) -> Ana2 [a] [a]
grpAna grpingfun = undefined -- see up

concatAna :: ([[a]] -> [a]) -> Ana2 [a] [a]
concatAna = undefined

instance Functor (Ana2 z) where
   fmap f = (\(Analysis2 g) -> Statistics.Analysis2 (mapTuple (id,f) . g)) --(<.>)

-- fmap to create a new analysis (extended with a function)

analyseProgramCharacteristic :: (DataInfo a) => Metric a -> [EncapsulatedAnalysis]  -- [Analysis [Logging] (IO ())]
analyseProgramCharacteristic metric = undefined


-- tags for annotating the values, not usefull, unless you can espect smth about the a value, certain type classes, (Num, Ord) etc. GADT? mabye a solution
data NominalValue a = NominalValue a
   deriving (Show, Read, Eq)

data OrdinalValue a = OrdinalValue a
   deriving (Show, Read, Eq)

data IntervalValue a = IntervalValue a
   deriving (Show, Read, Eq)

data RatioValue a = RatioValue a
   deriving (Show, Read, Eq)

--- Simple [(key, value)] analyses

type Statistics.AnalysisNew a b = [(Maybe KeyNew, a)] -> [(Maybe KeyNew, b)]
type KeyNew = String

runAnalysisNew :: a -> Statistics.AnalysisNew a b -> [(Maybe KeyNew, b)]
runAnalysisNew input ana = ana [(Nothing, input)]

getAnaResult :: a -> Statistics.AnalysisNew a b -> [b]
getAnaResult input = map snd . runAnalysisNew input

-- helper functions
keyTransform :: {-Show a =>-} KeyNew -> Maybe KeyNew -> Maybe KeyNew
keyTransform newkey orikey =
   case orikey of
      Nothing -> Just $ newkey -- ++ " (over set: " ++ show subject ++ ")"
      Just key -> Just $ newkey ++ ";" ++ key

-- peek the result only if analysis is a singleton list
peekResult :: Statistics.AnalysisNew a b -> a -> b
peekResult ana input = snd $ head $ ana [(Nothing, input)]

getFun :: Statistics.AnalysisNew a b -> (a -> b)
getFun = peekResult

getKeyTransf :: Statistics.AnalysisNew a b -> Maybe KeyNew -> Maybe KeyNew
getKeyTransf ana = (\keyin -> fst $ head $ ana [(keyin, undefined)])

filterAna :: KeyNew -> (AnalysisNew a Bool) -> Statistics.AnalysisNew [a] [a]
filterAna descr selectana =
   mapTuple (keyTransform descr {-"filtered, only (multiple) "-}, filter (\value -> peekResult selectana value))

createAnalysis :: KeyNew -> (a->b) -> Statistics.AnalysisNew a b -- type AnaFunction    a b = (a -> b)
createAnalysis descr fun = mapTuple (keyTransform descr, fun)

applyAnalysis :: (a -> b) -> Statistics.AnalysisNew a b -> Statistics.AnalysisNew b c
applyAnalysis = undefined

applySetAnalysis :: Statistics.AnalysisNew a b -> Statistics.AnalysisNew [a] [b]  -- mapAnalysis (over each set of [log] in input, creatMappedAnalysis
applySetAnalysis ana = 
   let analysisfun = getFun ana
       keyfun = getKeyTransf ana
       newAna = (\(key,value) -> (keyTransform "applied over a set" $ keyfun key, map analysisfun value) )
   in 
      (\input -> map (newAna) input)

--mapOverInput :: ((Maybe Key, a) -> (Maybe Key, b)) -> [(Maybe Key, a)] -> [(Maybe Key, b)]
--mapOverInput = mapTuple  -- fmap of data DataSetNew a = Input [(Maybe KeyNew, a)] -- :k: (*->*)

-- $ like combinator
mapAna :: Statistics.AnalysisNew a b -> [(Key, a)] -> [(Key, b)]
mapAna ana =
   let analysisfun = getFun ana
       keyfun = getKeyTransf ana
       newAna = (\(key,value) -> (keyTransform "applied over a set" $ keyfun key, analysisfun value) )
   in (\input -> concat . (map (keyfun, analysisfun) input))

(<.>) :: Statistics.AnalysisNew b c -> Statistics.AnalysisNew a b -> Statistics.AnalysisNew a c
-- [(KeyNew, b)] -> [(KeyNew, c)] -> [(KeyNew, a)] -> [(KeyNew, b)] -> [(KeyNew, a)] -> [(KeyNew, c)]
ana2 <.> ana1 = (\input -> mapAna ana2 $  mapAna ana1 input)

--applyGroupingAnalysis descr fun = mapTuple (keyTransform descr, fun)

-- simple example Statistics.AnalysisNew
countLoggings :: Statistics.AnalysisNew [Logging] Int
countLoggings = createAnalysis "number of loggings" length

lOC :: Statistics.AnalysisNew Logging Int
lOC = createAnalysis "Lines of code" m1_2_4

allLOC :: Statistics.AnalysisNew [Logging] [Int]
allLOC = applySetAnalysis lOC

isLoggingFromUser:: Identity -> Statistics.AnalysisNew Logging Bool
isLoggingFromUser identity = createAnalysis ("log from " ++ show identity) (isIdentity identity)

loggingsFromUser :: Identity -> Statistics.AnalysisNew [Logging] [Logging]
loggingsFromUser identity = -- applyAnalysis ("Loggings from " ++ show identity) (filter (isIdentity identity) )
   filterAna ("Loggings from " ++ show identity) (isLoggingFromUser identity)

usersInLoggingSelection :: Statistics.Analysis [Logging] [Identity]
usersInLoggingSelection = undefined -- createAnalysis "All identities of Loggings" 

-- simple combining analysis
countLoggingsFromUser :: Identity -> Statistics.AnalysisNew [Logging] Int
countLoggingsFromUser identity =
   undefined --countLoggings <.> loggingsFromUser identity

-- Testers
logs = [log1, log2, log3, log4]

testSetAna = runAnalysisNew logs
testSingleLogAna = runAnalysisNew log1