module Thesis.WXGrid where

import Graphics.UI.WX
import Graphics.UI.WXCore hiding (Event)

--------------------------------------------------------------------------------
-- Grid Control to Wx Library?  (from samples/wx/Grid.hs)
--------------------------------------------------------------------------------

-- Creation of grid widget with WXCore (type of initialWindow :: forall a w. (Id -> Rect -> [Prop (Window w)] -> Style -> a) -> [Prop (Window w)] -> Style -> a
gridCtrl :: Window a -> [Prop (Grid ())] -> IO (Grid ())
gridCtrl parent props
  = feed2 props 0 $
    initialWindow $ \id rect -> \props flags ->
    do g <- gridCreate parent id rect flags
       gridCreateGrid g 0 0 0
       set g props
       return g

gridEvent :: Event (Grid a) (EventGrid -> IO ())
gridEvent
  = newEvent "gridEvent" gridGetOnGridEvent gridOnGridEvent


gridMoveNext :: Grid a -> IO ()
gridMoveNext gLoggings
  = do row <- gridGetGridCursorRow gLoggings
       col <- gridGetGridCursorCol gLoggings
       rowCount <- gridGetNumberRows gLoggings
       colCount <- gridGetNumberCols gLoggings
       let (r,c) = if (row+1 >= rowCount)
                   then if (col+1 >= colCount)
                        then (0,0)
                        else (0,col+1)
                   else (row+1,col)
       gridSetGridCursor gLoggings r c
       gridMakeCellVisible gLoggings r c
       return ()

-- Add list of (header and empty column) to a grid, columnname in the label (-1st row)
appendColumns :: Grid a -> [String] -> IO ()
appendColumns gLoggings []
  = return ()
appendColumns gLoggings labels
  = do n <- gridGetNumberCols gLoggings
       gridAppendCols gLoggings (length labels) True
       mapM_ (\(i,label) -> gridSetColLabelValue gLoggings i label) (zip [n..] labels)

-- Add empty rows to a grid, with a number in the label (-1st column)
appendRows :: Grid a -> [String] -> IO ()
appendRows gLoggings []
  = return ()
appendRows gLoggings labels
  = do n <- gridGetNumberRows gLoggings
       gridAppendRows gLoggings (length labels) True
       mapM_ (\(i,label) -> gridSetRowLabelValue gLoggings i label) (zip [n..] labels)

-- Put list of string is cells of appointed rows (in Grid.hs not under lib section)
setRow :: Grid a -> (Int, [String]) -> IO ()
setRow gLoggings (row,values)
  = mapM_ (\(col,value) -> gridSetCellValue gLoggings row col value) (zip [0..] values)

  
-- own grid functions
  
-- add a GridTable :: [[String]] to grid (automatically numbers the leftmost -1st column)
appendGridContent :: Grid a -> [[String]] -> IO ()
appendGridContent gLoggings input = do
   n <- gridGetNumberRows gLoggings
   -- append empty rows with number as label
   appendRows    gLoggings (map show [ n+1 .. n+(length input)])
   -- add content to grid
   mapM_ (setRow gLoggings) (zip [n..] input)
   -- resize widget to new content
   gridAutoSize gLoggings

-- add a GridTable :: [[String]] to grid (automatically numbers the leftmost -1st column)
clearGridContent :: Grid a -> IO ()
clearGridContent gLoggings = do
   n <- gridGetNumberRows gLoggings
   -- clean contents
   gridClearGrid gLoggings 
   -- delete row labels
   gridDeleteRows gLoggings 0 n True
   gridAutoSize gLoggings

-- add a GridTable :: [[String]] to grid (automatically numbers the leftmost -1st column)
resetGridContent :: Grid a -> [[String]] -> IO ()
resetGridContent gLoggings input = do
   clearGridContent gLoggings
   appendGridContent gLoggings input
   gridAutoSize gLoggings

   
   -- setup grid with standard style
setUpGrid :: Window a -> [Prop (Grid ())] -> [String] -> IO (Grid ())  -- () of a? of alleen voor Prop  -- props and style, zie gridCtrl
setUpGrid parent props columnnames = 
   do g <- gridCtrl parent props
      gridSetGridLineColour g (colorSystem Color3DFace)
      gridSetCellHighlightColour g black
      -- set columnnames
      appendColumns g columnnames  
      gridAutoSize g   -- necessary, and influence on other widgets??
      return g